//----------------------------------------------------------------------------
//  Project hypoplastic model
//  
//  Copyright  1996. All Rights Reserved.
//
//  SUBSYSTEM:    hypo Application
//  FILE:         QHypoApp.cpp
//  AUTHOR:       
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of QHypoApp (TApplication).
//  NOTE: for BC 5.0 or above
//----------------------------------------------------------------------------

#include "OWL_doanh.h"
#pragma hdrstop

#if !defined(OWL_BUTTONGA_H)
	#include <owl/buttonga.h>
#endif
#if !defined(OWL_STATUSBA_H)
	#include "owl\statusba.h"
#endif
#if !defined(OWL_DOCMANAG_H)
	#include <owl/docmanag.h>
#endif
#if !defined(OWL_FILEDOC_H)
	#include <owl/filedoc.h>
#endif

#if !defined(CLASSLIB_CMDLINE_H)
#include <classlib/cmdline.h>
#endif
#if !defined(CLASSLIB_FILENAME_H)
#include <classlib/filename.h>
#endif

#if !defined(H_HYPO_APPICATION_H)
	#include "QHypoApp.h"
#endif
#if !defined(HYPOPLASTIC_RC)
	#include "hypoapp.rc"
#endif

#if !defined(HWOLFFER_HYPO_DOC_H)
	#include "wolfferdoc.h"
#endif
#if !defined(HQAXE_H)
	#include "qaxes.h"
#endif
#if !defined(HQCURVES_H)
	#include "qcurves.h"
#endif
#if !defined(HGRAPH_H)
	#include "qgraph.h"
#endif
#if !defined(HQVIEWLIST_H)
	#include "qviewlist.h"
#endif
#if !defined(H_HYPO_PARAMETER_H)
	#include "qparameter.h"
#endif
#if !defined(H_HYPO_ABOUT_H)
	#include "hypoAbout.h"
#endif


//{{QHypoApp Implementation}}



//{{DOC_VIEW}}
DEFINE_DOC_TEMPLATE_CLASS(QHypoplasticDoc, QViewList, ViewListTemplate);
DEFINE_DOC_TEMPLATE_CLASS(QHypoplasticDoc, QGraph_q_eps, Graphe_q_epsTemplate);
DEFINE_DOC_TEMPLATE_CLASS(QHypoplasticDoc, QGraph_u_eps, Graphe_u_epsTemplate);
DEFINE_DOC_TEMPLATE_CLASS(QHypoplasticDoc, QGraph_epsv_eps, Graphe_epsv_epsTemplate);
DEFINE_DOC_TEMPLATE_CLASS(QHypoplasticDoc, QGraph_q_p, Graphe_q_pTemplate);
DEFINE_DOC_TEMPLATE_CLASS(QHypoplasticDoc, QGraph_eps1_eps3v2, QGraph_eps1_eps3v2Template);
DEFINE_DOC_TEMPLATE_CLASS(QHypoplasticDoc, QGraph_sig1_sig3v2, QGraph_sig1_sig3v2Template);


//{{DOC_VIEW_END}}

//{{DOC_MANAGER}}
ViewListTemplate viewList("Hypoplastic results (*.CHP)","*.chp",0,"CHP",dtAutoDelete|dtHidden);
Graphe_q_epsTemplate graphQEps("Hypoplastic results (*.*)","*.*",0,"CHP",dtAutoOpen|dtAutoDelete|dtUpdateDir);
Graphe_u_epsTemplate graphUEps("Hypoplastic results (*.CHP)","*.chp",0,"CHP",dtAutoOpen|dtAutoDelete|dtHidden);
Graphe_epsv_epsTemplate graphEpsvEps("Hypoplastic results (*.CHP)","*.chp",0,"CHP",dtAutoOpen|dtAutoDelete|dtHidden);
Graphe_q_pTemplate graphQP("Hypoplastic results (*.CHP)","*.chp",0,"CHP",dtAutoOpen|dtAutoDelete|dtHidden);
QGraph_eps1_eps3v2Template graphEps1Eps3v2("Hypoplastic results (*.CHP)","*.chp",0,"CHP",dtAutoOpen|dtAutoDelete|dtHidden);
QGraph_sig1_sig3v2Template graphSig1Sig3v2("Hypoplastic results (*.CHP)","*.chp",0,"CHP",dtAutoOpen|dtAutoDelete|dtHidden);
//{{DOC_MANAGER_END}}


const int kToolbarId = 1234;

// TMyToolbar class by Christopher Kohlhoff
// ~~~~~~~~~~~~~~~~
// A dockable toolbar close that remembers its position
// when it is hidden/shown
//
class TMyToolbar : public TDockableControlBar
{
public:
  TMyToolbar(TWindow* parent) : TDockableControlBar(parent) {}

  void Hide()
  {
    GetHarbor()->Remove(*this);
  }

  void Show(THarbor* harbor)
  {
    harbor->Insert(*this, Location, &Rect.TopLeft());
    harbor->Move(*this, Location, &Rect.TopLeft());
  }

  void SavePosition()
  {
    // get the absolute location of the toolbar
    //
    TDockingSlip* dockingSlip = TYPESAFE_DOWNCAST(Parent, TDockingSlip);
    Location = (dockingSlip ? dockingSlip->GetLocation() : alTop);

    // get the rectangle of the toolbar
    //
    TFloatingSlip* floatingSlip = TYPESAFE_DOWNCAST(Parent, TFloatingSlip);
    if (floatingSlip) floatingSlip->GetWindowRect(Rect);
    else GetRect(Rect);
  }

  TResult WindowProc(uint msg, TParam1 p1, TParam2 p2)
  {
    if (msg == WM_SHOWWINDOW && !p1) // we are being hidden
      SavePosition();
    return TDockableControlBar::WindowProc(msg, p1, p2);
  }

protected:
  TRect Rect;
  TAbsLocation Location;
};


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE2(QHypoApp, TRecentFiles, TApplication)
//{{QHypoAppRSP_TBL_BEGIN}}
  EV_OWLVIEW(dnCreate, EvNewView),
  EV_OWLVIEW(dnClose,  EvCloseView),
  EV_COMMAND(CM_ABOUT, CmHelpAbout),

  EV_COMMAND(CM_VIEWTOOLBAR, CmViewToolbar),
  EV_COMMAND_ENABLE(CM_VIEWTOOLBAR, CeViewToolbar),

  EV_WM_DROPFILES,
  EV_WM_WININICHANGE,
  EV_OWLDOCUMENT(dnCreate, EvOwlDocument),
  EV_OWLDOCUMENT(dnRename, EvOwlDocument),
  EV_REGISTERED(MruFileMessage, CmFileSelected),
//{{QHypoAppRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// QHypoApp
// ~~~~~
//
QHypoApp::QHypoApp() : TApplication("HypoPlasticity"), TRecentFiles(".\\hypo.ini", 4)
{
  Printer = 0;
  Printing = 0;

  SetDocManager(new TDocManager(dmMDI | dmMenu, this));


  // INSERT>> Your constructor code here.

}


QHypoApp::~QHypoApp()
{
  delete Printer;

  // INSERT>> Your destructor code here.

}


void QHypoApp::CreateGadgets(TDockableControlBar* cb, bool server)
{
  if (!server) {
    cb->Insert(*new TButtonGadget(CM_FILENEW, CM_FILENEW, TButtonGadget::Command));
    cb->Insert(*new TButtonGadget(CM_FILEOPEN, CM_FILEOPEN, TButtonGadget::Command));
    cb->Insert(*new TButtonGadget(CM_FILESAVE, CM_FILESAVE, TButtonGadget::Command));
    cb->Insert(*new TButtonGadget(CM_FILESAVEAS, CM_FILESAVEAS, TButtonGadget::Command));
    //cb->Insert(*new TButtonGadget(CM_FILEPRINT, CM_FILEPRINT));
    //cb->Insert(*new TButtonGadget(CM_FILEPRINTPREVIEW, CM_FILEPRINTPREVIEW));
    cb->Insert(*new TSeparatorGadget(6));
  }

  cb->Insert(*new TSeparatorGadget(6));
  cb->Insert(*new TButtonGadget(CM_SHOW_EXPERIMENT, CM_SHOW_EXPERIMENT, TButtonGadget::Command));
  cb->Insert(*new TButtonGadget(CM_REMOVE_EXPERIMENT, CM_REMOVE_EXPERIMENT, TButtonGadget::Command));
  cb->Insert(*new TSeparatorGadget);
  cb->Insert(*new TButtonGadget(CM_Q_EPS, CM_Q_EPS, TButtonGadget::Command));
  cb->Insert(*new TButtonGadget(CM_U_EPS, CM_U_EPS, TButtonGadget::Command));
  cb->Insert(*new TButtonGadget(CM_Q_P, CM_Q_P, TButtonGadget::Command));
  cb->Insert(*new TButtonGadget(CM_EPSV_EPS, CM_EPSV_EPS, TButtonGadget::Command));
  cb->Insert(*new TSeparatorGadget);
  cb->Insert(*new TButtonGadget(CM_ABOUT, CM_ABOUT, TButtonGadget::Command));

  // Add caption and fly-over help hints.
  //
  cb->SetCaption("Toolbar");
  cb->SetHintMode(TGadgetWindow::EnterHints);
}


void QHypoApp::SetupSpeedBar(TDecoratedMDIFrame* frame)
{
  ApxHarbor = new THarbor(*frame);

  // Create new remembered toolbar and associate toolbar buttons with commands.
  fToolbar = new TMyToolbar(frame);
  CreateGadgets(fToolbar);

  // Setup the toolbar ID used by OLE 2 for toolbar negotiation.
  fToolbar->Attr.Id = IDW_TOOLBAR;

  ApxHarbor->Insert(*fToolbar, alTop);
}


//--------------------------------------------------------
// QHypoApp
// ~~~~~
// Application main window construction & intialization.
//
void QHypoApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  // Construct the decorated frame window
  TDecoratedMDIFrame* frame = new TDecoratedMDIFrame("Hypoplastic model (P.A. Wolffersdorff)", 0,
												*(new TMDIClient), true, this);

  // Enable acceptance of dropped files
  //
  frame->Attr.ExStyle |= WS_EX_ACCEPTFILES;

  nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  // Assign ICON w/ this application.
  frame->SetIcon(this, IDI_MDIAPPLICATION);
  frame->SetIconSm(this, IDI_MDIAPPLICATION);

  // Associate with the accelerator table.
  frame->Attr.AccelTable = IDM_MDI;

  SetupSpeedBar(frame);

  TStatusBar* sb = new TStatusBar(frame, TGadget::Recessed,
                                  TStatusBar::CapsLock        |
                                  TStatusBar::NumLock         |
                                  TStatusBar::ScrollLock);
  frame->Insert(*sb, TDecoratedFrame::Bottom);


  SetMainWindow(frame);

  frame->SetMenuDescr(TMenuDescr(IDM_MDI)); // IDM_MDI   "COMMANDS",1,1,0,0,1,1


}


//--------------------------------------------------------
// QHypoApp
// ~~~~~
// Application instance initialization.
//
void QHypoApp::InitInstance()
{
  TApplication::InitInstance();

  ProcessCmdLine(lpCmdLine);
}


//--------------------------------------------------------
// QHypoApp
// ~~~~~
// Process command line parameters.
//
void QHypoApp::ProcessCmdLine(char * CmdLine)
{
  TCmdLine cmd(CmdLine);

  while (cmd.Kind != TCmdLine::Done) {
    if (cmd.Kind == TCmdLine::Option) {
      if (strnicmp(cmd.Token, "unregister", cmd.TokenLen) == 0) {
        UnRegisterInfo();
        return;
      }
    }
    cmd.NextToken();
  }

  RegisterInfo();
}


//--------------------------------------------------------
// QHypoApp
// ~~~~~
// Register application info.
//
void QHypoApp::RegisterInfo()
{
  TAPointer<char> buffer = new char[_MAX_PATH];

  GetModuleFileName(buffer, _MAX_PATH);

  TRegKey(TRegKey::ClassesRoot, "hypo.Application\\DefaultIcon").SetDefValue(0, REG_SZ, buffer, strlen(buffer));
  strcat(buffer, ",1");
  TRegKey(TRegKey::ClassesRoot, "hypo.Document.1\\DefaultIcon").SetDefValue(0, REG_SZ, buffer, strlen(buffer));
  strcpy(buffer, "hypo.Document.1");
  TRegKey(TRegKey::ClassesRoot, ".txt").SetDefValue(0, REG_SZ, buffer, strlen(buffer));
}


//--------------------------------------------------------
// QHypoApp
// ~~~~~
// Unregister application info.
//
void QHypoApp::UnRegisterInfo()
{
  TAPointer<char> buffer = new char[_MAX_PATH];

  GetModuleFileName(buffer, _MAX_PATH);

  TRegKey(TRegKey::ClassesRoot, "hypo.Application").DeleteKey("DefaultIcon");
  TRegKey(TRegKey::ClassesRoot, "hypo.Document.1").DeleteKey("DefaultIcon");

  TRegKey::ClassesRoot.DeleteKey("hypo.Application");
  TRegKey::ClassesRoot.DeleteKey("hypo.Document.1");
  TRegKey::ClassesRoot.DeleteKey(".txt");
}



//--------------------------------------------------------
// QHypoApp
// ~~~~~
// Response Table handlers:
//
void QHypoApp::EvNewView(TView& view)
{
  TMDIClient* mdiClient = TYPESAFE_DOWNCAST(GetMainWindow()->GetClientWindow(), TMDIClient);
  if (mdiClient) {
    TMDIChild* child = new TMDIChild(*mdiClient, 0, view.GetWindow());

    // Associate ICON w/ this child window.
    //
    child->SetIcon(this, IDI_DOC);
    child->SetIconSm(this, IDI_DOC);
    // merge menu of this child
    if (view.GetViewMenu())
      child->SetMenuDescr(*view.GetViewMenu());
    child->Create();
  }

}


void QHypoApp::EvCloseView(TView&)
{
  // Nothing needs to be done here for MDI
}


//--------------------------------------------------------
// QHypoApp
// ~~~~~~~~~~~
// Menu Help About hypo command
void QHypoApp::CmHelpAbout()
{
  // Show the modal dialog.
  //
  THypoAboutDlg(GetMainWindow(), IDD_ABOUT).Execute();
}

// Hide or show toolbar menu command
//
void QHypoApp::CmViewToolbar()
{
  if (fToolbar->IsWindowVisible()) fToolbar->Hide();
  else fToolbar->Show(ApxHarbor);
}

void QHypoApp::CeViewToolbar(TCommandEnabler &tce)
{
  tce.SetCheck(fToolbar->IsWindowVisible());
}

void QHypoApp::EvDropFiles(TDropInfo drop)
{
  TFileDropletList files;

  // Iterate thru the entries in drop and create FileDrops objects for each
  // one.
  //
  int fileCount = drop.DragQueryFileCount();  // Number of files dropped.
  for (int i = 0; i < fileCount; i++)
    files.Add(new TFileDroplet(drop, i));

  // Open the files that were dropped.
  //
  AddFiles(files);

  // Release the memory allocated for this handle with DragFinish.
  //
  drop.DragFinish();
}


void QHypoApp::AddFiles(TFileDropletList& files)
{
  // Open all files dragged in.
  //
  for (TFileDropletListIter fileIter(files); fileIter; fileIter++) {
    TDocTemplate* tpl = GetDocManager()->MatchTemplate(fileIter.Current()->GetName());
    if (tpl)
      GetDocManager()->CreateDoc(tpl, fileIter.Current()->GetName());
  }
}


void QHypoApp::EvOwlDocument(TDocument& doc)
{
  if (doc.GetDocPath())
    SaveMenuChoice(doc.GetDocPath());
}


int32 QHypoApp::CmFileSelected(uint wp, int32)
{
  TAPointer<char> text = new char[_MAX_PATH];

  GetMenuText(wp, text, _MAX_PATH);
  TDocTemplate* tpl = GetDocManager()->MatchTemplate(text);
  if (tpl)
    GetDocManager()->CreateDoc(tpl, text);
  return 0;
}


void QHypoApp::EvWinIniChange(char far* section)
{
  if (strcmp(section, "windows") == 0) {
    // If the device changed in the WIN.INI file then the printer
    // might have changed.  If we have a TPrinter(Printer) then
    // check and make sure it's identical to the current device
    // entry in WIN.INI.
    //
    if (Printer) {
      const int bufferSize = 255;
      char printDBuffer[bufferSize];
      LPSTR printDevice = printDBuffer;
      LPSTR devName;
      LPSTR driverName = 0;
      LPSTR outputName = 0;
      if (::GetProfileString("windows", "device", "", printDevice, bufferSize)) {
        // The string which should come back is something like:
        //
        //      HP LaserJet III,hppcl5a,LPT1:
        //
        // Where the format is:
        //
        //      devName,driverName,outputName
        //
        devName = printDevice;
        while (*printDevice) {
          if (*printDevice == ',') {
            *printDevice++ = 0;
            if (!driverName)
              driverName = printDevice;
            else
              outputName = printDevice;
          }
          else
            printDevice = ::AnsiNext(printDevice);
        }

        if (Printer->GetSetup().Error != 0                ||
            strcmp(devName, Printer->GetSetup().GetDeviceName()) != 0  ||
            strcmp(driverName, Printer->GetSetup().GetDriverName()) != 0 ||
            strcmp(outputName, Printer->GetSetup().GetOutputName()) != 0 ) {
          // New printer installed so get the new printer device now.
          //
          delete Printer;
          Printer = new TPrinter(this);
        }
      }
      else {
        // No printer installed(GetProfileString failed).
        //
        delete Printer;
        Printer = new TPrinter(this);
      }
    }
  }
}


int OwlMain(int , char* [])
{
  QHypoApp   app;
  return app.Run();
}


/********************* EOF ************************************************/