//----------------------------------------------------------------------------
//  Project hypoplastic model
//  
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    hypo.app Application
//  FILE:         QInputGraphAxis.cpp
//  AUTHOR:
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for QInputGraphDlg.
//
//----------------------------------------------------------------------------
#include "OWL_doanh.h"
#pragma hdrstop

#if !defined(HQ_INPUT_GRAPH_AXIS_H)
	#include "QInputGraphAxis.h"
#endif
#if !defined(HWOLFFER_HYPO_DOC_H)
	#include "wolfferdoc.h"
#endif
#if !defined(HGRAPH_H)
	#include "qgraph.h"
#endif

#if !defined(__tdrngvld_h)
	#include "tdrngvld.h"
#endif
#if !defined(OWL_EDIT_H)
	# include <owl/edit.h>
#endif
#if !defined(OWL_CHECKBOX_H)
	# include <owl/checkbox.h>
#endif
#if !defined(OWL_VALIDATE_H)
	# include <owl/validate.h>
#endif
#if !defined(OWL_COMBOBOX_H)
	# include <owl/combobox.h>
#endif

#include <math.h>
#include <stdio.h> // sprintf()

#define MAXCHAR_BUFFER 255

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(QInputGraphDlg, TDialog)
//{{QInputGraphDlgRSP_TBL_BEGIN}}
	EV_CHILD_NOTIFY(IDC_CHECKBOX1, BN_CLICKED, HHandleCheckBoxAutoScaleMsg),
//{{QInputGraphDlgRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{QInputGraphDlg Implementation}}


QInputGraphDlg::QInputGraphDlg(TWindow* parent, QHypoplasticDoc *doc, QViewGraph *graph,
	TResId resId, TModule* module)
:
    TDialog(parent, resId, module)
{
  // INSERT>> Your constructor code here.
  fDoc = doc;
  fGraph = graph;
  Edit_Ymin = new TEdit(this, IDC_EDIT1, MAXCHAR_BUFFER);  // No validator
  Edit_Ymax = new TEdit(this, IDC_EDIT2, MAXCHAR_BUFFER);
  Edit_Xmin = new TEdit(this, IDC_EDIT3, MAXCHAR_BUFFER);
  Edit_Xmax = new TEdit(this, IDC_EDIT4, MAXCHAR_BUFFER);

  Checkbox_AutoScale = new TCheckBox(this, IDC_CHECKBOX1);


}


QInputGraphDlg::~QInputGraphDlg()
{
  Destroy();

  // INSERT>> Your destructor code here.
}


void QInputGraphDlg::SetupWindow()
{
  TDialog::SetupWindow();

  // INSERT>> Your code here.
  char buffer[MAXCHAR_BUFFER];

  sprintf(buffer, "%g", fGraph->fAxeY.HMinAxe());
  Edit_Ymin->SetText(buffer);
  sprintf(buffer, "%g", fGraph->fAxeY.HMaxAxe());
  Edit_Ymax->SetText(buffer);
  sprintf(buffer, "%g", fGraph->fAxeX.HMinAxe());
  Edit_Xmin->SetText(buffer);
  sprintf(buffer, "%g", fGraph->fAxeX.HMaxAxe());
  Edit_Xmax->SetText(buffer);

  if ( fGraph->fAxeX.HAxeAuto() == QAxe::automatic )
   	Checkbox_AutoScale->Check();
  	// enable or disable the TEdits depending an autoScale
  HHandleCheckBoxAutoScaleMsg();

  	// Set the corrected of the axes
  TStatic *staticText; 		// do not need a permanent pointer
  staticText = new TStatic(this, IDC_STATIC1, MAXCHAR_BUFFER);
  staticText->Create(); 	// Create() so must be called
  staticText->SetText(fGraph->fAxeX.HAxeLabel());

  staticText = new TStatic(this, IDC_STATIC2, MAXCHAR_BUFFER);
  staticText->Create();
  staticText->SetText(fGraph->fAxeY.HAxeLabel());

    // center this dialog on the desktop
  TRect rc;
  GetWindowRect(rc);
  TScreenDC dc;
  int nX = dc.GetDeviceCaps(HORZRES) / 2 - rc.Width() / 2;
  int nY = dc.GetDeviceCaps(VERTRES) / 2 - rc.Height() / 2;
    SetWindowPos(HWND_TOP, nX, nY, 0, 0, SWP_NOSIZE);
}


void QInputGraphDlg::CloseWindow(int retVal)
{
  TDialog::CloseWindow(retVal);

  // INSERT>> Your code here.
  char buffer[MAXCHAR_BUFFER];

  if (retVal)
  {
   float min, max;
   Edit_Ymin->GetText(buffer,MAXCHAR_BUFFER);
   min = atof(buffer);
   Edit_Ymax->GetText(buffer,MAXCHAR_BUFFER);
   max = atof(buffer);
   if ( min == max )     	// min must be < max
   	max = min + 1.0;
   if ( min < max ) {
     fGraph->fAxeY.HMinAxe( min );
     fGraph->fAxeY.HMaxAxe( max );
   } else {
   	 fGraph->fAxeY.HMinAxe( max );
     fGraph->fAxeY.HMaxAxe( min );
   }

   Edit_Xmin->GetText(buffer,MAXCHAR_BUFFER);
   min = atof(buffer);
   Edit_Xmax->GetText(buffer,MAXCHAR_BUFFER);
   max = atof(buffer);
   if ( min == max )
   	max = min + 1.0;
   if ( min < max ) {
     fGraph->fAxeX.HMinAxe( min );
     fGraph->fAxeX.HMaxAxe( max );
   } else {
   	 fGraph->fAxeX.HMinAxe( max );
     fGraph->fAxeX.HMaxAxe( min );
   }

   if ( Checkbox_AutoScale->GetCheck() ) {
     		// if autoaxis, then recompute the max and min values of both axis
   	 fGraph->fAxeX.HAxeAuto(QAxe::automatic);
     fGraph->fAxeY.HAxeAuto(QAxe::automatic);
     fGraph->fAxeX.HMaxAxe(fGraph->HMaxMinX().x); //maximum, convention
		 fGraph->fAxeX.HMinAxe(fGraph->HMaxMinX().y); //manimum, convention
		 fGraph->fAxeY.HMaxAxe(fGraph->HMaxMinY().x);
     fGraph->fAxeY.HMinAxe(fGraph->HMaxMinY().y);
   } else {
     fGraph->fAxeX.HAxeAuto(QAxe::manual);
     fGraph->fAxeY.HAxeAuto(QAxe::manual);
   }
  } // end if retVal
}


TResult QInputGraphDlg::EvCommand(uint id, THandle hWndCtl, uint notifyCode)
{
  TResult result;

  result = TDialog::EvCommand(id, hWndCtl, notifyCode);

  // INSERT>> Your code here.

  return result;
}

void  QInputGraphDlg::HHandleCheckBoxAutoScaleMsg()
{
  if (Checkbox_AutoScale->GetCheck() == BF_CHECKED) {
    Edit_Xmin->SetReadOnly(true);
    Edit_Xmax->SetReadOnly(true);
    Edit_Ymin->SetReadOnly(true);
    Edit_Ymax->SetReadOnly(true);
  } else {
    Edit_Xmin->SetReadOnly(false);
    Edit_Xmax->SetReadOnly(false);
    Edit_Ymin->SetReadOnly(false);
    Edit_Ymax->SetReadOnly(false);
  }

}
/*
HBRUSH MainDialog::EvCtlColor (HDC dc, HWND hWndChild, uint ctlType)
{
   // need to call the base, esp. if you're using 3D controls
   HBRUSH result = TDialog::EvCtlColor(dc, hWndChild, ctlType);

   // now check what you want to override
   //
   // myStatic is an instance variable corresponding to your static control,
   // set it up the usual way using ClassExpert if you haven't already
   //
   if (hWndChild == myStatic->HWindow)
   {
      // go wild, set anything you like for this child control
      //
      ::SetTextColor(dc, _nA26life < 0.001 ? TColor(0,127,0) : TColor::LtRed);
      TFont someFont("Arial");
      ::SelectObject(dc, someFont);
   }
   // can continue with 'else if it's another control' here to get all your
   // controls set up properly; this is useful for changing fonts, setting the
   // background colours, etc., whatever you want to do to (er, I mean, for<g>)
   // its device context

   return result;
}
*/
/********************* EOF ************************************************/
