//----------------------------------------------------------------------------
//  Project hypoplastic model
//  
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    hypo.app Application
//  FILE:         QInputParam.cpp
//  AUTHOR:
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for QInputParamDlg.
//
//----------------------------------------------------------------------------
#include "OWL_doanh.h"
#pragma hdrstop

#if !defined(HQ_INPUT_PARAMETER_H)
	#include "QInputParam.h"
#endif
#if !defined(HWOLFFER_HYPO_DOC_H)
	#include "wolfferdoc.h"
#endif

#if !defined(__tdrngvld_h)
	#include "tdrngvld.h"
#endif
#if !defined(OWL_EDIT_H)
	# include <owl/edit.h>
#endif
#if !defined(OWL_CHECKBOX_H)
	# include <owl/checkbox.h>
#endif
#if !defined(OWL_VALIDATE_H)
	# include <owl/validate.h>
#endif

#if !defined(OWL_COMBOBOX_H)
	# include <owl/combobox.h>
#endif


#include <math.h>
#include <stdio.h> // sprintf()

#define MAXCHAR_BUFFER 255

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(QInputParamDlg, TDialog)
//{{QInputParamDlgRSP_TBL_BEGIN}}
	EV_CHILD_NOTIFY(IDC_COMBOBOX1, LBN_SELCHANGE, HHandleTestTypeMsg),
//{{QInputParamDlgRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{QInputParamDlg Implementation}}


QInputParamDlg::QInputParamDlg(TWindow* parent, QHypoplasticDoc *doc, TResId resId, TModule* module)
:
    TDialog(parent, resId, module)
{
  // INSERT>> Your constructor code here.
  fDoc = doc;
  short WuModel = fDoc->fHypoModel->defmodel == WEI_WU;
  Edit_phy = new TEdit(this, IDC_EDIT1, MAXCHAR_BUFFER);
  if ( !WuModel )
  Edit_phy->SetValidator(new TDoRangeValidator(0.0, 60.0));
  Edit_hs = new TEdit(this, IDC_EDIT2, MAXCHAR_BUFFER);
  if ( !WuModel )
  Edit_hs->SetValidator(new TDoRangeValidator(0.0001, 10000.0));
  Edit_n = new TEdit(this, IDC_EDIT3, MAXCHAR_BUFFER);
  if ( !WuModel )
  Edit_n->SetValidator(new TDoRangeValidator(0.0, 1.0));
  Edit_alpha = new TEdit(this, IDC_EDIT4, MAXCHAR_BUFFER);
  if ( !WuModel )
  Edit_alpha->SetValidator(new TDoRangeValidator(0.01, 10.0));
  Edit_beta = new TEdit(this, IDC_EDIT5, MAXCHAR_BUFFER);
  Edit_beta->SetValidator(new TDoRangeValidator(0.01, 20.0));
  Edit_ei0 = new TEdit(this, IDC_EDIT12, MAXCHAR_BUFFER);
  Edit_ei0->SetValidator(new TDoRangeValidator(0.0001, 20.0));
  Edit_ec0 = new TEdit(this, IDC_EDIT13, MAXCHAR_BUFFER);
  Edit_ec0->SetValidator(new TDoRangeValidator(0.0001, 20.0));
  Edit_ed0 = new TEdit(this, IDC_EDIT14, MAXCHAR_BUFFER);
  Edit_ed0->SetValidator(new TDoRangeValidator(0.0001, 20.0));

  Edit_sigma1 = new TEdit(this, IDC_EDIT6, MAXCHAR_BUFFER);
  Edit_sigma1->SetValidator(new TDoRangeValidator(0.0, 10000.0));
  Edit_sigma3 = new TEdit(this, IDC_EDIT7, MAXCHAR_BUFFER);
  Edit_sigma3->SetValidator(new TDoRangeValidator(0.0, 10000.0));
  Edit_eps1 = new TEdit(this, IDC_EDIT8, MAXCHAR_BUFFER);
  Edit_eps1->SetValidator(new TDoRangeValidator(-1.0, 1.0));
  Edit_eps3 = new TEdit(this, IDC_EDIT9, MAXCHAR_BUFFER);
  Edit_eps3->SetValidator(new TDoRangeValidator(-1.0, 1.0));
  Edit_e = new TEdit(this, IDC_EDIT15, MAXCHAR_BUFFER);
  Edit_e->SetValidator(new TDoRangeValidator(0.0001, 20.0));

  Edit_maxsigma = new TEdit(this, IDC_EDIT11, MAXCHAR_BUFFER);
  Edit_maxsigma->SetValidator(new TDoRangeValidator(0.0, 10000.0));
  Edit_maxeps = new TEdit(this, IDC_EDIT10, MAXCHAR_BUFFER);
  Edit_maxeps->SetValidator(new TDoRangeValidator(0.0, 1.0));
  Edit_timeStep = new TEdit(this, IDC_EDIT16, MAXCHAR_BUFFER);
  Edit_timeStep->SetValidator(new TDoRangeValidator(0.000001, 100));
  Edit_dSigmaAngle = new TEdit(this, IDC_EDIT17, MAXCHAR_BUFFER);
  Edit_dSigmaAngle->SetValidator(new TDoRangeValidator(0.0, 360.));

  ComboBox_TestType = new TComboBox(this, IDC_COMBOBOX1, 25);
  ComboBox_TestDirection = new TComboBox(this, IDC_COMBOBOX2, 25);
}


QInputParamDlg::~QInputParamDlg()
{
  Destroy();

  // INSERT>> Your destructor code here.
}


void QInputParamDlg::SetupWindow()
{
  TDialog::SetupWindow();

  // INSERT>> Your code here.
  char buffer[MAXCHAR_BUFFER];
  QHypoplasticModel *hypoModel = fDoc->fHypoModel;
  TModule *module = GetParentO()->GetModule();

  sprintf(buffer, "%g", hypoModel->defphi);
  Edit_phy->SetText(buffer);
  sprintf(buffer, "%g", hypoModel->defhs);
  Edit_hs->SetText(buffer);
  sprintf(buffer, "%g", hypoModel->defn);
  Edit_n->SetText(buffer);
  sprintf(buffer, "%g", hypoModel->defalpha);
  Edit_alpha->SetText(buffer);
  sprintf(buffer, "%g", hypoModel->defbeta);
  Edit_beta->SetText(buffer);
  sprintf(buffer, "%g", hypoModel->defei0);
  Edit_ei0->SetText(buffer);
  sprintf(buffer, "%g", hypoModel->defec0);
  Edit_ec0->SetText(buffer);
  sprintf(buffer, "%g", hypoModel->defed0);
  Edit_ed0->SetText(buffer);

  sprintf(buffer, "%g", 1000*fabs(hypoModel->defsigma[1]));
  Edit_sigma1->SetText(buffer);
  sprintf(buffer, "%g", 1000*fabs(hypoModel->defsigma[3]));
  Edit_sigma3->SetText(buffer);
  sprintf(buffer, "%g", fabs(hypoModel->defepsilon[1]));
  Edit_eps1->SetText(buffer);
  sprintf(buffer, "%g", fabs(hypoModel->defepsilon[3]));
  Edit_eps3->SetText(buffer);
  sprintf(buffer, "%g", hypoModel->defe);
  Edit_e->SetText(buffer);

  sprintf(buffer, "%g", 1000*fabs(hypoModel->defsigmamax));
  Edit_maxsigma->SetText(buffer);
  sprintf(buffer, "%g", fabs(hypoModel->defepsilonmax));
  Edit_maxeps->SetText(buffer);
  sprintf(buffer, "%g", hypoModel->defdt);
  Edit_timeStep->SetText(buffer);
  sprintf(buffer, "%g", hypoModel->defdIncAngle);
  Edit_dSigmaAngle->SetText(buffer);

  ComboBox_TestType->AddString(module->LoadString(IDS_TESTTYPE_TRIAXIAL_CID).c_str());
  ComboBox_TestType->AddString(module->LoadString(IDS_TESTTYPE_TRIAXIAL_CIU).c_str());
  ComboBox_TestType->AddString(module->LoadString(IDS_TESTTYPE_ISOTROPE).c_str());
  ComboBox_TestType->AddString(module->LoadString(IDS_TESTTYPE_OEDOMETER).c_str());
  ComboBox_TestType->AddString(module->LoadString(IDS_TESTTYPE_TRIAXIAL_CKD).c_str());
  ComboBox_TestType->AddString(module->LoadString(IDS_TESTTYPE_STRAIN_RESP_ENV).c_str());
  ComboBox_TestType->AddString(module->LoadString(IDS_TESTTYPE_STRESS_RESP_ENV).c_str());
  ComboBox_TestType->AddString(module->LoadString(IDS_TESTTYPE_LINEAR_STRAIN).c_str());
  ComboBox_TestType->AddString(module->LoadString(IDS_TESTTYPE_CIRCULAR_STRESS_PATH).c_str());

  Edit_dSigmaAngle->SetReadOnly(false); // true <<<----
  if ( hypoModel->deftype == TRIAXIAL_DRAINED )
   	ComboBox_TestType->SetSelString(module->LoadString(IDS_TESTTYPE_TRIAXIAL_CID).c_str(), 0);
  else if ( hypoModel->deftype == TRIAXIAL_UNDRAINED )
   	ComboBox_TestType->SetSelString(module->LoadString(IDS_TESTTYPE_TRIAXIAL_CIU).c_str(), 0);
  else if ( hypoModel->deftype == ISOTROPIC )
   	ComboBox_TestType->SetSelString(module->LoadString(IDS_TESTTYPE_ISOTROPE).c_str(), 0);
  else if ( hypoModel->deftype == OEDOMETRIC )
   	ComboBox_TestType->SetSelString(module->LoadString(IDS_TESTTYPE_OEDOMETER).c_str(), 0);
  else if ( hypoModel->deftype == STRESS_RESPONSE_ENVELOPE )
   	ComboBox_TestType->SetSelString(module->LoadString(IDS_TESTTYPE_STRESS_RESP_ENV).c_str(), 0);
  else if ( hypoModel->deftype == STRAIN_RESPONSE_ENVELOPE )
   	ComboBox_TestType->SetSelString(module->LoadString(IDS_TESTTYPE_STRAIN_RESP_ENV).c_str(), 0);
  else if ( hypoModel->deftype == LINEAR_STRAIN_PATH )
   	ComboBox_TestType->SetSelString(module->LoadString(IDS_TESTTYPE_LINEAR_STRAIN).c_str(), 0);
  else if ( hypoModel->deftype == LINEAR_STRESS_PATH ) {
   	ComboBox_TestType->SetSelString(module->LoadString(IDS_TESTTYPE_TRIAXIAL_CKD).c_str(), 0);
    Edit_dSigmaAngle->SetReadOnly(false);
  } else if ( hypoModel->deftype == CIRCULAR_STRESS_PATH )
   	ComboBox_TestType->SetSelString(module->LoadString(IDS_TESTTYPE_CIRCULAR_STRESS_PATH).c_str(), 0);
  /*else if ( hypoModel->deftype == CIRCULAR_STRAIN_PATH )
   	ComboBox_TestType->SetSelString(module->LoadString(IDS_TESTTYPE_CIRCULAR_STRAIN_PATH).c_str(), 0); */

  ComboBox_TestDirection->AddString(module->LoadString(IDS_TESTDIRECTION_COMPRESSION).c_str());
  ComboBox_TestDirection->AddString(module->LoadString(IDS_TESTDIRECTION_EXTENSION).c_str());

  if ( hypoModel->defdir == D_COMPRESSION )
    ComboBox_TestDirection->SetSelString(module->LoadString(IDS_TESTDIRECTION_COMPRESSION).c_str(),1);
  else if ( hypoModel->defdir == D_EXTENSION )
   	ComboBox_TestDirection->SetSelString(module->LoadString(IDS_TESTDIRECTION_EXTENSION).c_str(),2);

    // center this dialog on the desktop
  TRect rc;
  GetWindowRect(rc);
  TScreenDC dc;
  int nX = dc.GetDeviceCaps(HORZRES) / 2 - rc.Width() / 2;
  int nY = dc.GetDeviceCaps(VERTRES) / 2 - rc.Height() / 2;
    SetWindowPos(HWND_TOP, nX, nY, 0, 0, SWP_NOSIZE);
}


void QInputParamDlg::CloseWindow(int retVal)
{
  TDialog::CloseWindow(retVal);

  // INSERT>> Your code here.
  char buffer[MAXCHAR_BUFFER];
  QHypoplasticModel *hypoModel = fDoc->fHypoModel;
  TModule *module = GetParentO()->GetModule();
  
  if (retVal)
  {
  	Edit_phy->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defphi = atof(buffer);
   Edit_hs->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defhs = atof(buffer);
   Edit_n->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defn = atof(buffer);
   Edit_alpha->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defalpha = atof(buffer);
   Edit_beta->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defbeta = atof(buffer);
   Edit_ei0->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defei0 = atof(buffer);
   Edit_ec0->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defec0 = atof(buffer);
   Edit_ed0->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defed0 = atof(buffer);

   	// check the consistancy of ei0, ec0, ed0
   if((hypoModel->defed0 > hypoModel->defec0) ||
   		(hypoModel->defed0 > hypoModel->defei0) ||
      (hypoModel->defec0 > hypoModel->defei0) ){
		 MessageBox(module->LoadString(IDS_ER_EDOECOEIO).c_str(),
     					module->LoadString(IDS_ER_MODEL_PARAMETERS).c_str(), MB_OK);
     if ( hypoModel->defed0 > hypoModel->defec0 )
     	hypoModel->defed0 = hypoModel->defec0;
     if ( hypoModel->defec0 > hypoModel->defei0 )
     	hypoModel->defec0 = hypoModel->defei0;
   }

   Edit_sigma1->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defsigma[1] = -0.001*atof(buffer);
   Edit_sigma3->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defsigma[2] =
   	hypoModel->defsigma[3] = -0.001*atof(buffer);
   Edit_eps1->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defepsilon[1] = -atof(buffer);
   Edit_eps3->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defepsilon[2] =
   	hypoModel->defepsilon[3] = -atof(buffer);
   Edit_e->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defe = atof(buffer);

   	// check the range of current void ratio
   if((hypoModel->defed0 > hypoModel->defe) ||
      (hypoModel->defe > hypoModel->defei0) ){
		 MessageBox(module->LoadString(IDS_ER_EDOEEIO).c_str(),
     					module->LoadString(IDS_ER_MODEL_PARAMETERS).c_str(), MB_OK);
     if ( hypoModel->defed0 > hypoModel->defe )
     	hypoModel->defe = hypoModel->defed0;
     if ( hypoModel->defe > hypoModel->defei0 )
     	hypoModel->defe = hypoModel->defei0;
   }

   Edit_maxsigma->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defsigmamax = -0.001*atof(buffer);
   Edit_maxeps->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defepsilonmax = -atof(buffer);
   Edit_timeStep->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defdt = fabs(atof(buffer));
   Edit_dSigmaAngle->GetText(buffer,MAXCHAR_BUFFER);
   hypoModel->defdIncAngle = fabs(atof(buffer));

   char buf[256] = "";
   int  index = ComboBox_TestType->GetSelIndex();

   if (index != -1) {     // is something selected?
    ComboBox_TestType->GetString(buf, index);
    if ( strstr(buf, module->LoadString(IDS_TESTTYPE_TRIAXIAL_CID).c_str()) )
   		hypoModel->deftype = TRIAXIAL_DRAINED;
   	else if ( strstr(buf, module->LoadString(IDS_TESTTYPE_TRIAXIAL_CIU).c_str()) )
   		hypoModel->deftype = TRIAXIAL_UNDRAINED;
   	else if ( strstr(buf, module->LoadString(IDS_TESTTYPE_ISOTROPE).c_str()) )
   		hypoModel->deftype = ISOTROPIC;
   	else if ( strstr(buf, module->LoadString(IDS_TESTTYPE_OEDOMETER).c_str()) )
   		hypoModel->deftype = OEDOMETRIC;
    	else if ( strstr(buf, module->LoadString(IDS_TESTTYPE_TRIAXIAL_CKD).c_str()) )
   		hypoModel->deftype = LINEAR_STRESS_PATH;
    	else if ( strstr(buf, module->LoadString(IDS_TESTTYPE_STRAIN_RESP_ENV).c_str()) )
   		hypoModel->deftype = STRAIN_RESPONSE_ENVELOPE;
    	else if ( strstr(buf, module->LoadString(IDS_TESTTYPE_STRESS_RESP_ENV).c_str()) )
   		hypoModel->deftype = STRESS_RESPONSE_ENVELOPE;
    	else if ( strstr(buf, module->LoadString(IDS_TESTTYPE_LINEAR_STRAIN).c_str()) )
   		hypoModel->deftype = LINEAR_STRAIN_PATH;
      else if ( strstr(buf, module->LoadString(IDS_TESTTYPE_CIRCULAR_STRESS_PATH).c_str()) )
   		hypoModel->deftype = CIRCULAR_STRESS_PATH;
      /*else if ( strstr(buf, module->LoadString(IDS_TESTTYPE_CIRCULAR_STRAIN_PATH).c_str()) )
   		hypoModel->deftype = CIRCULAR_STRAIN_PATH;  */
   }

   index = ComboBox_TestDirection->GetSelIndex();

   if (index != -1) {     // is something selected?
    ComboBox_TestDirection->GetString(buf, index);
    if ( strstr(buf, module->LoadString(IDS_TESTDIRECTION_COMPRESSION).c_str()) )
   		hypoModel->defdir = D_COMPRESSION;
   	else if ( strstr(buf, module->LoadString(IDS_TESTDIRECTION_EXTENSION).c_str()) )
   		hypoModel->defdir = D_EXTENSION;
   }

   if ( hypoModel->defdir == D_EXTENSION ) {
    	hypoModel->defsigmamax = -hypoModel->defsigmamax;
      hypoModel->defepsilonmax = -hypoModel->defepsilonmax;
   }

  }
}


TResult QInputParamDlg::EvCommand(uint id, THandle hWndCtl, uint notifyCode)
{
  TResult result;

  result = TDialog::EvCommand(id, hWndCtl, notifyCode);

  // INSERT>> Your code here.

  return result;
}

void  QInputParamDlg::HHandleTestTypeMsg()
{
  char buf[256] = "";
  TModule *module = GetParentO()->GetModule();
  int  index = ComboBox_TestType->GetSelIndex();

   if (index != -1) {     // is something selected?
    ComboBox_TestType->GetString(buf, index);
    if ( strstr(buf, module->LoadString(IDS_TESTTYPE_TRIAXIAL_CKD).c_str()) )
   		Edit_dSigmaAngle->SetReadOnly(false);
    else {
    	Edit_dSigmaAngle->SetReadOnly(false); // true <<<----
   }
  }

}

/********************* EOF ************************************************/

