//----------------------------------------------------------------------------
//  Project hypoplastic model
//
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    hypo.app Application
//  FILE:         qaxes.cpp
//  AUTHOR:
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for QAxe, QAxeX, QAxeY
//
//----------------------------------------------------------------------------

#include "OWL_doanh.h"
#pragma hdrstop

#include <stdio.h>
#include <math.h>
#include <string.h>

#if !defined(HQAXE_H)
	#include "qaxes.h"
#endif

	// static data members


/*  ***************************************************************
QAxe class

******************************************************************* */
QAxe::QAxe(const char *name, const char *label, eGraduation repr, eAxeAuto axeAuto):
  	fAxeStep(5),fMinAxe(0),fMaxAxe(1),fLowerPixel(1),fUpperPixel(0),fWhiteGap(1),
    fGraduation(repr), fAxeAuto(axeAuto)
{
	fName = name;
  fLabel = label;
}

void QAxe::HAxeName(const char *name)
{
  fName = name;
}

void QAxe::HAxeLabel(const char *label)
{
	fLabel = label;
}

void QAxe::HComputeScaleAxe() {
	if ( fMaxAxe != fMinAxe )
  	fScale = ( fUpperPixel - fLowerPixel )/( fMaxAxe - fMinAxe );
	else
  	fScale = 1000000;
}

void QAxe::HUnit(const char *unit)
{
  fUnit = unit;
}



/*  ***************************************************************
QAxeX derived class

******************************************************************* */
void QAxeX::HInitQAxe(TWindow &window){
	TRect rect;
	window.GetClientRect(rect);
	HUpperPixel(0.9*rect.right);
	HLowerPixel(0.15*rect.right);
	HWhiteGap(0.8*rect.bottom);
	HComputeScaleAxe();
}

void QAxeX::HDisplay(TDC &dc){
	TPen*  Pen;
	LOGFONT flFont1;
	LOGFONT flFont2;
	TFont* fFont1;
	TFont* fFont2;
	Pen = new TPen(TColor::Black, 1);
	dc.SelectObject(*Pen);
	dc.MoveTo(fUpperPixel,fWhiteGap);
	dc.LineTo(fLowerPixel,fWhiteGap);
	if(fAxeStep){
		int currentValue=fLowerPixel;
		float value;
		char s[10];
		char *font="Times New Roman";
    	// all items of flFont1 must be explicitly defined
		flFont1.lfHeight=(fUpperPixel-fLowerPixel)/fAxeStep*0.2;
		flFont1.lfWidth=(fUpperPixel-fLowerPixel)/fAxeStep*0.06;
		flFont1.lfEscapement=0;
		flFont1.lfWeight=FW_NORMAL;
		flFont1.lfItalic=FALSE;
		flFont1.lfUnderline=FALSE;
		flFont1.lfStrikeOut=FALSE;
		flFont1.lfCharSet=DEFAULT_CHARSET;
		flFont1.lfOutPrecision=OUT_DEFAULT_PRECIS;
		flFont1.lfClipPrecision=CLIP_DEFAULT_PRECIS;
		flFont1.lfQuality=DRAFT_QUALITY;
		flFont1.lfPitchAndFamily=DEFAULT_PITCH;
		flFont1.lfFaceName[16]=*font;
		flFont2=flFont1;
		flFont2.lfWidth=fUpperPixel*0.015;
		flFont2.lfHeight=fUpperPixel*0.03;

		dc.SetTextAlign(TA_CENTER);
		fFont2=new TFont(&flFont2);
		fFont1=new TFont(&flFont1);
		dc.SelectObject(*fFont2);
		dc.TextOut(fUpperPixel*0.5/0.9, fWhiteGap*1.13, fLabel.c_str(), fLabel.length());
		dc.TextOut(fUpperPixel*1.07, fWhiteGap-fUpperPixel*0.015, fName.c_str(), fName.length());
		dc.SelectObject(*fFont1);

		while ( currentValue < fUpperPixel ){
			dc.MoveTo(currentValue, 0.99*fWhiteGap);
			dc.LineTo(currentValue, 1.01*fWhiteGap);
			if ( fGraduation == linear ){
				value = fMinAxe + (currentValue-fLowerPixel)/fScale;
				sprintf(s, "%.3f" , value);
				dc.TextOut(currentValue,1.03*fWhiteGap, s, strlen(s));
				currentValue += (fUpperPixel-fLowerPixel)/fAxeStep;
			}
		}

		delete fFont1;
		delete fFont2;
	 }
	 delete Pen;
}

int QAxeX::HToPixel(float value){
	if ( fGraduation == linear ) {
		return	fLowerPixel + (value-fMinAxe)*fabs(fScale);
	}
	return 0;
}

float QAxeX::HFromPixel(int value){
	if ( fGraduation == linear ) {
		return	fMinAxe + (value-fLowerPixel)/fabs(fScale);
	}
	return 0;
}



/*  ***************************************************************
QAxeX derived class

******************************************************************* */
void QAxeY::HInitQAxe(TWindow &window){
	TRect rect;
	window.GetClientRect(rect);
	HUpperPixel(0.1*rect.bottom);
	HLowerPixel(0.8*rect.bottom);
	HWhiteGap(0.15*rect.right);
	HComputeScaleAxe();
}

void QAxeY::HDisplay(TDC &dc){
	TPen*  Pen;
	LOGFONT flFont1;
	LOGFONT flFont2;
	TFont* fFont1;
	TFont* fFont2;
	Pen = new TPen(TColor::Black, 1);
	dc.SelectObject(*Pen);
	dc.MoveTo(fWhiteGap,fUpperPixel);
	dc.LineTo(fWhiteGap,fLowerPixel);
	if(fAxeStep){
		int currentValue=fLowerPixel;
		float value;
		char s[10];
		char *font="Times New Roman";

		flFont1.lfHeight=fWhiteGap*0.20;
		flFont1.lfWidth=fWhiteGap*0.06;
		flFont1.lfEscapement=0;
		flFont1.lfWeight=FW_NORMAL;
		flFont1.lfItalic=FALSE;
		flFont1.lfUnderline=FALSE;
		flFont1.lfStrikeOut=FALSE;
		flFont1.lfCharSet=DEFAULT_CHARSET;
		flFont1.lfOutPrecision=OUT_DEFAULT_PRECIS;
		flFont1.lfClipPrecision=CLIP_DEFAULT_PRECIS;
		flFont1.lfQuality=DRAFT_QUALITY;
		flFont1.lfPitchAndFamily=DEFAULT_PITCH;
		flFont1.lfFaceName[16]=*font;
		flFont2=flFont1;
		flFont2.lfWidth=fLowerPixel*0.025;
		flFont2.lfHeight=fLowerPixel*0.05;
		flFont2.lfEscapement=900;

		fFont2=new TFont(&flFont2);
		fFont1=new TFont(&flFont1);
		dc.SelectObject(*fFont2);
		dc.TextOut(fWhiteGap*0.2, fLowerPixel*0.5/0.8, fLabel.c_str(), fLabel.length());
		dc.TextOut(fWhiteGap, fUpperPixel*0.55, fName.c_str(), fName.length());
		dc.SelectObject(*fFont1);
		dc.SetTextAlign(TA_BASELINE);

		while ( currentValue > fUpperPixel ){
			dc.MoveTo(0.96*fWhiteGap, currentValue);
			dc.LineTo(1.07*fWhiteGap, currentValue);
			if ( fGraduation == linear ){
				value = fMinAxe + (currentValue-fLowerPixel)/fScale;
			}
			sprintf(s, "%.3f" , value);
			dc.TextOut(0.48*fWhiteGap,currentValue+fWhiteGap*0.08, s, strlen(s));
			currentValue -= (fLowerPixel-fUpperPixel)/fAxeStep;
		}

		delete fFont1;
		delete fFont2;
	 }
	 delete Pen;
}

int QAxeY::HToPixel(float value){
	if ( fGraduation == linear ) {
		return	fLowerPixel - (value-fMinAxe)*fabs(fScale) ;
	}
	return 0;
}

float QAxeY::HFromPixel(int value){
	if ( fGraduation == linear ) {
		return	fMinAxe + (fLowerPixel-value)/fabs(fScale);
	}
	return 0;
}

/********************* EOF ************************************************/
