//----------------------------------------------------------------------------
//  Project hypoplastic model
//  
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    hypo.app Application
//  FILE:         qaxes.h
//  AUTHOR:
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for QAxe, QAxeX, QAxeY
//
//----------------------------------------------------------------------------
#if !defined(HQAXE_H)              // Sentry, use file only if it's not already included.
#define HQAXE_H

class TWindow;
class TDC;

/*  ***************************************************************
QAxe base class

******************************************************************* */
class QAxe{
public:
  enum eGraduation {
      linear = 0,
      logarithmic,
      exponential,
    };
  enum eAxeAuto {
      automatic = 0,
      manual,
    };

	QAxe(const char *name="Variable", const char *label="Axe Label",
  	eGraduation repr=linear, eAxeAuto axeAuto=automatic);
	virtual ~QAxe(){}
  
	virtual void HInitQAxe(TWindow &) = 0;
	virtual void HDisplay(TDC &) = 0;
	virtual int HToPixel(float) = 0;
  virtual float HFromPixel(int x) = 0;

		// Accessors and mutators for protected data members
	const char* HAxeName()    	{ return fName.c_str();}
  void  HAxeName(const char *name);
	const char* HAxeLabel()	{ return fLabel.c_str();}
  void  HAxeLabel(const char *label);

	int HGraduation()		{ return fGraduation;}
	void HGraduation(eGraduation graduation){ fGraduation = graduation;}
  eAxeAuto HAxeAuto()		{ return fAxeAuto;}
	void HAxeAuto(eAxeAuto axeAuto){ fAxeAuto = axeAuto;}
  bool HIsAutoAxis() { return fAxeAuto == automatic; }
	float HMinAxe() 		{ return fMinAxe;}
	void HMinAxe(float minAxe){
  	fMinAxe = minAxe;
    HComputeScaleAxe();
	}
	float HMaxAxe() 		{ return fMaxAxe;}
	void HMaxAxe(float maxAxe){
  	fMaxAxe = maxAxe;
    HComputeScaleAxe();
	}
	int HUpperPixel()		{ return fUpperPixel;}
	void HUpperPixel(int repr){
  	fUpperPixel = repr;
    HComputeScaleAxe();
	}
	int HLowerPixel()		{ return fLowerPixel;}
	void HLowerPixel(int repr){
  	fLowerPixel = repr;
    HComputeScaleAxe();
	}
	int HWhiteGap()		{ return fWhiteGap;}
	void HWhiteGap(int repr){ fWhiteGap=repr;}
	float HAxeStep()		{ return fAxeStep;}
	void HAxeStep(float axeStep){ fAxeStep = axeStep;}
	float HScaleAxe() { return fScale;}
	void HScaleAxe(float scale) { fScale = scale;}
	void HComputeScaleAxe();
  void HUnit(const char *unit);
  const char *HUnit(void) { return fUnit.c_str(); }

protected:
	eGraduation fGraduation;// linear, exponent, log...
  eAxeAuto fAxeAuto;
	float fMinAxe;  	// minimum user value of axe
	float fMaxAxe;  	// maximum user value of axe
	float fAxeStep; 	// major ticks
	float fScale; 		// from or to pixel
	int fUpperPixel;  // corresponds to maximum user value of axe
	int fLowerPixel;  // corresponds to minimum user value of axe
	int fWhiteGap;  	// space in pixels between the axis and the window
  string fName; 		 // short axis label at the axis
	string fLabel; 		// axis label below the axis
  string fUnit;     // SI Unit
};


/*  ***************************************************************
QAxeX derived class

******************************************************************* */
class QAxeX: public QAxe{
public:
	void HInitQAxe(TWindow &);
	void HDisplay(TDC &);
  int HToPixel(float x);
  float HFromPixel(int x);
};

/*  ***************************************************************
QAxeX derived class

******************************************************************* */
class QAxeY: public QAxe{
public:
	void HInitQAxe(TWindow &);
	void HDisplay(TDC &);
	int HToPixel(float x);
  float HFromPixel(int x);
};

#endif  //  sentry.

/********************* EOF ************************************************/
