//----------------------------------------------------------------------------
//  Project hypoplastic model
//  
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    hypo.app Application
//  FILE:         qcurves.cpp
//  AUTHOR:
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for PointCurve, QCurves.
//		QCurves_q_eps, QCurves_u_eps, QCurves_epsv_eps, QCurves_q_p
//
//----------------------------------------------------------------------------

#if !defined(HQCURVES_H)
	#include "qcurves.h"
#endif

#if !defined(HWOLFFER_HYPO_DOC_H)
	#include "wolfferdoc.h"
#endif


/*  ***************************************************************
QCurves class

******************************************************************* */

QCurves::QCurves( HHypoResultsVector& results, const char *axeNameX, const char *axeNameY,
  	const char *label, TColor color,
		int epp, int style )
    :QCurve(10,0,10),
    fHypoResults(results), fAxeNameX(axeNameX), fAxeNameY(axeNameY),
		fLabel(label), fColor(color), fWidth(epp), fStyle(style)
{

}

void QCurves::HAxeNameX(const char *axeName)
{
	fAxeNameX = axeName;
}

void QCurves::HAxeNameY(const char *axeName)
{
	fAxeNameY = axeName;
}

void QCurves::HLabel(const char *label)
{
	fLabel = label;
}

PointCurve QCurves::HGetPoint(int index){
	PointCurve point;
  point.x = HGetPointX(index);
	point.y = HGetPointY(index);
	return point;
}

// return the maximum PointCurve.x and minimum PointCurve.y of axe X
PointCurve QCurves::HMaxMinX(){
	double max=0,min=0,current;
	PointCurve point;
	int i,j;
	j = fHypoResults.GetItemsInContainer();
	if ( j ){ 									// initialize min/max;
		max = HGetPointX(0);
		min = max;
	}
	for(i=1; i<j; i++){
		current = HGetPointX(i);  // only one access to current point i of curve
		if ( current > max )
    	max = current;
		else if( current < min )
    	min = current;
	}
	if ( max == min ){
		if ( max == 0 )
    	max = 1; 								// sentinel
		max = 1.1*max;
		min = 0.9*min;
	}

	point.x = max; 							// convention
	point.y = min;

	return point;
}

PointCurve QCurves::HMaxMinY(){
	double max=0,min=0,current;
	PointCurve point;
	int i,j;
	j = fHypoResults.GetItemsInContainer();
	if ( j ){
		max = HGetPointY(0);
		min = max;
	}
	for( i=1; i<j; i++){
		current = HGetPointY(i);
		if ( current > max )
    	max = current;
		else if ( current < min )
    	min = current;
	}
	if ( max == min ){
		if ( max == 0 )
    	max = 1;
		max = 1.1*max;
		min = 0.9*min;
	}

	point.x = max;
	point.y = min;

	return point;
}


/*  ***************************************************************
QCurves derived class

******************************************************************* */
	/* avoid to call virtual methods in constructor */
void QCurves_q_eps::HInitQCurves_q_eps(void){
	fMaxMinX = HMaxMinX();
  fMaxMinY = HMaxMinY();
}

double QCurves_q_eps::HGetPointX(int index){
	double x;
  x = fHypoResults[index].epsilon1;
	return x;
}

double QCurves_q_eps::HGetPointY(int index){
	double y;
	y = (fHypoResults[index].sigma1-fHypoResults[index].sigma3);
	return y;
}



	/* avoid to call virtual methods in constructor */
void QCurves_u_eps::HInitQCurves_u_eps(void){
	fMaxMinX = HMaxMinX();
  fMaxMinY = HMaxMinY();
}

double QCurves_u_eps::HGetPointX(int index){
	double x;
  x = fHypoResults[index].epsilon1;
	return x;
}

double QCurves_u_eps::HGetPointY(int index){
	double y;
	y = (fHypoResults[0].sigma3-fHypoResults[index].sigma3);
	return y;
}


	/* avoid to call virtual methods in constructor */
void QCurves_u_eps_exp::HInitQCurves_u_eps_exp(void){
	fMaxMinX = HMaxMinX();
  fMaxMinY = HMaxMinY();
}

double QCurves_u_eps_exp::HGetPointX(int index){
	double x;
  x = fHypoResults[index].epsilon1;
	return x;
}
	// .sigma2 contains measured pore pressure
double QCurves_u_eps_exp::HGetPointY(int index){
	double y;
	y = (fHypoResults[index].sigma2-fHypoResults[0].sigma2);
	return y;
}



	/* avoid to call virtual methods in constructor */
void QCurves_epsv_eps::HInitQCurves_epsv_eps(void){
	fMaxMinX = HMaxMinX();
  fMaxMinY = HMaxMinY();
}

double QCurves_epsv_eps::HGetPointX(int index){
	double x;
  x = fHypoResults[index].epsilon1;
	return x;
}

double QCurves_epsv_eps::HGetPointY(int index){
	double y;
	y = (fHypoResults[index].epsilon1 + 2.0*fHypoResults[index].epsilon3);
	return y;
}

	/* avoid to call virtual methods in constructor */
void QCurves_epsv_eps_exp::HInitQCurves_epsv_eps_exp(void){
	fMaxMinX = HMaxMinX();
  fMaxMinY = HMaxMinY();
}

double QCurves_epsv_eps_exp::HGetPointX(int index){
	double x;
  x = fHypoResults[index].epsilon1;
	return x;
}
	// .e contains measured volumetric strain
  // .epsilon2 is not correct
double QCurves_epsv_eps_exp::HGetPointY(int index){
	double y;
	y = (fHypoResults[index].e - fHypoResults[0].e);
	return y;
}


	/* avoid to call virtual methods in constructor */
void QCurves_q_p::HInitQCurves_q_p(void){
	fMaxMinX = HMaxMinX();
  fMaxMinY = HMaxMinY();
}

double QCurves_q_p::HGetPointX(int index){
	double x;
  x = (fHypoResults[index].sigma1 + 2.0*fHypoResults[index].sigma3) / 3.0;
	return x;
}

double QCurves_q_p::HGetPointY(int index){
	double y;
	y = (fHypoResults[index].sigma1-fHypoResults[index].sigma3);
	return y;
}


	/* avoid to call virtual methods in constructor */
void QCurves_eps1_eps3v2::HInitQCurves_eps1_eps3v2(void){
	fMaxMinX = HMaxMinX();
  fMaxMinY = HMaxMinY();
}

double QCurves_eps1_eps3v2::HGetPointX(int index){
	double x;
  x = fHypoResults[index].epsilon3 * sqrt(2.0);
	return x;
}

double QCurves_eps1_eps3v2::HGetPointY(int index){
	double y;
	y = fHypoResults[index].epsilon1;
	return y;
}

	/* avoid to call virtual methods in constructor */
void QCurves_sig1_sig3v2::HInitQCurves_sig1_sig3v2(void){
	fMaxMinX = HMaxMinX();
  fMaxMinY = HMaxMinY();
}

double QCurves_sig1_sig3v2::HGetPointX(int index){
	double x;
  x = fHypoResults[index].sigma3 * sqrt(2.0);
	return x;
}

double QCurves_sig1_sig3v2::HGetPointY(int index){
	double y;
	y = fHypoResults[index].sigma1;
	return y;
}
/********************* EOF ************************************************/
