//----------------------------------------------------------------------------
//  Project hypoplastic model
//  
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    hypo.app Application
//  FILE:         qcurves.h
//  AUTHOR:
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for PointCurve, QCurves.
//		QCurves_q_eps, QCurves_u_eps, QCurves_epsv_eps, QCurves_q_p
//
//----------------------------------------------------------------------------
#if !defined(HQCURVES_H)              // Sentry, use file only if it's not already included.
#define HQCURVES_H


#include "OWL_doanh.h"
#pragma hdrstop

#include <classlib/arrays.h>
#include <string.h>

#if !defined(HWOLFFER_HYPO_DOC_H)
	#include "wolfferdoc.h"
#endif


class PointCurve{ // a simple structure of real coordinates like FPoint
			// with operator == and default constructor for BIDS
public:
	double x;
	double y;
	bool operator ==(const PointCurve& other) const
	{
		return &other == this;
	}
};

typedef TArray<PointCurve> QCurve;
typedef TArrayIterator<PointCurve> QCurveIterator;
typedef TIArrayAsVector<PointCurve> IAPointCurve;
typedef TIArrayAsVectorIterator<PointCurve> IAPointCurveIterator;


class QCurves: public QCurve{
public:
	QCurves(HHypoResultsVector& results, const char *axeNameX="X", const char *axeNameY="Y",
  	const char *label="Simulation", TColor color=TColor::Black,
		int epp=1, int style=PS_SOLID);

	virtual ~QCurves(){}
	virtual PointCurve HGetPoint(int index); // of index i
	virtual	PointCurve HMaxMinX();
	virtual	PointCurve HMaxMinY();
  virtual double HGetPointX(int index)=0;
  virtual double HGetPointY(int index)=0;

		// Accessors and Mutators
	const char* HAxeNameX()     		{ return fAxeNameX.c_str();}
	void HAxeNameX(const char *axeName);
	const char* HAxeNameY()     		{ return fAxeNameY.c_str();}
	void HAxeNameY(const char *axeName);
	const char* HLabel()  		{ return fLabel.c_str();}
	void HLabel(const char *label);
	int HWidth()  		{ return fWidth;}
	void HWidth(int width) { fWidth = width;}
	TColor HColor()  		{ return fColor;}
	void HColor(TColor color) { fColor = color;}
	int HStyle()  		{ return fStyle;}
	void HStyle(int style) { fStyle = style;}
	virtual bool operator ==(const QCurves& other) const
	{
		return &other == this;
	}

protected:
	string fAxeNameX;
	string fAxeNameY;
	string fLabel;    // curve legend
	int fWidth; 			// line width
	TColor fColor; 		// line color
	int fStyle; 			// line style
	HHypoResultsVector& fHypoResults;
	PointCurve fMaxMinX;
	PointCurve fMaxMinY;

};

typedef TIArrayAsVector<QCurves> IAQCurves;
typedef TIArrayAsVectorIterator<QCurves> IAQCurvesIterator;


class QCurves_q_eps: public QCurves{
public:
	QCurves_q_eps(HHypoResultsVector& results, const char *axeNameX="X", const char *axeNameY="Y",
		 const char *label="Simulation",TColor color=TColor::Black,int epp=1,int style=PS_SOLID):
	QCurves(results,axeNameX,axeNameY,label,color,epp,style){}
  void HInitQCurves_q_eps(void);
  virtual double HGetPointX(int index);
  virtual double HGetPointY(int index);

};

class QCurves_u_eps: public QCurves{
public:
	QCurves_u_eps(HHypoResultsVector& results, const char *axeNameX="X", const char *axeNameY="Y",
		 const char *label="Simulation",TColor color=TColor::Black,int epp=1,int style=PS_SOLID):
	QCurves(results,axeNameX,axeNameY,label,color,epp,style){}
  void HInitQCurves_u_eps(void);
  virtual double HGetPointX(int index);
  virtual double HGetPointY(int index);
};

class QCurves_u_eps_exp: public QCurves{
public:
	QCurves_u_eps_exp(HHypoResultsVector& results, const char *axeNameX="X", const char *axeNameY="Y",
		 const char *label="Experiment",TColor color=TColor::LtRed,int epp=1,int style=PS_SOLID):
	QCurves(results,axeNameX,axeNameY,label,color,epp,style){}
  void HInitQCurves_u_eps_exp(void);
  virtual double HGetPointX(int index);
  virtual double HGetPointY(int index);
};

class QCurves_epsv_eps: public QCurves{
public:
	QCurves_epsv_eps(HHypoResultsVector& results, const char *axeNameX="X", const char *axeNameY="Y",
		 const char *label="Simulation",TColor color=TColor::Black,int epp=1,int style=PS_SOLID):
	QCurves(results,axeNameX,axeNameY,label,color,epp,style){}
  void HInitQCurves_epsv_eps(void);
  virtual double HGetPointX(int index);
  virtual double HGetPointY(int index);
};

class QCurves_epsv_eps_exp: public QCurves{
public:
	QCurves_epsv_eps_exp(HHypoResultsVector& results, const char *axeNameX="X", const char *axeNameY="Y",
		 const char *label="Experiment",TColor color=TColor::LtRed,int epp=1,int style=PS_SOLID):
	QCurves(results,axeNameX,axeNameY,label,color,epp,style){}
  void HInitQCurves_epsv_eps_exp(void);
  virtual double HGetPointX(int index);
  virtual double HGetPointY(int index);
};

class QCurves_q_p: public QCurves{
public:
	QCurves_q_p(HHypoResultsVector& results, const char *axeNameX="X", const char *axeNameY="Y",
		 const char *label="Simulation",TColor color=TColor::Black,int epp=1,int style=PS_SOLID):
	QCurves(results,axeNameX,axeNameY,label,color,epp,style){}
  void HInitQCurves_q_p(void);
  virtual double HGetPointX(int index);
  virtual double HGetPointY(int index);
};

class QCurves_eps1_eps3v2: public QCurves{
public:
	QCurves_eps1_eps3v2(HHypoResultsVector& results, const char *axeNameX="X", const char *axeNameY="Y",
		 const char *label="Simulation",TColor color=TColor::Black,int epp=1,int style=PS_SOLID):
	QCurves(results,axeNameX,axeNameY,label,color,epp,style){}
  void HInitQCurves_eps1_eps3v2(void);
  virtual double HGetPointX(int index);
  virtual double HGetPointY(int index);
};

class QCurves_sig1_sig3v2: public QCurves{
public:
	QCurves_sig1_sig3v2(HHypoResultsVector& results, const char *axeNameX="X", const char *axeNameY="Y",
		 const char *label="Simulation",TColor color=TColor::Black,int epp=1,int style=PS_SOLID):
	QCurves(results,axeNameX,axeNameY,label,color,epp,style){}
  void HInitQCurves_sig1_sig3v2(void);
  virtual double HGetPointX(int index);
  virtual double HGetPointY(int index);
};


#endif  //  sentry.

/********************* EOF ************************************************/