//----------------------------------------------------------------------------
//  Project hypoplastic model
//  
//  Copyright  1997-1999. All Rights Reserved.
//
//  SUBSYSTEM:    hypo.app Application
//  FILE:         qgraph.cpp
//  AUTHOR:       Doanh T., Bourgeois C.
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for QViewGraph.
//		QGraph_q_eps, QGraph_u_eps, QGraph_epsv_eps, QGraph_q_p
//
//----------------------------------------------------------------------------

#if !defined(HGRAPH_H)
	#include "qgraph.h"
#endif

#if !defined(HQ_INPUT_GRAPH_AXIS_H)
	#include "QInputGraphAxis.h"
#endif

#include "hypoapp.rc"

#if !defined(OWL_MESSAGEB_H)
	#include <owl/messageb.h>
#endif

#include <stdio.h>


	// response table of QViewGraph
DEFINE_RESPONSE_TABLE1(QViewGraph, TWindowView)
	EV_VN_CHANGTPARAMETRES,
  EV_WM_LBUTTONDOWN,
  EV_WM_MOUSEMOVE,
  EV_WM_LBUTTONUP,
  EV_WM_RBUTTONDOWN,
  EV_WM_SIZE,
  EV_WM_LBUTTONDBLCLK,
  EV_WM_RBUTTONDBLCLK,

  EV_COMMAND(CM_Q_EPS, CmQ_EPS),
  EV_COMMAND(CM_U_EPS, CmU_EPS),
  EV_COMMAND(CM_Q_P, CmQ_P),
  EV_COMMAND(CM_EPSV_EPS, CmEPSV_EPS),
  EV_COMMAND(CM_Sig1_Sig3v2, CmSig1_Sig3v2),
  EV_COMMAND(CM_Eps1_Eps3v2, CmEps1_Eps3v2),

  EV_COMMAND(CM_INPUT_PARAMETER, CmInputParameters),
  EV_COMMAND_ENABLE(CM_INPUT_PARAMETER, CeInputParameters),
  EV_COMMAND(CM_GRAPH_SCALE, CmGetGraphAxisScale),
  EV_COMMAND_ENABLE(CM_GRAPH_SCALE, CeGetGraphAxisScale),
  EV_COMMAND(CM_SHOW_EXPERIMENT, HShowExperimentalData),
  EV_COMMAND_ENABLE(CM_SHOW_EXPERIMENT, CeGetExperimentalData),
  EV_COMMAND(CM_REMOVE_EXPERIMENT, HRemoveExperimentalData),
  EV_COMMAND_ENABLE(CM_REMOVE_EXPERIMENT, CeRemoveExperimentalData),
  EV_VN_COMMIT,
  EV_VN_REVERT,
END_RESPONSE_TABLE;

/*  ***************************************************************
QViewGraph base class

******************************************************************* */
QViewGraph::QViewGraph(QHypoplasticDoc& doc, TWindow* parent, const char *title):
	TWindowView(doc, parent), fHypoDoc(doc)
{
  fTitle = title;

  dc = 0;
  fModelCurve = 0;
  fExpeCurve = 0;
	//	SetViewMenu(new TMenuDescr(IDM_DRAWVIEW));
}

QViewGraph::~QViewGraph()
{
		delete fModelCurve;
		if (fExpeCurve)
			delete fExpeCurve;
	}

void QViewGraph::HInitGraph(){
  	// if autoaxis, then recompute the max and min values of both axis
  if ( fAxeX.HIsAutoAxis() || fAxeY.HIsAutoAxis() ) {
		fAxeX.HMaxAxe(HMaxMinX().x); //maximum, convention
		fAxeX.HMinAxe(HMaxMinX().y); //manimum, convention
		fAxeY.HMaxAxe(HMaxMinY().x);
		fAxeY.HMinAxe(HMaxMinY().y);
  }
	fAxeX.HInitQAxe(*this);
	fAxeY.HInitQAxe(*this);
}

void QViewGraph::HTitle(const char *title)
{
	fTitle = title;
}

void QViewGraph::HDisplay(){
	int j,i;
	bool newDC=FALSE;
	PointCurve point;
	if (!dc)	{
		dc = new TClientDC(*this);
		newDC = TRUE;
	}

  	// can not use iterator of fSimulatedData due to fCurve->HGetPoint(i)
	j = fHypoDoc.fHypoModel->fSimulatedData.GetItemsInContainer();
	if ( j ){
		fAxeX.HDisplay(*dc);
		fAxeY.HDisplay(*dc);
		point = fModelCurve->HGetPoint(0);
		dc->MoveTo(fAxeX.HToPixel(point.x), fAxeY.HToPixel(point.y));
		TPen pen(fModelCurve->HColor(), fModelCurve->HWidth(), fModelCurve->HStyle());
		dc->SelectObject(pen);
		for (i=1; i<j; i++){
			point = fModelCurve->HGetPoint(i);
      dc->LineTo(fAxeX.HToPixel(point.x), fAxeY.HToPixel(point.y));
		}
		HDisplayLabel(fModelCurve,0);
	}

  if ( fHypoDoc.fExperimentalData )
		j = fHypoDoc.fExperimentalData->GetItemsInContainer();
  else
  	j = 0;
	if ( fExpeCurve ){
		if ( j ){
			point = fExpeCurve->HGetPoint(0);
			dc->MoveTo(fAxeX.HToPixel(point.x), fAxeY.HToPixel(point.y));
			TPen pen(fExpeCurve->HColor(), fExpeCurve->HWidth(), fExpeCurve->HStyle());
			dc->SelectObject(pen);
			for (i=1; i<j; i++){
				point = fExpeCurve->HGetPoint(i);
				dc->LineTo(fAxeX.HToPixel(point.x), fAxeY.HToPixel(point.y));
			}
		}
		HDisplayLabel(fExpeCurve,1);
	}

	if ( newDC ){
		delete dc;
		dc = 0;
	}
}

void QViewGraph::HDisplayLabel(QCurves* fModelCurve,int i){
	char *fonte="Times New Roman";
	LOGFONT flFont1;
	TFont* fFont1;
	flFont1.lfHeight = fAxeY.HWhiteGap()*0.20;
	flFont1.lfWidth = fAxeY.HWhiteGap()*0.06;
	flFont1.lfEscapement = 0;
	flFont1.lfWeight = FW_NORMAL;
	flFont1.lfItalic = FALSE;
	flFont1.lfUnderline = FALSE;
	flFont1.lfStrikeOut = FALSE;
	flFont1.lfCharSet = DEFAULT_CHARSET;
	flFont1.lfOutPrecision = OUT_DEFAULT_PRECIS;
	flFont1.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	flFont1.lfQuality = DRAFT_QUALITY;
	flFont1.lfPitchAndFamily = DEFAULT_PITCH;
	flFont1.lfFaceName[16] = *fonte;
	fFont1=new TFont(&flFont1);
	dc->SelectObject(*fFont1);
	TPen pen(fModelCurve->HColor(), fModelCurve->HWidth(), fModelCurve->HStyle());
	dc->SelectObject(pen);
	if(i==0){
		dc->TextOut(fAxeY.HWhiteGap()*6, fAxeY.HLowerPixel()*0.4,
    	fModelCurve->HLabel(), strlen(fModelCurve->HLabel()));// simulation
		dc->MoveTo(fAxeY.HWhiteGap()*5.5, fAxeY.HLowerPixel()*0.4);
		dc->LineTo(fAxeY.HWhiteGap()*5.9, fAxeY.HLowerPixel()*0.4);
	}
	if(i==1){
		dc->TextOut(fAxeY.HWhiteGap()*6, fAxeY.HLowerPixel()*0.3,
    	fModelCurve->HLabel(), strlen(fModelCurve->HLabel()));// experiment
		dc->MoveTo(fAxeY.HWhiteGap()*5.5, fAxeY.HLowerPixel()*0.3);
		dc->LineTo(fAxeY.HWhiteGap()*5.9, fAxeY.HLowerPixel()*0.3);
	}
	delete fFont1;
}

void QViewGraph::HCreateThisViewFromDoc(TDocument *doc, char* viewName)
{
	TDocTemplate* tpl;
  for (tpl = doc->GetDocManager().GetTemplateList(); tpl; tpl = tpl->GetNextTemplate()) {
    if (tpl->IsMyKindOfDoc(*doc)) {
      // Find the needed view
      const char far* tplViewName = tpl->GetViewName();
      if ( strstr(tplViewName,viewName)) {
        tpl->InitView(tpl->ConstructView(*doc));
        goto exitLoop;
      }
    }
  }
exitLoop:
}

void QViewGraph::HCreateOnlyOneOfThisViewFromDoc(TDocument *doc, char* viewName)
{
	TDocTemplate* tpl;
  for (tpl = doc->GetDocManager().GetTemplateList(); tpl; tpl = tpl->GetNextTemplate()) {
    if (tpl->IsMyKindOfDoc(*doc)) {
      // Find the existing view
      const char far* tplViewName;
      int found = 0;
      TView *searchView = 0, *view = doc->NextView(0);
      while ( view && ! searchView ) {
        tplViewName = view->GetViewName();
        if ( strstr(tplViewName, viewName)) {
          found = 1;
          searchView = view;
        	break;
        }
        view = doc->NextView(view);
      }
      	// if found an existing view, bring it to top, else create a new view
      if ( found ) {
        searchView->GetWindow()->BringWindowToTop();
        if ( searchView->GetWindow()->IsIconic() )
          searchView->GetWindow()->Show(SW_SHOWNORMAL);
      } else {
      	tpl->InitView(tpl->ConstructView(*doc));
      }
    }
  }
}



bool QViewGraph::VnChangtparam(){
	if(fExpeCurve){
		delete fExpeCurve;
		fExpeCurve=0;
	}
	if(fHypoDoc.fExperimentalData){
		HCreateExperimentalCurve();
	}
	Invalidate();
	HInitGraph();
	HDisplay();
	return true;
}

PointCurve QViewGraph::HMaxMinX(){
	PointCurve exp,cal;
	cal = fModelCurve->HMaxMinX();
	if (fExpeCurve){
		exp = fExpeCurve->HMaxMinX();
		if ( exp.x > cal.x )
    	cal.x = exp.x;
		if ( exp.y < cal.y )
    	cal.y = exp.y;
	}
	return cal;
}

PointCurve QViewGraph::HMaxMinY(){
	PointCurve exp,cal;
	cal = fModelCurve->HMaxMinY();
	if (fExpeCurve){
		exp = fExpeCurve->HMaxMinY();
		if ( exp.x > cal.x )
    	cal.x = exp.x;
		if ( exp.y < cal.y )
    	cal.y = exp.y;
	}
	return cal;
}


void QViewGraph::Paint(TDC&,bool,TRect&){
	HDisplay();
}

void QViewGraph::EvSize(uint,TSize&){
	Invalidate();
	HInitGraph();
}

void QViewGraph::EvLButtonDown(uint modKeys, TPoint& point)
{
    TWindowView::EvLButtonDown(modKeys, point);

  // inside the axis ? Ctrl key forces the graph scale dialog
  if ( modKeys & MK_CONTROL ) {
    if ( QInputGraphDlg(this, &fHypoDoc, this).Execute() == IDOK) {
    	Invalidate();
    }
  } else if ( modKeys & MK_SHIFT ){
    CmInputParameters();
  }
}

void QViewGraph::EvMouseMove(uint modKeys, TPoint& point)
{
    TWindowView::EvMouseMove(modKeys, point);

	char s[80];
	double x, y;
		// convert pixel point to current real unit
  x = fAxeX.HFromPixel(point.x);
  y = fAxeY.HFromPixel(point.y);
	sprintf(s, "(%d,%d, x=%lf, y=%lf)", point.x, point.y, x,y);
  	// get the status bar from the TDecoratedMDIFrame and restore text of status bar
  /*if (fMessageBar)
    	fMessageBar->SetText(s);*/
  TDecoratedMDIFrame* frame = dynamic_cast<TDecoratedMDIFrame*>(GetApplication()->GetMainWindow());
  if ( frame ) {
  TMessageBar*  messageBar = TYPESAFE_DOWNCAST(frame->ChildWithId(IDW_STATUSBAR),
                                                   TMessageBar);
  	if (messageBar)
    	messageBar->SetText(s);
  }

}

void QViewGraph::EvLButtonUp(uint modKeys, TPoint& point)
{
    TWindowView::EvLButtonUp(modKeys, point);

  // INSERT>> Your code here.

}

void QViewGraph::EvRButtonDown(uint modKeys, TPoint& point)
{
    TWindowView::EvRButtonDown(modKeys, point);

  // INSERT>> Your code here.

}

void QViewGraph::EvLButtonDblClk(uint modKeys, TPoint& point)
{
    TWindowView::EvLButtonDblClk(modKeys, point);

  // INSERT>> Your code here.
  CmInputParameters();
}

void QViewGraph::EvRButtonDblClk(uint modKeys, TPoint& point)
{
    TWindowView::EvRButtonDblClk(modKeys, point);

  // INSERT>> Your code here.
  // create a group of related files. first attempt
  if ( fHypoDoc.fHypoModel->deftype == TRIAXIAL_DRAINED ) {
    CmQ_EPS();
    CmEPSV_EPS();
    CmQ_P();
  } else if ( fHypoDoc.fHypoModel->deftype == TRIAXIAL_UNDRAINED ){
    //CmQ_EPS();
    CmU_EPS();
    CmQ_P();
    CmSig1_Sig3v2();
    CmEps1_Eps3v2();
  }
}

void QViewGraph::SetupWindow()
{
  /*fMessageBar = 0;
  TDecoratedMDIFrame* frame = dynamic_cast<TDecoratedMDIFrame*>(GetApplication()->GetMainWindow());
  if ( frame ) {
  	TMessageBar*  fMessageBar = TYPESAFE_DOWNCAST(frame->ChildWithId(IDW_STATUSBAR),
                                                   TMessageBar);
  } */


}



void QViewGraph::CmU_EPS(){
	HCreateThisViewFromDoc(&fHypoDoc,
  	(char*)GetParentO()->GetModule()->LoadString(IDS_GRAPH_TYPE_U_EPS).c_str());
}

void QViewGraph::CmQ_EPS(){
	HCreateThisViewFromDoc(&fHypoDoc,
  	(char*)GetParentO()->GetModule()->LoadString(IDS_GRAPH_TYPE_Q_EPS).c_str());
}

void QViewGraph::CmQ_P(){
	HCreateThisViewFromDoc(&fHypoDoc,
  	(char*)GetParentO()->GetModule()->LoadString(IDS_GRAPH_TYPE_Q_P).c_str());
}

void QViewGraph::CmEPSV_EPS(){
	HCreateThisViewFromDoc(&fHypoDoc,
  	(char*)GetParentO()->GetModule()->LoadString(IDS_GRAPH_TYPE_EPSV_EPS).c_str());
}

void QViewGraph::CmSig1_Sig3v2(){
	HCreateThisViewFromDoc(&fHypoDoc,
  	(char*)GetParentO()->GetModule()->LoadString(IDS_GRAPH_TYPE_SIG1_SIG3v2).c_str());
}

void QViewGraph::CmEps1_Eps3v2(){
	HCreateThisViewFromDoc(&fHypoDoc,
  	(char*)GetParentO()->GetModule()->LoadString(IDS_GRAPH_TYPE_EPS1_EPS3v2).c_str());
}

void QViewGraph::HShowExperimentalData(){
	fHypoDoc.HShowExperimentalData();
}

void QViewGraph::HRemoveExperimentalData(){
	fHypoDoc.HRemoveExperimentalData();
}

void QViewGraph::CeGetExperimentalData(TCommandEnabler& ce){
	ce.Enable(fHypoDoc.fExperimentalDataCopy);
}

void QViewGraph::CeRemoveExperimentalData(TCommandEnabler& ce){
	ce.Enable(fHypoDoc.fExperimentalData);
}

void QViewGraph::CmInputParameters()
{
  // the doc will update others views
  if ( fHypoDoc.HInputParameters(this) == IDOK) {
   Invalidate();
  }

}

void QViewGraph::CeInputParameters(TCommandEnabler &/*tce*/)
{
  // INSERT>> Your code here.

}

void QViewGraph::CmGetGraphAxisScale()
{
  // INSERT>> Your code here.
  if ( QInputGraphDlg(this, &fHypoDoc, this).Execute() == IDOK) {
    Invalidate();
  }

}

void QViewGraph::CeGetGraphAxisScale(TCommandEnabler &/*tce*/)
{
  // INSERT>> Your code here.

}




/*  ***************************************************************
QGraph_q_eps derived class
	(char*)GetParentO()->GetModule()->LoadString(IDS_GRAPH_EXPERIMENT).c_str()
******************************************************************* */

QGraph_q_eps::QGraph_q_eps(QHypoplasticDoc& doc, TWindow* parent):
	QViewGraph(doc, parent)
{
		dc  = 0;
		fExpeCurve = 0;
		fModelCurve = new	QCurves_q_eps(doc.fHypoModel->fSimulatedData);

    	// dynamic cast is needed to access a method of derived class
    ((QCurves_q_eps*)fModelCurve)->HInitQCurves_q_eps();
		if (fHypoDoc.fExperimentalData){
			HCreateExperimentalCurve();
		}
}

void QGraph_q_eps::SetupWindow()
{
  TWindowView::SetupWindow();
  	// Set the necessary texts from stringtable ressource
  TModule *module = GetParentO()->GetModule();
  HTitle(module->LoadString(IDS_GRAPH_Q_EPS).c_str());

  fAxeX.HAxeName(module->LoadString(IDS_SLABEL_EPS).c_str());
  fAxeX.HAxeLabel(module->LoadString(IDS_LABEL_EPS).c_str());
  fAxeX.HUnit(module->LoadString(IDS_UNIT_PERCENT).c_str());

  fAxeY.HAxeName(module->LoadString(IDS_SLABEL_Q).c_str());
  fAxeY.HAxeLabel(module->LoadString(IDS_LABEL_Q).c_str());
  fAxeY.HUnit(module->LoadString(IDS_UNIT_kPa).c_str());

  fModelCurve->HLabel(module->LoadString(IDS_GRAPH_SIMULATION).c_str());
}

void QGraph_q_eps::HCreateExperimentalCurve(){
	fExpeCurve = new QCurves_q_eps(*fHypoDoc.fExperimentalData,
  	"X", "Y", "Experiment",
    TColor::LtRed, 1, PS_SOLID);
}



QGraph_u_eps::QGraph_u_eps(QHypoplasticDoc& doc, TWindow* parent):
	QViewGraph(doc, parent)
{
		dc  = 0;
		fExpeCurve = 0;
		if ( fHypoDoc.fExperimentalData ){
			HCreateExperimentalCurve();
		}
		fModelCurve = new	QCurves_u_eps(doc.fHypoModel->fSimulatedData);

    ((QCurves_u_eps*)fModelCurve)->HInitQCurves_u_eps();
	}

void QGraph_u_eps::SetupWindow()
{
  TWindowView::SetupWindow();
  	// Set the necessary texts from stringtable ressource
  TModule *module = GetParentO()->GetModule();
  HTitle(module->LoadString(IDS_GRAPH_U_EPS).c_str());

  fAxeX.HAxeName(module->LoadString(IDS_SLABEL_EPS).c_str());
  fAxeX.HAxeLabel(module->LoadString(IDS_LABEL_EPS).c_str());
  fAxeX.HUnit(module->LoadString(IDS_UNIT_kPa).c_str());

  fAxeY.HAxeName(module->LoadString(IDS_SLABEL_U).c_str());
  fAxeY.HAxeLabel(module->LoadString(IDS_LABEL_U).c_str());
  fAxeX.HUnit(module->LoadString(IDS_UNIT_PERCENT).c_str());

  fModelCurve->HLabel(module->LoadString(IDS_GRAPH_SIMULATION).c_str());
}

void QGraph_u_eps::HCreateExperimentalCurve(){
	fExpeCurve = new QCurves_u_eps_exp(*fHypoDoc.fExperimentalData,
  	"X", "Y", "Experiment",
    TColor::LtRed, 1, PS_SOLID);
}



QGraph_epsv_eps::QGraph_epsv_eps(QHypoplasticDoc& doc, TWindow* parent):
	QViewGraph(doc, parent)
{
		dc  = 0;
		fExpeCurve = 0;
		if ( fHypoDoc.fExperimentalData ){
			HCreateExperimentalCurve();
		}
		fModelCurve = new	QCurves_epsv_eps(doc.fHypoModel->fSimulatedData);

    ((QCurves_epsv_eps*)fModelCurve)->HInitQCurves_epsv_eps();
}

void QGraph_epsv_eps::SetupWindow()
{
  TWindowView::SetupWindow();
  	// Set the necessary texts from stringtable ressource
  TModule *module = GetParentO()->GetModule();
  HTitle(module->LoadString(IDS_GRAPH_EPSV_EPS).c_str());

  fAxeX.HAxeName(module->LoadString(IDS_SLABEL_EPS).c_str());
  fAxeX.HAxeLabel(module->LoadString(IDS_LABEL_EPS).c_str());
  fAxeX.HUnit(module->LoadString(IDS_UNIT_PERCENT).c_str());

  fAxeY.HAxeName(module->LoadString(IDS_SLABEL_EPSV).c_str());
  fAxeY.HAxeLabel(module->LoadString(IDS_LABEL_EPSV).c_str());
  fAxeX.HUnit(module->LoadString(IDS_UNIT_PERCENT).c_str());

  fModelCurve->HLabel(module->LoadString(IDS_GRAPH_SIMULATION).c_str());
}

void QGraph_epsv_eps::HCreateExperimentalCurve(){
	fExpeCurve = new QCurves_epsv_eps_exp(*fHypoDoc.fExperimentalData,
  	"X", "Y", "Experiment",
    TColor::LtRed, 1, PS_SOLID);
}



QGraph_q_p::QGraph_q_p(QHypoplasticDoc& doc, TWindow* parent):
	QViewGraph(doc, parent)
{
		dc  = 0;
		fExpeCurve = 0;
		fModelCurve = new	QCurves_q_p(doc.fHypoModel->fSimulatedData);

    ((QCurves_q_p*)fModelCurve)->HInitQCurves_q_p();
		if(fHypoDoc.fExperimentalData){
			HCreateExperimentalCurve();
		}
}

void QGraph_q_p::HCreateExperimentalCurve(){
	fExpeCurve = new QCurves_q_p(*fHypoDoc.fExperimentalData,
  	"X", "Y", "Experiment",
    TColor::LtRed, 1, PS_SOLID);
}

void QGraph_q_p::SetupWindow()
{
  TWindowView::SetupWindow();
  	// Set the necessary texts from stringtable ressource
  TModule *module = GetParentO()->GetModule();
  HTitle(module->LoadString(IDS_GRAPH_Q_P).c_str());

  fAxeX.HAxeName(module->LoadString(IDS_SLABEL_P).c_str());
  fAxeX.HAxeLabel(module->LoadString(IDS_LABEL_P).c_str());
  fAxeX.HUnit(module->LoadString(IDS_UNIT_kPa).c_str());

  fAxeY.HAxeName(module->LoadString(IDS_SLABEL_Q).c_str());
  fAxeY.HAxeLabel(module->LoadString(IDS_LABEL_Q).c_str());
  fAxeX.HUnit(module->LoadString(IDS_UNIT_kPa).c_str());

  fModelCurve->HLabel(module->LoadString(IDS_GRAPH_SIMULATION).c_str());
}


QGraph_eps1_eps3v2::QGraph_eps1_eps3v2(QHypoplasticDoc& doc, TWindow* parent):
	QViewGraph(doc, parent)
{
		dc  = 0;
		fExpeCurve = 0;
		fModelCurve = new	QCurves_eps1_eps3v2(doc.fHypoModel->fSimulatedData);

    ((QCurves_eps1_eps3v2*)fModelCurve)->HInitQCurves_eps1_eps3v2();
		if(fHypoDoc.fExperimentalData){
			HCreateExperimentalCurve();
		}
}

void QGraph_eps1_eps3v2::HCreateExperimentalCurve(){
	fExpeCurve = new QCurves_eps1_eps3v2(*fHypoDoc.fExperimentalData,
  	"X", "Y", "Experiment",
    TColor::LtRed, 1, PS_SOLID);
}

void QGraph_eps1_eps3v2::SetupWindow()
{
  TWindowView::SetupWindow();
  	// Set the necessary texts from stringtable ressource
  TModule *module = GetParentO()->GetModule();
  HTitle(module->LoadString(IDS_GRAPH_EPS1_EPS3v2).c_str());

  fAxeX.HAxeName(module->LoadString(IDS_SLABEL_EPS3v2).c_str());
  fAxeX.HAxeLabel(module->LoadString(IDS_LABEL_EPS3v2).c_str());
  fAxeX.HUnit(module->LoadString(IDS_UNIT_PERCENT).c_str());

  fAxeY.HAxeName(module->LoadString(IDS_SLABEL_EPS).c_str());
  fAxeY.HAxeLabel(module->LoadString(IDS_LABEL_EPS).c_str());
  fAxeX.HUnit(module->LoadString(IDS_UNIT_PERCENT).c_str());

  fModelCurve->HLabel(module->LoadString(IDS_GRAPH_SIMULATION).c_str());
}


QGraph_sig1_sig3v2::QGraph_sig1_sig3v2(QHypoplasticDoc& doc, TWindow* parent):
	QViewGraph(doc, parent)
{
		dc  = 0;
		fExpeCurve = 0;
		fModelCurve = new	QCurves_sig1_sig3v2(doc.fHypoModel->fSimulatedData);

    ((QCurves_sig1_sig3v2*)fModelCurve)->HInitQCurves_sig1_sig3v2();
		if(fHypoDoc.fExperimentalData){
			HCreateExperimentalCurve();
		}
}

void QGraph_sig1_sig3v2::HCreateExperimentalCurve(){
	fExpeCurve = new QCurves_sig1_sig3v2(*fHypoDoc.fExperimentalData,
  	"X", "Y", "Experiment",
    TColor::LtRed, 1, PS_SOLID);
}

void QGraph_sig1_sig3v2::SetupWindow()
{
  TWindowView::SetupWindow();
  	// Set the necessary texts from stringtable ressource
  TModule *module = GetParentO()->GetModule();
  HTitle(module->LoadString(IDS_GRAPH_SIG1_SIG3v2).c_str());

  fAxeX.HAxeName(module->LoadString(IDS_SLABEL_SIG3v2).c_str());
  fAxeX.HAxeLabel(module->LoadString(IDS_LABEL_SIG3v2).c_str());
  fAxeX.HUnit(module->LoadString(IDS_UNIT_kPa).c_str());

  fAxeY.HAxeName(module->LoadString(IDS_SLABEL_SIG1).c_str());
  fAxeY.HAxeLabel(module->LoadString(IDS_LABEL_SIG1).c_str());
  fAxeX.HUnit(module->LoadString(IDS_UNIT_kPa).c_str());

  fModelCurve->HLabel(module->LoadString(IDS_GRAPH_SIMULATION).c_str());
}


/********************* EOF ************************************************/
