//----------------------------------------------------------------------------
//  Project hypoplastic model
//  
//  Copyright  1997-1999. All Rights Reserved.
//
//  SUBSYSTEM:    hypo.app Application
//  FILE:         qgraph.h
//  AUTHOR:       Doanh T., Bourgeois C.
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for QViewGraph.
//		QGraph_q_eps, QGraph_u_eps, QGraph_epsv_eps, QGraph_q_p
//    QGraph_eps1_eps3v2, QGraph_sig1_sig3v2
//----------------------------------------------------------------------------
#if !defined(HGRAPH_H)              // Sentry, use file only if it's not already included.
#define HGRAPH_H

#include "OWL_doanh.h"
#pragma hdrstop

#if !defined(HWOLFFER_HYPO_DOC_H)
	#include "wolfferdoc.h"
#endif
#if !defined(HQCURVES_H)
	#include "qcurves.h"
#endif
#if !defined(HQAXE_H)
	#include "qaxes.h"
#endif

class TMessageBar;

/*  ***************************************************************
QViewGraph base class

******************************************************************* */
class _USERCLASS QViewGraph : public TWindowView {
public:
	QViewGraph(QHypoplasticDoc& doc,TWindow* parent = 0,const char *title="Titre");
	virtual ~QViewGraph();

	const char far* StaticName(){return "Generic graph";}
	const char far* GetViewName(){
		return StaticName();
	}

  	// Accessors and mutators for protected data members
  const char* HTitle()    	{ return fTitle.c_str();}
  void  HTitle(const char *title);
  TColor HColor();
	int HWidth();
	int HStyle();

  	// With experiment
	void HShowExperimentalData();
	void HRemoveExperimentalData();
  void CeGetExperimentalData(TCommandEnabler& ce);
  void CeRemoveExperimentalData(TCommandEnabler& ce);

  	// Create a view without using the menu
  static void HCreateThisViewFromDoc(TDocument *doc, char* viewName);
  static void HCreateOnlyOneOfThisViewFromDoc(TDocument *doc, char* viewName);

	void HInitGraph();
	void HDisplay();

	PointCurve HMaxMinX();
	PointCurve HMaxMinY();
	PointCurve HGetPoint(int);
	PointCurve HGetPointexp(int);

	virtual	void HCreateExperimentalCurve()=0;// each view creates the needed experimental curves
	void HDisplayLabel(QCurves*,int);

//protected:
	QCurves*	fModelCurve; 			// results of hypoplastic model
	QCurves* 	fExpeCurve; 			// experimental results
	TDC*		dc;
	QHypoplasticDoc& fHypoDoc;  // associated document
	string 	fTitle; 						// graph title
	QAxeX		fAxeX;
	QAxeY		fAxeY;

  TMessageBar *fMessageBar;

  protected:
  	// virtual method of TWindow
  void SetupWindow();
  
		// Message response functions
	void Paint(TDC&, bool, TRect&);
  void EvLButtonDown(uint modKeys, TPoint& point);
  void EvMouseMove(uint modKeys, TPoint& point);
  void EvLButtonUp(uint modKeys, TPoint& point);
  void EvRButtonDown(uint modKeys, TPoint& point);
  void EvSize(uint sizeType, TSize& size);
  void EvLButtonDblClk(uint modKeys, TPoint& point);
  void EvRButtonDblClk(uint modKeys, TPoint& point);
  	// Commands
  void CmInputParameters();
  void CeInputParameters(TCommandEnabler &tce);
  void CmGetGraphAxisScale();
  void CeGetGraphAxisScale(TCommandEnabler &tce);

  void CmU_EPS();
  void CmQ_EPS();
  void CmQ_P();
  void CmEPSV_EPS();

  void CmSig1_Sig3v2();
  void CmEps1_Eps3v2();

		// Document notifications
	bool VnCommit(bool force){ return TRUE;}
	bool VnRevert(bool clear){ return VnChangtparam();}
	bool VnChangtparam();

	DECLARE_RESPONSE_TABLE(QViewGraph);
};


/*  ***************************************************************
QGraph_q_eps derived class

******************************************************************* */

class _USERCLASS QGraph_q_eps:public QViewGraph{
public:
	QGraph_q_eps(QHypoplasticDoc& doc, TWindow* parent=0);

  virtual void SetupWindow();
	static const char far* StaticName(){
		return "Graph q/eps";
	}
	void HCreateExperimentalCurve();
};

class _USERCLASS QGraph_u_eps:public QViewGraph{
public:
	QGraph_u_eps(QHypoplasticDoc& doc, TWindow* parent=0);

  virtual void SetupWindow();
	static const char far* StaticName(){
		return "Graph U/eps";
	}
	void HCreateExperimentalCurve();
};

class _USERCLASS QGraph_epsv_eps:public QViewGraph{
public:
	QGraph_epsv_eps(QHypoplasticDoc& doc,TWindow* parent=0);

  virtual void SetupWindow();
	static const char far* StaticName(){
		return "Graph epsv/eps";
	}
	void HCreateExperimentalCurve();
};

class _USERCLASS QGraph_q_p:public QViewGraph{
public:
	QGraph_q_p(QHypoplasticDoc& doc,TWindow* parent=0);
  
  virtual void SetupWindow();
	static const char far* StaticName(){
		return "Graph q/p";
	}
	void HCreateExperimentalCurve();
};

class _USERCLASS QGraph_eps1_eps3v2:public QViewGraph{
public:
	QGraph_eps1_eps3v2(QHypoplasticDoc& doc,TWindow* parent=0);
  
  virtual void SetupWindow();
	static const char far* StaticName(){
		return "Graph eps1/eps3 sqrt(2)";
	}
	void HCreateExperimentalCurve();
};

class _USERCLASS QGraph_sig1_sig3v2:public QViewGraph{
public:
	QGraph_sig1_sig3v2(QHypoplasticDoc& doc,TWindow* parent=0);
  
  virtual void SetupWindow();
	static const char far* StaticName(){
		return "Graph sig1/sig3 sqrt(2)";
	}
	void HCreateExperimentalCurve();
};


#endif  //  sentry.

/********************* EOF ************************************************/
