//----------------------------------------------------------------------------
//  Project hypoplastic model
//
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    hypo.app Application
//  FILE:         qparameter.cpp
//  AUTHOR:
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for QMaterialDialog, QInitialStateDialog, QTestDataDialog, QGroupBox.
//
//----------------------------------------------------------------------------

#if !defined(H_HYPO_PARAMETER_H)
	#include "qparameter.h"
#endif

#include "hypoapp.rc"

#include <stdio.h>

#include "OWL_doanh.h"
#pragma hdrstop

#if !defined(OWL_STATIC_H)
	#include <owl/static.h>
#endif
#if !defined(OWL_EDIT_H)
	#include <owl/edit.h>
#endif
#if !defined(OWL_VALIDATE_H)
	#include <owl/validate.h>
#endif
#if !defined(OWL_BUTTON_H)
	#include <owl/button.h>
#endif
#if !defined(OWL_RADIOBUT_H)
	#include <owl/radiobut.h>
#endif


DEFINE_DOC_TEMPLATE_CLASS(QHypoplasticDoc, QMaterialDialog, MaterialDialogTemplate);
DEFINE_DOC_TEMPLATE_CLASS(QHypoplasticDoc, QInitialStateDialog, InitialStateDialogTemplate);
DEFINE_DOC_TEMPLATE_CLASS(QHypoplasticDoc, QTestDataDialog, TestDataDialogTemplate);

MaterialDialogTemplate materialWin("Hypoplastic results (*.CHP)","*.chp",0,"CHP",dtAutoDelete|dtHidden);
InitialStateDialogTemplate initialStateWin("Hypoplastic results (*.CHP)","*.chp",0,"CHP",dtAutoDelete|dtHidden);
TestDataDialogTemplate testDataWin("Hypoplastic results (*.CHP)","*.chp",0,"CHP",dtAutoDelete|dtHidden);


/*  ***************************************************************
QMaterialDialog

******************************************************************* */
DEFINE_RESPONSE_TABLE1(QMaterialDialog,TWindowView)
	EV_WM_SIZE,
	EV_COMMAND(CM_U_EPS, HShowExperimentalData),
	EV_COMMAND(CM_Q_EPS, HRemoveExperimentalData),
	EV_COMMAND(ID_getData,HGetDialogData),
	EV_COMMAND(ID_cancel,HCancel),
END_RESPONSE_TABLE;


void QMaterialDialog::HInitValue(){
	//  attributs de la fenetre
	Attr.Style= WS_CHILD | WS_VISIBLE | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_MINIMIZEBOX | WS_MAXIMIZEBOX;
	SetCaption("Material parameters:");

	TStatic* statik;
	TEdit* edit;
	sprintf(cphi,"%.1f",fHypoDoc.fHypoModel->defphi);
	sprintf(chs,"%.0f",fHypoDoc.fHypoModel->defhs);
	sprintf(calpha,"%.2f",fHypoDoc.fHypoModel->defalpha);
	sprintf(cbeta,"%.2f",fHypoDoc.fHypoModel->defbeta);
	sprintf(ced0,"%.2f",fHypoDoc.fHypoModel->defed0);
	sprintf(cei0,"%.2f",fHypoDoc.fHypoModel->defei0);
	sprintf(cec0,"%.2f",fHypoDoc.fHypoModel->defec0);
	sprintf(cn,"%.2f",fHypoDoc.fHypoModel->defn);
// cration des textes et des cadres de saisie
	statik=new TStatic(this, -1, "Phi()", 20, 20, 80, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_phi,cphi,65,20,38,20,5);
	edit->SetValidator(new TFilterValidator("0-9."));

	statik=new TStatic(this, -1, "Hs(kPa)", 110, 20, 80, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_hs,chs,165,20,38,20,5);
	edit->SetValidator(new TFilterValidator("0-9."));

	statik=new TStatic(this, -1, "n", 45, 50, 80, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_n,cn,65,50,38,20,5);
	edit->SetValidator(new TFilterValidator("0-9."));

	statik=new TStatic(this, -1, "ed0", 130, 50, 80, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_ed0,ced0,165,50,38,20,5);
	edit->SetValidator(new TFilterValidator("0-9."));

	statik=new TStatic(this, -1, "Alpha", 20, 80, 80, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_alpha,calpha,65,80,38,20,5);
	edit->SetValidator(new TFilterValidator("0-9."));

	statik=new TStatic(this, -1, "ec0", 130, 80, 80, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_ec0,cec0,165,80,38,20,5);
	edit->SetValidator(new TFilterValidator("0-9."));

	statik=new TStatic(this, -1, "Beta", 20, 110, 80, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_beta,cbeta,65,110,38,20,5);
	edit->SetValidator(new TFilterValidator("0-9."));

	statik=new TStatic(this, -1, "ei0", 130, 110, 80, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_ei0,cei0,165,110,38,20,5);
	edit->SetValidator(new TFilterValidator("0-9."));

	new TButton(this,ID_getData,"Simulation",40,140,80,25,TRUE);
	new TButton(this,ID_cancel,"Cancel",130,140,80,25);
}

void QMaterialDialog::EvSize(uint,TSize&){
	char text[5];
	char text1[5];
	sprintf(text,"%d",Attr.W);
	sprintf(text1,"%d",Attr.H);
//	MessageBox(text,text1,MB_OK);
}

void QMaterialDialog::HGetDialogData(){
// conversion caractres-> valeurs
	GetDlgItemText(ID_phi, cphi, 5);
	GetDlgItemText(ID_alpha, calpha, 5);
	GetDlgItemText(ID_beta, cbeta, 5);
	GetDlgItemText(ID_ed0, ced0, 5);
	GetDlgItemText(ID_ec0, cec0, 5);
	GetDlgItemText(ID_ei0, cei0, 5);
	GetDlgItemText(ID_n, cn, 5);
	phi=atof(cphi);
	alpha=atof(calpha);
	beta=atof(cbeta);
	n=atof(cn);
	ed0=atof(ced0);
	ec0=atof(cec0);
	ei0=atof(cei0);
	hs=atof(chs);
/*		MessageBox(cphi,"",MB_OK);
		MessageBox(chs,"",MB_OK);
		MessageBox(ced0,"",MB_OK);
		MessageBox(cei0,"",MB_OK);
		MessageBox(cec0,"",MB_OK);
		MessageBox(cn,"",MB_OK);
		MessageBox(calpha,"",MB_OK);
		MessageBox(cbeta,"",MB_OK);    */
// on vrifie la validit des paramtres
	if((ed0>ec0)||(ed0>ei0)||(ec0>ei0)){
		 MessageBox("Relation ed0<ec0<ei0 required", "Errors in material parameters", MB_OK);
	}
	else{
// on remplit la structure parametres materiau et on demande le changement au document
		materiau.phi=phi;
		materiau.ed0=ed0;
		materiau.ec0=ec0;
		materiau.ei0=ei0;
		materiau.n=n;
		materiau.alpha=alpha;
		materiau.beta=beta;
		materiau.hs=hs;
/*		sprintf(cphi,"%.1f",materiau.phi);
		sprintf(chs,"%.0f",materiau.hs);
		sprintf(calpha,"%.1f",materiau.alpha);
		sprintf(cbeta,"%.1f",materiau.beta);
		sprintf(ced0,"%.1f",materiau.ed0);
		sprintf(cei0,"%.1f",materiau.ei0);
		sprintf(cec0,"%.1f",materiau.ec0);
		sprintf(cn,"%.1f",materiau.n);
		MessageBox(cphi,"",MB_OK);
		MessageBox(chs,"",MB_OK);
		MessageBox(ced0,"",MB_OK);
		MessageBox(cei0,"",MB_OK);
		MessageBox(cec0,"",MB_OK);
		MessageBox(cn,"",MB_OK);
		MessageBox(calpha,"",MB_OK);
		MessageBox(cbeta,"",MB_OK);       */
		fHypoDoc.HMaterialChanged(materiau);
	}
}

void QMaterialDialog::HCancel(){
// on revient aux valeurs prcdentes
	SetDlgItemText(ID_phi, cphi);
	SetDlgItemText(ID_alpha, calpha);
	SetDlgItemText(ID_beta, cbeta);
	SetDlgItemText(ID_ed0, ced0);
	SetDlgItemText(ID_ec0, cec0);
	SetDlgItemText(ID_ei0, cei0);
	SetDlgItemText(ID_n, cn);
}

void QMaterialDialog::HShowExperimentalData(){
	fHypoDoc.HShowExperimentalData();
}

void QMaterialDialog::HRemoveExperimentalData(){
	fHypoDoc.HRemoveExperimentalData();
}


/*  ***************************************************************
QInitialStateDialog

******************************************************************* */

DEFINE_RESPONSE_TABLE1(QInitialStateDialog,TWindowView)
	EV_WM_SIZE,
	EV_COMMAND(CM_U_EPS, HShowExperimentalData),
	EV_COMMAND(CM_Q_EPS, HRemoveExperimentalData),
	EV_COMMAND(ID_getData,HGetDialogData),
	EV_COMMAND(ID_cancel,HCancel),
END_RESPONSE_TABLE;


void QInitialStateDialog::HInitValue(){
	//  attributs de la fenetre
	Attr.Style= WS_CHILD | WS_VISIBLE | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_MINIMIZEBOX | WS_MAXIMIZEBOX;
	Attr.W = 270;
	Attr.H = 170;
	SetCaption("Initial state:");

	TStatic* statik;
	TEdit* edit;
	sprintf(csigma1,"%.0f",-1000*fHypoDoc.fHypoModel->defsigma[1]);
	sprintf(csigma3,"%.0f",-1000*fHypoDoc.fHypoModel->defsigma[3]);
	sprintf(ceps1,"%.2f",-fHypoDoc.fHypoModel->defepsilon[1]);
	sprintf(ceps3,"%.2f",-fHypoDoc.fHypoModel->defepsilon[3]);
	sprintf(ce,"%.2f",fHypoDoc.fHypoModel->defe);
// cration des textes et des cadres de saisie
	statik=new TStatic(this, -1, "Stress 1 (kPa)", 10, 20, 100, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_sigma1,csigma1,100,20,38,20,5);
	edit->SetValidator(new TFilterValidator("0-9"));

	statik=new TStatic(this, -1, "Strain 1", 155, 20, 100, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_eps1,ceps1,215,20,38,20,5);
	edit->SetValidator(new TFilterValidator("0-9."));

	statik=new TStatic(this, -1, "Stress 3 (kPa)", 10, 50,100, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_sigma3,csigma3,100,50,38,20,5);
	edit->SetValidator(new TFilterValidator("0-9"));

	statik=new TStatic(this, -1, "Strain 3", 155, 50,100, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_eps3,ceps3,215,50,38,20,5);
	edit->SetValidator(new TFilterValidator("0-9."));

	statik=new TStatic(this, -1, "Void ratio", 40, 80,150, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_e,ce,155,80,38,20,5);
	edit->SetValidator(new TFilterValidator("0-9."));

	new TButton(this,ID_getData,"Simulation",60,110,80,25,TRUE);
	new TButton(this,ID_cancel,"Cancel",150,110,80,25);
}

void QInitialStateDialog::EvSize(uint,TSize&){
}

void QInitialStateDialog::HGetDialogData(){
// conversion caractres-> valeurs
	GetDlgItemText(ID_sigma1, csigma1,5);
	GetDlgItemText(ID_sigma3, csigma3,5);
	GetDlgItemText(ID_eps1, ceps1,5);
	GetDlgItemText(ID_eps3, ceps3,5);
	GetDlgItemText(ID_e, ce,5);
	sigma1=-0.001*atof(csigma1);
	sigma3=-0.001*atof(csigma3);
	eps1=-atof(ceps1);
	eps3=-atof(ceps3);
	e=atof(ce);
/*		MessageBox(cphi,"",MB_OK);
		MessageBox(chs,"",MB_OK);
		MessageBox(ced0,"",MB_OK);
		MessageBox(cei0,"",MB_OK);
		MessageBox(cec0,"",MB_OK);
		MessageBox(cn,"",MB_OK);
		MessageBox(calpha,"",MB_OK);
		MessageBox(cbeta,"",MB_OK);    */
// on remplit la structure parametres materiau et on demande le changement au document
	initialState.sigma1=sigma1;
	initialState.sigma3=sigma3;
	initialState.epsilon1=eps1;
	initialState.epsilon3=eps3;
	initialState.e=e;
/*		sprintf(cphi,"%.1f",materiau.phi);
		sprintf(chs,"%.0f",materiau.hs);
		sprintf(calpha,"%.1f",materiau.alpha);
		sprintf(cbeta,"%.1f",materiau.beta);
		sprintf(ced0,"%.1f",materiau.ed0);
		sprintf(cei0,"%.1f",materiau.ei0);
		sprintf(cec0,"%.1f",materiau.ec0);
		sprintf(cn,"%.1f",materiau.n);
		MessageBox(cphi,"",MB_OK);
		MessageBox(chs,"",MB_OK);
		MessageBox(ced0,"",MB_OK);
		MessageBox(cei0,"",MB_OK);
		MessageBox(cec0,"",MB_OK);
		MessageBox(cn,"",MB_OK);
		MessageBox(calpha,"",MB_OK);
		MessageBox(cbeta,"",MB_OK);       */
  if((fHypoDoc.fHypoModel->fMaterialParameter.ed0>e)||(e>fHypoDoc.fHypoModel->fMaterialParameter.ei0)){
		 MessageBox("Relation ed0 < e < ei0 required", "Errors in material parameters", MB_OK);
	} else {
		fHypoDoc.HInitialStateChanged(initialState);
  }
}

void QInitialStateDialog::HCancel(){
// on revient aux valeurs prcdentes
	SetDlgItemText(ID_sigma1, csigma1);
	SetDlgItemText(ID_sigma3, csigma3);
	SetDlgItemText(ID_eps1, ceps1);
	SetDlgItemText(ID_eps3, ceps3);
	SetDlgItemText(ID_e, ce);
}

void QInitialStateDialog::HShowExperimentalData(){
	fHypoDoc.HShowExperimentalData();
}

void QInitialStateDialog::HRemoveExperimentalData(){
	fHypoDoc.HRemoveExperimentalData();
}


/*  ***************************************************************
QGroupBox, QTestDataDialog

******************************************************************* */

void QGroupBox::SelectionChanged(int controlId){
  switch (controlId) {
  case ID_triaxial:
	 choix = "t";
	 break;
  case ID_undrained:
	 choix = "u";
	 break;
  case ID_isotrope:
	 choix = "i";
	 break;
  case ID_oedometre:
	 choix = "o";
	 break;
  case ID_compression:
	 choix = "1";
	 break;
  case ID_extension:
	 choix = "2";
	 break;
  };
}



DEFINE_RESPONSE_TABLE1(QTestDataDialog,TWindowView)
	EV_WM_SIZE,
	EV_COMMAND(CM_U_EPS, HShowExperimentalData),
	EV_COMMAND(CM_Q_EPS, HRemoveExperimentalData),
	EV_COMMAND(ID_getData,HGetDialogData),
	EV_COMMAND(ID_cancel,HCancel),
END_RESPONSE_TABLE;


void QTestDataDialog::HInitValue(){
	//  attributs de la fenetre
	Attr.Style= WS_CHILD | WS_VISIBLE | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_MINIMIZEBOX | WS_MAXIMIZEBOX;
	TStatic* statik;
	TEdit* edit;

	Attr.W = 380;
	Attr.H = 260;
	sprintf(cdt,"%.4f",fHypoDoc.fHypoModel->defdt);
	sprintf(csigmamax,"%.0f",-1000*fHypoDoc.fHypoModel->defsigmamax);
	sprintf(cepsmax,"%.2f",-fHypoDoc.fHypoModel->defepsilonmax);
	typeGroup = new QGroupBox(this, ID_typeessai, "Test type:",20,20,150,105);
	new TRadioButton(this, ID_triaxial, "Triaxial", 30, 40, 110, 17, typeGroup);
	new TRadioButton(this, ID_undrained, "Undrained", 30, 60, 110, 17, typeGroup);
	new TRadioButton(this, ID_isotrope, "Isotrope", 30, 80, 110, 17, typeGroup);
	new TRadioButton(this, ID_oedometre, "Oedometer", 30, 100, 110, 17, typeGroup);
	directionGroup = new QGroupBox(this, ID_direction, "Direction", 189, 20, 150, 75);
	new TRadioButton(this, ID_compression, "Compression", 200, 40, 110, 17, directionGroup);
	new TRadioButton(this, ID_extension, "Extension", 200, 65, 110, 17, directionGroup);

	statik=new TStatic(this, -1, "Time step: dt", 180, 100,150, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_dt,cdt,315,100,50,20,7);
	edit->SetValidator(new TFilterValidator("0-9."));

	statik=new TStatic(this, -1, "Max stress 1 (kPa)", 190, 130,150, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_sigmamax,csigmamax,315,130,40,20,5);
	edit->SetValidator(new TFilterValidator("0-9"));

	statik=new TStatic(this, -1, "Max strain 1", 20, 130,150, 20,5);
	statik->Attr.Style = (statik->Attr.Style) | WS_VISIBLE | SS_LEFT;
	edit=new TEdit(this,ID_epsmax,cepsmax,120,130,40,20,5);
	edit->SetValidator(new TFilterValidator("0-9."));

	new TButton(this,ID_getData,"Simulation",100,175,80,25,TRUE);
	new TButton(this,ID_cancel,"Cancel",190,175,80,25);
}

void QTestDataDialog::EvSize(uint,TSize&){
}

void QTestDataDialog::HGetDialogData(){
// conversion caractres-> valeurs
	string zero("0");   // si rien n'est choisi.
	if((!typeGroup->choix.is_null()) && (!directionGroup->choix.is_null())){
		*type=(typeGroup->choix.is_null() ? zero :typeGroup->choix)[0];
		*dir=(directionGroup->choix.is_null() ? zero :directionGroup->choix)[0];
		if(((type[0]=='u')||(type[0]=='i')||(type[0]=='o')||(type[0]=='t'))
			&&((dir[0]=='1')||(dir[0]=='2'))){
			GetDlgItemText(ID_sigmamax, csigmamax,5);
			GetDlgItemText(ID_epsmax, cepsmax,5);
			GetDlgItemText(ID_dt, cdt,7);
			if(dir[0]=='1'){
				epsmax=-atof(cepsmax);
				sigmamax=-0.001*atof(csigmamax);
			}
			if(dir[0]=='2'){         // si c'est un essai d'extension
				epsmax=atof(cepsmax); // on garde une valeur positive.
				sigmamax=0.001*atof(csigmamax);
			}
			dt=atof(cdt);
// on remplit la structure parametres materiau et on demande le changement au document
			fTestParameter.test_type=type[0];
			fTestParameter.direction=dir[0];
			fTestParameter.dt=dt;
			fTestParameter.sigmamax=sigmamax;
			fTestParameter.epsilonmax=epsmax;
			fHypoDoc.HTestChanged(fTestParameter);
		}
	}
	else{
		MessageBox("Missing parameters !","",MB_OK);
	}
}

void QTestDataDialog::HCancel(){
// on revient aux valeurs prcdentes
	SetDlgItemText(ID_sigmamax, csigmamax);
	SetDlgItemText(ID_epsmax, cepsmax);
	SetDlgItemText(ID_dt, cdt);
}

void QTestDataDialog::HShowExperimentalData(){
	fHypoDoc.HShowExperimentalData();
}

void QTestDataDialog::HRemoveExperimentalData(){
	fHypoDoc.HRemoveExperimentalData();
}

/********************* EOF ************************************************/
