//----------------------------------------------------------------------------
//  Project hypoplastic model
//  
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    hypo.app Application
//  FILE:         qparameter.h
//  AUTHOR:
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for QMaterialDialog, QInitialStateDialog, QTestDataDialog, QGroupBox.
//
//----------------------------------------------------------------------------
#if !defined(H_HYPO_PARAMETER_H)              // Sentry, use file only if it's not already included.
#define H_HYPO_PARAMETER_H

#include "OWL_doanh.h"
#pragma hdrstop

#if !defined(OWL_DOCVIEW_H)
	#include <owl/docview.h>
#endif
#if !defined(OWL_GROUPBOX_H)
	#include <owl/groupbox.h>
#endif

#if !defined(HWOLFFER_HYPO_DOC_H)
	#include "wolfferdoc.h"
#endif

/*  ***************************************************************
QMaterialDialog

******************************************************************* */

const int ID_phi 			= 101;
const int ID_hs 			= 102;
const int ID_n 				= 103;
const int ID_alpha 		= 104;
const int ID_beta 		= 105;
const int ID_ed0 			= 106;
const int ID_ec0 			= 107;
const int ID_ei0 			= 108;
const int ID_getData 	= 109;
const int ID_cancel 	= 110;

class QMaterialDialog : public TWindowView {
public:
	QMaterialDialog::QMaterialDialog(QHypoplasticDoc& doc,TWindow* parent = 0):
	TWindowView(doc, parent),fHypoDoc(doc){
		HInitValue();
	}

	static const char far* StaticName(){return "Material parameters";}
	const char far* GetViewName(){
		return StaticName();
	}

protected:
	void HInitValue();
	void EvSize(UINT,TSize&);
	void HGetDialogData();
	void HCancel();
	void HRemoveExperimentalData();
	void HShowExperimentalData();

	QHypoplasticDoc& fHypoDoc;
	DECLARE_RESPONSE_TABLE(QMaterialDialog);
	char cphi[5];
	char chs[5];
	char cn[5];
	char calpha[5];
	char cbeta[5];
	char ced0[5];
	char cec0[5];
	char cei0[5];
	float phi,hs,n,ed0,ec0,ei0,alpha,beta;
	HMaterialParameter materiau;
};


/*  ***************************************************************
QInitialStateDialog

******************************************************************* */
const int ID_sigma1 			= 121;
const int ID_sigma3 			= 122;
const int ID_eps1 				= 123;
const int ID_eps3 				= 124;
const int ID_e 						= 125;

class QInitialStateDialog : public TWindowView {
public:
	QInitialStateDialog::QInitialStateDialog(QHypoplasticDoc& doc,TWindow* parent = 0):
	TWindowView(doc, parent),fHypoDoc(doc){
		HInitValue();
	}

	static const char far* StaticName(){return "Initial state";}
	const char far* GetViewName(){
		return StaticName();
	}

protected:
	void HInitValue();
	void EvSize(UINT,TSize&);
	void HGetDialogData();
	void HCancel();
	void HRemoveExperimentalData();
	void HShowExperimentalData();

	QHypoplasticDoc& fHypoDoc;
	DECLARE_RESPONSE_TABLE(QInitialStateDialog);
	char csigma1[5];
	char csigma3[5];
	char ceps1[5];
	char ceps3[5];
	char ce[5];
	float sigma1,sigma3,eps1,eps3,e;
	HHypoResults initialState;
};


/*  ***************************************************************
QGroupBox, QTestDataDialog

******************************************************************* */

const int ID_triaxial     = 131;
const int ID_undrained    = 132;
const int ID_isotrope     = 133;
const int ID_oedometre    = 134;
const int ID_compression  = 135;
const int ID_extension    = 136;
const int ID_dt           = 137;
const int ID_sigmamax     = 138;
const int ID_epsmax       = 139;
const int ID_typeessai	  = 142;
const int ID_direction	  = 143;

class QGroupBox : public TGroupBox {
public:
	 QGroupBox(TWindow* parent,int id,const char far* text,int X, int Y,
	  int W, int H,TModule* module = 0):TGroupBox(parent, id, text, X, Y, W, H, module){}

	 void SelectionChanged(int controlId);
	 string choix;// choix de la selection
};

class QTestDataDialog : public TWindowView {
public:
	QTestDataDialog(QHypoplasticDoc& doc,TWindow* parent = 0):
	TWindowView(doc, parent),fHypoDoc(doc){
		HInitValue();
	}

	~QTestDataDialog(){
	}

	static const char far* StaticName(){return "Test parameters";}
	const char far* GetViewName(){
		return StaticName();
	}

	Saisieessai(TWindow* parent, const char* title);

	QGroupBox* typeGroup;
	QGroupBox* directionGroup;

protected:
	void HInitValue();
	void EvSize(UINT,TSize&);
	void HGetDialogData();
	void HCancel();
	void HRemoveExperimentalData();
	void HShowExperimentalData();

	QHypoplasticDoc& fHypoDoc;
	char cdt[7];
	char csigmamax[5];
	char cepsmax[5];
	char type[1];
	char dir[1];
	float sigmamax,epsmax,dt;
	HTestParameter fTestParameter;

	DECLARE_RESPONSE_TABLE(QTestDataDialog);
};


#endif  //  sentry.
/********************* EOF ************************************************/
