//----------------------------------------------------------------------------
//  Project hypoplastic model
//  
//  Copyright  1997. All Rights Reserved.
//
//  SUBSYSTEM:    hypo.app Application
//  FILE:         qviewlist.cpp
//  AUTHOR:
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for QViewList and ...
//
//----------------------------------------------------------------------------


#if !defined(QHVIEWLIST_H)
	#include "qviewlist.h"
#endif

#include "hypoapp.rc"

#include <iostream.h>
#include <iomanip.h>   //Requiredforparameterizedmanipulators.

	// internal temporary structure
struct OutNumber {
 	short width;
  short precision;
  char fill;
};
	// manipulator with struct OutNumber
ios& _iomanip_sw(ios& os, OutNumber r)
{
 	os.fill(r.fill);
  os.width(r.width);
  os.precision(r.precision);
  os.setf(ios::fixed, ios::floatfield);
  return os;
}

smanip<OutNumber> sw(short width, short precision, char fill)
{
 	OutNumber tmp;
  tmp.fill = fill;
  tmp.width = width;
  tmp.precision = precision;
  return smanip<OutNumber>(_iomanip_sw, tmp);
}



DEFINE_RESPONSE_TABLE1(QViewList, TListBox)
	EV_VN_CHANGTPARAMETRES,
	EV_COMMAND(CM_U_EPS, HShowExperimentalData),
	EV_COMMAND(CM_Q_EPS, HRemoveExperimentalData),
	EV_VN_ISWINDOW,
//	EV_VN_COMMIT,
//	EV_VN_REVERT,
END_RESPONSE_TABLE;

QViewList::QViewList(QHypoplasticDoc& doc,TWindow* parent) :
	TView(doc), TListBox(parent, GetNextViewId(), 0,0,0,0),fHypoDoc(doc){
	Attr.Style &= ~(WS_BORDER | LBS_SORT);
	Attr.Style |= LBS_NOINTEGRALHEIGHT;
//	Attr.AccelTable = IDA_DRAWLISTVIEW;
//	SetViewMenu(new TMenuDescr(IDM_DRAWLISTVIEW,0,1,0,0,0,0));
}

bool QViewList::Create(){
	TListBox::Create();
	HGetResults();
	return true;
}

bool QViewList::VnChangtparam(){
	Invalidate();
	HGetResults();
	return true;
}

void QViewList::HShowExperimentalData(){
	fHypoDoc.HShowExperimentalData();
}

void QViewList::HRemoveExperimentalData(){
	fHypoDoc.HRemoveExperimentalData();
}

void QViewList::HGetResults(){
	ClearList();
	int i,j = fHypoDoc.fHypoModel->fSimulatedData.GetItemsInContainer();
	for(i=0;i<j;i++){
		HDisplayResults(fHypoDoc.fHypoModel->fSimulatedData[i],i,0);
	}
	int k=i; // and experiment results now
	if(fHypoDoc.fExperimentalData){
		char *buf="  Experiment:";
		AddString(buf);
		int j=fHypoDoc.fExperimentalData->GetItemsInContainer();
		for(int i=0;i<j;i++){
			HDisplayResults((*fHypoDoc.fExperimentalData)[i],i,k+1);
		}
		SetSelIndex(0);
	}
}

void QViewList::HDisplayResults(HHypoResults& fHypoResults,int i,int /*indexFirst*/){
	char buf[255];
  ostrstream os(buf, sizeof(buf));
  if ( i == 0 ) {
    os.seekp(0) << " Index     Sigma1      Sigma2      Sigma3      Epsilon1      Epsilon2      Epsilon3     e" << ends;
   	AddString(buf);
  } else {
  	os.seekp(0) << sw(11,3,' ') << i
   				<< sw(11,3,' ') << fHypoResults.sigma1
               << sw(11,3,' ') << fHypoResults.sigma2
    				<< sw(11,3,' ') << fHypoResults.sigma3
    				<< sw(11,5,' ') << fHypoResults.epsilon1
               << sw(11,5,' ') << fHypoResults.epsilon2
               << sw(11,5,' ') << fHypoResults.epsilon3
               << sw(11,3,' ') << fHypoResults.e << ends;
   	AddString(buf);
  }
}

void QViewList_q_eps::HDisplayResults(HHypoResults& fHypoResults,int i,int indexFirst){
	char buf[255];
  ostrstream os(buf, sizeof(buf));
  if ( i == 0 ) {
    os.seekp(0) << " Index,     Deviatoric stress,      Epsilon1 " << ends;
   	AddString(buf);
  } else {
  	os.seekp(0) << sw(11,3,' ') << i
                << sw(11,3,' ') << (fHypoResults.sigma1-fHypoResults.sigma3)
    						<< sw(11,3,' ') << fHypoResults.epsilon1 << ends;
   	AddString(buf);
  }
	DeleteString(i+indexFirst);
	InsertString(buf, i+indexFirst);
	SetSelIndex(i+indexFirst);
}

void QViewList_u_eps::HDisplayResults(HHypoResults& fHypoResults,int i,int indexFirst){
	char buf[255];
  ostrstream os(buf, sizeof(buf));
  if ( i == 0 ) {
    os.seekp(0) << " Index,     Pore pressure,      Epsilon1 " << ends;
   	AddString(buf);
  } else {
  	os.seekp(0) << sw(11,3,' ') << i
                << sw(11,3,' ') << (fHypoDoc.fHypoModel->fSimulatedData[0].sigma3-fHypoResults.sigma3)
    						<< sw(11,3,' ') << fHypoResults.epsilon1 << ends;
   	AddString(buf);
  }
	DeleteString(i+indexFirst);
	InsertString(buf, i+indexFirst);
	SetSelIndex(i+indexFirst);
}

void QViewList_epsv_eps::HDisplayResults(HHypoResults& fHypoResults,int i,int indexFirst){
	char buf[255];
  ostrstream os(buf, sizeof(buf));
  if ( i == 0 ) {
    os.seekp(0) << " Index,     Volumetric strain,      Epsilon1 " << ends;
   	AddString(buf);
  } else {
  	os.seekp(0) << sw(11,3,' ') << i
                << sw(11,3,' ') << (fHypoResults.epsilon1+2*fHypoResults.epsilon3)
    						<< sw(11,3,' ') << fHypoResults.epsilon1 << ends;
   	AddString(buf);
  }
	DeleteString(i+indexFirst);
	InsertString(buf, i+indexFirst);
	SetSelIndex(i+indexFirst);
}

void QViewList_q_p::HDisplayResults(HHypoResults& fHypoResults,int i,int indexFirst){
	char buf[255];
  ostrstream os(buf, sizeof(buf));
  if ( i == 0 ) {
    os.seekp(0) << " Index,     Deviatoric stress,      Mean effective pressure " << ends;
   	AddString(buf);
  } else {
  	os.seekp(0) << sw(11,3,' ') << i
                << sw(11,3,' ') << (fHypoResults.sigma1-fHypoResults.sigma3)
    						<< sw(11,3,' ') << ((fHypoResults.sigma1+2*fHypoResults.sigma3)*0.5) << ends;
   	AddString(buf);
  }
	DeleteString(i+indexFirst);
	InsertString(buf, i+indexFirst);
	SetSelIndex(i+indexFirst);
}

/********************* EOF ************************************************/
