//----------------------------------------------------------------------------
//  Project hypoplastic model
//
//  Copyright  1997-1999. All Rights Reserved.
//
//  SUBSYSTEM:    hypo.app Application
//  FILE:         wolfferdoc.cpp
//  AUTHOR:       Doanh T. based on hyptest.c of I. Herle
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for QWolffer_hypoplasticity (TFileDocument).
//
//----------------------------------------------------------------------------


#if !defined(HWOLFFER_HYPO_DOC_H)
	#include "wolfferdoc.h"
#endif
#if !defined(HQ_INPUT_PARAMETER_H)
	#include "QInputParam.h"
#endif
#if !defined(TWAITCURSOR_H)
	#include "TWaitCursor.h"
#endif

#include <iostream.h>
#include <owl/editfile.rh>

#define WANT_STREAM                  // include.h will get stream fns
#define WANT_MATH                    // include.h will get math fns
                                     // newmatap.h will get include.h

#include "newmat.h"                	 // need matrix package

#include "newmatio.h"                // need matrix output routines

#include "newmatap.h"                // need matrix SVD routines



/*  ***************************************************************
hypoplastic document
******************************************************************* */

QHypoplasticDoc::QHypoplasticDoc(TDocument* parent)
	: TFileDocument(parent){

  //QWolffersdorffModel, QWolffersdorffExtendedModel, QWeiWuModel,
  //QKolymbasModel, QWolffersdorff4ParamModel
  fHypoModel = new QWolffersdorffModel;

  fHypoModel->HInitValues();
  fHypoModel->HInitSimulation();
  fHypoModel->HSimulation();
  fExperimentalData = fExperimentalDataCopy = 0;
}

QHypoplasticDoc::~QHypoplasticDoc(){
  if ( fExperimentalData )
			delete fExperimentalData;
  if ( fHypoModel )
    	delete fHypoModel;
}

bool QHypoplasticDoc::Open(int /*mode*/, const char far* path)
{
int error;

	if (path)
		SetDocPath(path);
	if (GetDocPath()) {
		TInStream* is = InStream(ofRead);
		if (!is)
			return false;

    	// identify .DOC or .CHP
    char filePath[_MAX_PATH];
    strcpy(filePath,GetDocPath());
    if ( strstr(filePath,".doc") || strstr(filePath,".DOC") ) {
  		error = HGetTestDataFromStream(*is);
  	} else {
    	error = HGetDocDataFromStream(*is);
    }

		delete is;
    if ( !error ) {
			fHypoModel->HInitSimulation();
			fHypoModel->HSimulation();
			NotifyViews(vnParametersChanged);
    }
	}
	return true;
}

bool QHypoplasticDoc::Commit(bool /*force*/){
	TOutStream* os=OutStream(ofWrite);
	if(!os)
		return FALSE;
  TApplication *app = GetDocManager().GetApplication();
  	// save header
	*os << app->LoadString(IDS_DOC_HYPOPLASTIC) << endl;
   *os << app->LoadString(IDS_DOC_VERSION) << endl;
	*os << app->LoadString(IDS_DOC_MATERIAL_PARAM) << endl;
	*os << app->LoadString(IDS_DOC_MODEL_PHI) << fHypoModel->defphi << endl;
	*os << app->LoadString(IDS_DOC_MODEL_HS) << fHypoModel->defhs << endl;
	*os << app->LoadString(IDS_DOC_MODEL_N) << fHypoModel->defn << endl;
	*os << app->LoadString(IDS_DOC_MODEL_EDO) << fHypoModel->defed0 << endl;
	*os << app->LoadString(IDS_DOC_MODEL_ECO) << fHypoModel->defec0 << endl;
	*os << app->LoadString(IDS_DOC_MODEL_EIO) << fHypoModel->defei0 << endl;
	*os << app->LoadString(IDS_DOC_MODEL_ALPHA) << fHypoModel->defalpha << endl;
	*os << app->LoadString(IDS_DOC_MODEL_BETA) << fHypoModel->defbeta << endl;
	*os << app->LoadString(IDS_DOC_MODEL_TEST_PARAM) << endl;
	*os << app->LoadString(IDS_DOC_MODEL_TEST_TYPE) << fHypoModel->deftype << endl;
	*os << app->LoadString(IDS_DOC_MODEL_TEST_DIRECTION) << fHypoModel->defdir << endl;
   *os << app->LoadString(IDS_DOC_MODEL_TEST_INC_ANGLE) << fHypoModel->defdIncAngle << endl;
	*os << app->LoadString(IDS_DOC_MODEL_MAX_STRESS) << fHypoModel->defsigmamax << endl;
	*os << app->LoadString(IDS_DOC_MODEL_MAX_STRAIN) << fHypoModel->defepsilonmax << endl;
	*os << app->LoadString(IDS_DOC_MODEL_TIMESTEP) << fHypoModel->defdt << endl;
	*os << app->LoadString(IDS_DOC_MODEL_SIMU_RESULTS) << endl;
  	// save simulated data
	int j = fHypoModel->fSimulatedData.GetItemsInContainer();
	*os << app->LoadString(IDS_DOC_MODEL_NUMBER_DATA)<< j << endl;
	*os << app->LoadString(IDS_DOC_MODEL_LINE_RESULTS) << endl;

	for(int i=0; i<j; i++)
		*os << fHypoModel->fSimulatedData[i];
	delete os;
	SetDirty(FALSE);

	return TRUE;
}

void QHypoplasticDoc::HShowExperimentalData(){
	if ( fExperimentalDataCopy ) {
  	fExperimentalData = fExperimentalDataCopy;
    fExperimentalDataCopy = 0;
    NotifyViews(vnParametersChanged);
  } else {
    ::MessageBox(NULL, "Open an experimental data file (DOC extension)",
    	GetDocManager().GetApplication()->LoadString(IDS_ER_READ_EXPERIMENT_FILE).c_str(),
      MB_ICONEXCLAMATION|MB_OK);
  }
}

void QHypoplasticDoc::HRemoveExperimentalData()
{
	if ( fExperimentalData ){
		fExperimentalDataCopy = fExperimentalData;
		fExperimentalData = 0;
		NotifyViews(vnParametersChanged);
	}
}

int QHypoplasticDoc::HGetTestDataFromStream(istream& is)
{
	char c, line[_MAX_PATH];
	if (fExperimentalData)
  	delete fExperimentalData;
	fExperimentalData = new HHypoResultsVector(10,0,10);
	HHypoResults res;
	int i, maxLine, lineLength, colums, numberData;
  is >> maxLine >> c >> lineLength >>  c >> colums >>  c >> numberData;
	for( i=1; i<maxLine; i++){// don't take the header
		if(!is.eof()){
			is.get(line,_MAX_PATH,'\n');
			is.ignore();
		}
	}
  	// NOTE: res.e contains measured volumetric strain
    // res.sigma2 contains pore pressure
  while(!is.eof()){
    is >> i >>  res.sigma1 >> res.sigma3
    	 >> res.epsilon1 >> res.epsilon3 >>  res.e >> res.sigma2;
    is.get(line,_MAX_PATH,'\n');
		is.ignore();
		fExperimentalData->Add(res);
	}
  return 0; // no error
}

int QHypoplasticDoc::HGetDocDataFromStream(istream& is)
{
  char c, line[_MAX_PATH];
	fHypoModel->fSimulatedData.Flush();

    // "Hypoplasticity, model of P.A. von Wolffersdorff" >> "\n "
    is.get(line,_MAX_PATH,'\n'); is.ignore();
    // "Document version:" >> "\n "
    is.get(line,_MAX_PATH,'\n'); is.ignore();

    	// Check the document version
    if ( !strstr(line, "2") ) {
    	::MessageBox(NULL, "Error in HGetDocDataFromStream()",
    		GetDocManager().GetApplication()->LoadString(IDS_ER_DOC_VERSION).c_str(),
         MB_ICONEXCLAMATION|MB_OK);
      return -4;
    }
    // "Material parameters:" >> "\n "
    is.get(line,_MAX_PATH,'\n'); is.ignore();
    // "Phi: ">> defphi>>"\n ";
    is.get(line,_MAX_PATH,':');
    is >> c >> ws >> fHypoModel->defphi; is.ignore();
		// "Hs: ">> defhs>>"\n ";
    is.get(line,_MAX_PATH,':');
    is >> c >> ws >> fHypoModel->defhs; is.ignore();
		// "n: ">> defn>>"\n ";
    is.get(line,_MAX_PATH,':');
    is >> c >> ws >> fHypoModel->defn; is.ignore();
		// "ed0: ">> defed0>>"\n ";
    is.get(line,_MAX_PATH,':');
    is >> c >> ws >> fHypoModel->defed0; is.ignore();
		// "ec0: ">> defec0>>"\n ";
    is.get(line,_MAX_PATH,':');
    is >> c >> ws >> fHypoModel->defec0; is.ignore();
		// "ei0: ">> defei0>>"\n ";
    is.get(line,_MAX_PATH,':');
    is >> c >> ws >> fHypoModel->defei0; is.ignore();
		// "alpha: ">> defalpha>>"\n ";
    is.get(line,_MAX_PATH,':');
    is >> c >> ws >> fHypoModel->defalpha; is.ignore();
		// "beta: ">> defbeta>>"\n ";
    is.get(line,_MAX_PATH,':');
    is >> c >> ws >> fHypoModel->defbeta; is.ignore();
		// "Test parameters:">>"\n ";
    is.get(line,_MAX_PATH,'\n'); is.ignore();
		// "Test type: ">>deftype >> "\n ";
    is.get(line,_MAX_PATH,':'); // cast unamed enum to int to avoid warning
    is >> c >> ws >> (int)fHypoModel->deftype; is.ignore();
		// "Direction: ">>defdir>>"\n ";
    is.get(line,_MAX_PATH,':');
    is >> c >> ws >> (int)fHypoModel->defdir; is.ignore();
    // "Increment angle: ">>defdir>>"\n ";
    is.get(line,_MAX_PATH,':');
    is >> c >> ws >> fHypoModel->defdIncAngle; is.ignore();
		// "Maximum stress: ">>defsigmamax>>"\n ";
    is.get(line,_MAX_PATH,':');
    is >> c >> ws >> fHypoModel->defsigmamax; is.ignore();
		// "Maximum strain: ">>defepsilonmax>>"\n ";
    is.get(line,_MAX_PATH,':');
    is >> c >> ws >> fHypoModel->defepsilonmax; is.ignore();
		// "Time step: ">> defdt>>"\n ";
    is.get(line,_MAX_PATH,':');
    is >> c >> ws >> fHypoModel->defdt; is.ignore();
		// "Simulation results:">>"\n ";
    is.get(line,_MAX_PATH,'\n'); is.ignore();

    is.get(line,_MAX_PATH,':');
    unsigned numberOfData;
    is >> c >> ws >> numberOfData;
    is.ignore();
			// "Sigma1 Sigma2 Sigma3 Epsilon1 Epsilon2 Epsilon3 Void ratio">>"\n "
    is.get(line,_MAX_PATH,'\n'); is.ignore();
		HHypoResults res;
		for(unsigned i=0; i<numberOfData; i++){
			is >> res;
			fHypoModel->fSimulatedData.Add(res);
      	// set the initial state
      	if ( i == 0 ) {
        	fHypoModel->defsigma[1] = -0.001*res.sigma1;  // MPa in HypoModel
				fHypoModel->defsigma[2] = -0.001*res.sigma3;
				fHypoModel->defsigma[3] = -0.001*res.sigma3;
				fHypoModel->defepsilon[1] = res.epsilon1;
				fHypoModel->defepsilon[2] = res.epsilon3;
				fHypoModel->defepsilon[3] = res.epsilon3;
				fHypoModel->defe = res.e;
      	}
		}
    return 0;
}

void QHypoplasticDoc::HMaterialChanged(HMaterialParameter& material){
  	// get the values
	fHypoModel->HMaterialChanged(material);
	NotifyViews(vnParametersChanged);
	SetDirty(true);
}

void QHypoplasticDoc::HTestChanged(HTestParameter& test){
	fHypoModel->HTestChanged(test);
	NotifyViews(vnParametersChanged);
	SetDirty(true);
}

void QHypoplasticDoc::HInitialStateChanged(HHypoResults& state){
	fHypoModel->HInitialStateChanged(state);
	NotifyViews(vnParametersChanged);
	SetDirty(true);
}

short QHypoplasticDoc::HInputParameters(TWindow* aWindow)
{
  short returnDialog;
  int 	error;

  returnDialog = (short) QInputParamDlg(aWindow, this).Execute();
  if ( returnDialog == IDOK) {
    fHypoModel->HInitSimulation();
    error = fHypoModel->HSimulation();
    NotifyViews(vnParametersChanged);
    SetDirty(true);
    if ( error ) {
  		::MessageBox(NULL, "Error in HSimulation() method",
    	GetDocManager().GetApplication()->LoadString(error).c_str(),
      MB_ICONEXCLAMATION|MB_OK);
  	}
  }
  return returnDialog;
}


/***************************************************************************
abstract class of hypoplastic model

 ***************************************************************************/



/***************************************************************************
hypoplastic model of Wolffersdorff

 ***************************************************************************/

void QWolffersdorffModel::HMaterialChanged(HMaterialParameter& material){
  	// get the values
	defphi = material.phi;
	defhs = material.hs;
	defed0 = material.ed0;
	defec0 = material.ec0;
	defei0 = material.ei0;
	defn = material.n;
	defalpha = material.alpha;
	defbeta = material.beta;
  	// simulate with these values
	HInitSimulation();
	HSimulation();
}

void QWolffersdorffModel::HTestChanged(HTestParameter& test){
	deftype = test.test_type;
	defdir = test.direction;
	defdt = test.dt;
	defsigmamax = test.sigmamax;
	defepsilonmax = test.epsilonmax;
  	defdIncAngle = test.defdIncAngle;
	HInitSimulation();
	HSimulation();
}

void QWolffersdorffModel::HInitialStateChanged(HHypoResults& state){
	defsigma[1] = state.sigma1;
	defsigma[2] = state.sigma3;
	defsigma[3] = state.sigma3;
	defepsilon[1] = state.epsilon1;
	defepsilon[2] = state.epsilon3;
	defepsilon[3] = state.epsilon3;
	defe = state.e;
	HInitSimulation();
	HSimulation();
}

void QWolffersdorffModel::HInitValues(){
  	// default values of Hostun RF sand
	defphi = 31;
	defhs = 1000;
	defn = 0.29;
	defed0 = 0.61;
	defec0 = 0.91;
	defei0 = 1.09;
	defalpha = 0.13;
	defbeta = 2.0;

	deftype = TRIAXIAL_UNDRAINED;
	defdir = D_COMPRESSION;
	defdt = 0.0005;
  	defdIncAngle = 125.3; 		// constant q test
	defsigmamax = -1;
	defepsilonmax = -0.1;

  	defsigma[0] = defDsigma[0] = 	defepsilon[0] = defDepsilon[0] = 0;
	defsigma[1] = -0.1;  		// MPa
	defsigma[2] = -0.1;
	defsigma[3] = -0.1;
	defDsigma[1] = -0.001;   	//  MPa
	defDsigma[2] = -0.0005;
	defDsigma[3] = -0.0005;
	defepsilon[1] = 0;  			// natural
	defepsilon[2] = 0;
	defepsilon[3] = 0;
	defDepsilon[1] = 0;
	defDepsilon[2] = 0;
	defDepsilon[3] = 0;
	defe = 0.95;        			// very loose

      // uncomment to get default values of Hostun sand (response envelope)
      /*
   defphi = 33;     // d50 = 0.8mm
	defhs = 1000;
	defn = 0.29;
	defed0 = 0.529;
	defec0 = 0.820;
	defei0 = 0.984;  // 1.2*emax
	defalpha = 0.13;
	defbeta = 2.0;

   deftype = TRIAXIAL_DRAINED;
   defe = 0.56;        			// dense Dr=89%
   */
}

void QWolffersdorffModel::HInitSimulation(){

	fMaterialParameter.phi = defphi;
	fMaterialParameter.hs = defhs;
	fMaterialParameter.n = defn;
	fMaterialParameter.ed0 = defed0;
	fMaterialParameter.ec0 = defec0;
	fMaterialParameter.ei0 = defei0;
	fMaterialParameter.alpha = defalpha;
	fMaterialParameter.beta = defbeta;

	fTestParameter.test_type = deftype;
	fTestParameter.direction = defdir;
	fTestParameter.dt = defdt;
	fTestParameter.sigmamax = defsigmamax;
	fTestParameter.epsilonmax = defepsilonmax;

  	// index zero not used
  fCurrentState.sigma[0] = fCurrentState.Dsigma[0] = 0;
  fCurrentState.epsilon[0] = fCurrentState.Depsilon[0] = 0;

	fCurrentState.sigma[1] = defsigma[1];  	// in MPa
	fCurrentState.sigma[2] = defsigma[2];
	fCurrentState.sigma[3] = defsigma[3];
	fCurrentState.Dsigma[1] = defDsigma[1];   // in MPa
	fCurrentState.Dsigma[2] = defDsigma[2];
	fCurrentState.Dsigma[3] = defDsigma[3];
	fCurrentState.epsilon[1] = defepsilon[1];
	fCurrentState.epsilon[2] = defepsilon[2];
	fCurrentState.epsilon[3] = defepsilon[3];
	fCurrentState.Depsilon[1] = defDepsilon[1];
	fCurrentState.Depsilon[2] = defDepsilon[2];
	fCurrentState.Depsilon[3] = defDepsilon[3];
	fCurrentState.e = defe;

	fSimulatedData.Flush();
}


/*
	Convention in hypoplastic model: compression negative, MPa, natural strain (not %)
  Attempt: ControlKeyDown + test_type == 'a' --> SpecialTests() like
  liquefaction of anisotropic samples
*/
int QWolffersdorffModel::HSimulation()
{
	short ControlKeyDown = HiUint16(::GetKeyState(VK_CONTROL)); // VK_RCONTROL
  	TWaitCursor waitCursor;

  	if ( fTestParameter.test_type == LINEAR_STRESS_PATH && ControlKeyDown ){
  		error = SpecialTests();
   } else if ( fTestParameter.test_type == TRIAXIAL_UNDRAINED && ControlKeyDown ){
  		error = HCheckFromStrainState();
  	} else if ( fTestParameter.test_type == STRAIN_RESPONSE_ENVELOPE ){
      if ( !ControlKeyDown )
      	defdIncAngle = 45.0;  // in triaxial plane
  		error =  HStrainResponseEnvelope(defsigma[3], defsigma[1]);
   } else if ( fTestParameter.test_type == STRESS_RESPONSE_ENVELOPE ){
   	if ( !ControlKeyDown )
      	defdIncAngle = 45.0;
  		error =  HStressResponseEnvelope(defsigma[3], defsigma[1]);
   } else if ( fTestParameter.test_type == LINEAR_STRAIN_PATH ){
  		error = HLinearStrainPath();
   } else if ( fTestParameter.test_type == CIRCULAR_STRESS_PATH ) {
      if ( !ControlKeyDown )
      	defdIncAngle = 45.0;
      double radius = -0.1;
   	error = HCircularStressPath(defsigma[3], defsigma[1], radius);
  	} else {
    	error = ClassicalTests();
  	}
  	return error;
}

/*
	Triaxial CID, CIU, CKD, CKU, Oedometer, Isotropic tests
*/
int QWolffersdorffModel::ClassicalTests(void)
{
double dtLimit;
HHypoResults currentResults;

		/* initialize
      	sig += dsig / dt in integration step below. The time step is gien,
         thus dsig is a constant (0.1) and also deps (1)
      */
  error = 0;
	fCurrentState.sigma[3]=fCurrentState.sigma[2];
	if ( fTestParameter.direction == D_COMPRESSION )
  		fCurrentState.Depsilon[1] = -1;   // compression
	else
  		fCurrentState.Depsilon[1] = 1;    // extension

	if ( fTestParameter.test_type == TRIAXIAL_UNDRAINED ){
		fCurrentState.Depsilon[2] = -fCurrentState.Depsilon[1]*0.5;
		fCurrentState.Depsilon[3] = fCurrentState.Depsilon[2];
	} else if ( fTestParameter.test_type == ISOTROPIC ){
		fCurrentState.Depsilon[2] = fCurrentState.Depsilon[1];
		fCurrentState.Depsilon[3] = fCurrentState.Depsilon[1];
	} else if ( fTestParameter.test_type == LINEAR_STRESS_PATH ){
    fCurrentState.Dsigma[1] = -.1*sin(Deg2Rad(defdIncAngle));
    fCurrentState.Dsigma[3] = -.1*cos(Deg2Rad(defdIncAngle))/sqrt(2.0);
    fCurrentState.Dsigma[2] = fCurrentState.Dsigma[3];
	}

	HIntermediateConstants();

		// loop until stop conditions
	do {
		// last state
		HSetCurrentState(currentResults); // and store

		// stress rate
		error = CoreModel();

		// stop conditions
		dtLimit = Limits();
		if (dtLimit!=0)
    		fTestParameter.dt = dtLimit;

		// integration
		for(int i=1; i<=3; i++) {
			fCurrentState.sigma[i] += fCurrentState.Dsigma[i]*defdt;
			fCurrentState.epsilon[i] += fCurrentState.Depsilon[i]*defdt;
		}
		fCurrentState.e += (1.+fCurrentState.e)
      						 *Trace3(fCurrentState.Depsilon)*fTestParameter.dt;
	}
	while( dtLimit == 0. && !error );

	// last result
	HSetCurrentState(currentResults);

  return error;
}

int QWolffersdorffModel::HIntermediateConstants(void)
{
double expon;

  f_a = sqrt(3.)*(3.-sin(Deg2Rad(fMaterialParameter.phi))) /
		(2.*sqrt(2.)*sin(Deg2Rad(fMaterialParameter.phi)));
  f_hi = fMaterialParameter.hs/fMaterialParameter.n*pow(fMaterialParameter.ei0 /
   		fMaterialParameter.ec0,fMaterialParameter.beta) /
         ( 3.+f_a*f_a-sqrt(3.)*f_a*pow((fMaterialParameter.ei0-fMaterialParameter.ed0)/
         (fMaterialParameter.ec0-fMaterialParameter.ed0),fMaterialParameter.alpha) );
  expon = exp(-pow(-Trace3(fCurrentState.sigma)/fMaterialParameter.hs,fMaterialParameter.n));
  f_ei = fMaterialParameter.ei0*expon;
  f_ec = fMaterialParameter.ec0*expon;
  f_ed = fMaterialParameter.ed0*expon;
  f_fe = pow(f_ec/fCurrentState.e,fMaterialParameter.beta);
  f_fd = pow((fCurrentState.e - f_ed)/(f_ec - f_ed),fMaterialParameter.alpha);
  f_fb = f_hi*(1. + f_ei)/f_ei*pow(-Trace3(fCurrentState.sigma)
   		/fMaterialParameter.hs,1.-fMaterialParameter.n);

	return 0; // no error
}

void QWolffersdorffModel::HSetCurrentState(HHypoResults& currentResults)
{
	currentResults.sigma1 = - fCurrentState.sigma[1]* 1000;  	// stress kPa
   currentResults.sigma2 = - fCurrentState.sigma[2]* 1000;  	// stress kPa
   currentResults.sigma3 = - fCurrentState.sigma[3]* 1000;  	// stress kPa
   currentResults.epsilon1 = - fCurrentState.epsilon[1] * 100; // strain %
   currentResults.epsilon2 = - fCurrentState.epsilon[2] * 100; // strain %
   currentResults.epsilon3 = - fCurrentState.epsilon[3] * 100; // strain %
   currentResults.e = fCurrentState.e;
   fSimulatedData.Add(currentResults);
}

/* are the test limits already reached?
	 yes --> correction of the last dt to reach the limit exactly
*/
double QWolffersdorffModel::Limits(void){
	double dtLimit=0;

	if (fTestParameter.direction == D_COMPRESSION){
		if ( (fCurrentState.sigma[1]
      	+ fCurrentState.Dsigma[1]*fTestParameter.dt) < fTestParameter.sigmamax )
			dtLimit = (fTestParameter.sigmamax-fCurrentState.sigma[1])/fCurrentState.Dsigma[1];
		else if ( (fCurrentState.epsilon[1]
      	+ fCurrentState.Depsilon[1]*fTestParameter.dt) < fTestParameter.epsilonmax )
			dtLimit = (fTestParameter.epsilonmax-fCurrentState.epsilon[1])/fCurrentState.Depsilon[1];
	} else if (fTestParameter.direction == D_EXTENSION) {
		if ( (fCurrentState.sigma[1]
      	+ fCurrentState.Dsigma[1]*fTestParameter.dt) > fTestParameter.sigmamax )
			dtLimit = (fCurrentState.sigma[1]-fTestParameter.sigmamax)/fCurrentState.Dsigma[1];
		else if ( (fCurrentState.epsilon[1]
      	+ fCurrentState.Depsilon[1]*fTestParameter.dt) > fTestParameter.epsilonmax )
			dtLimit = (fCurrentState.epsilon[1]-fTestParameter.epsilonmax)/fCurrentState.Depsilon[1];
	}
	return dtLimit;
}

/* constitutive equation for the stress rate
	.. hypoplastic model of P.A. von Wolffersdorff and Gudehus
   "A hypoplastic relation for granular materials with a predefined limit state surface"
   Mechanics of Cohesive-Frictional Materials, vol 1, 4, 251 - 271, 1996
*/
int QWolffersdorffModel::CoreModel(void){
  double Td[4], Td2[4], Tsd[4], Tsd2[4], Tsd3[4], TdD[4];
  double tan_psi, cos_3th;
  double expon;
  int i;

  for(i=1; i<=3; i++) {
	 Td[i] = fCurrentState.sigma[i]/Trace3(fCurrentState.sigma);
	 Td2[i] = Td[i]*Td[i];
	 Tsd[i] = Td[i]-1./3.;
	 Tsd2[i] = Tsd[i]*Tsd[i];
	 Tsd3[i] = Tsd2[i]*Tsd[i];
  }

  expon = exp(-pow(-Trace3(fCurrentState.sigma)/fMaterialParameter.hs,fMaterialParameter.n));
  f_ei = fMaterialParameter.ei0*expon;
  f_ec = fMaterialParameter.ec0*expon;
  f_ed = fMaterialParameter.ed0*expon;
  f_fe = pow(f_ec/fCurrentState.e,fMaterialParameter.beta);
  f_fd = pow((fCurrentState.e - f_ed)/(f_ec - f_ed),fMaterialParameter.alpha);
  f_fb = f_hi*(1. + f_ei)/f_ei*pow(-Trace3(fCurrentState.sigma)
   		/fMaterialParameter.hs,1.-fMaterialParameter.n);

  tan_psi = sqrt(3.)*Norm3(Tsd);
  if (tan_psi == 0.)
	 f_F = 1.;
  else {
	 cos_3th = -sqrt(6.)*Trace3(Tsd3)/pow(Trace3(Tsd2),1.5);
	 if (cos_3th > 1.)
		cos_3th = 1.;
	 else if (cos_3th < -1.)
		cos_3th = -1.;
	 f_F = sqrt( tan_psi*tan_psi/8.+(2.-tan_psi*tan_psi) /
					  (2.+sqrt(2.)*tan_psi*cos_3th) ) - tan_psi/2./sqrt(2.);
  }

  	/* unknown stress rate in classical triaxial CID */
  if ( fTestParameter.test_type == TRIAXIAL_DRAINED )
  	error = TriaxStrainIncControlled();

  	/* unknown strain rate in triaxial CKD
   	NOT very object oriented with HSimulation(), new strategy is necessary
      for the next version
   */
  if ( fTestParameter.test_type == LINEAR_STRESS_PATH
  		|| fTestParameter.test_type == STRAIN_RESPONSE_ENVELOPE
  		|| fTestParameter.test_type == CIRCULAR_STRESS_PATH )
  	error = HFindStrainIncrement();
  else {
   error = 0;
   HSolve();
  	/* instead of using HSolve(),
   	the code below gives incorrect response of undrained test on RF loose
   	sand, eO = 0.960, up to at least 50%, in q/p plane WHY?
   for(i=1; i<=3; i++)
  		TdD[i] = Td[i]*fCurrentState.Depsilon[i];
  	for(i=1; i<=3; i++) {
   	fCurrentState.Dsigma[i] = f_fb*f_fe/Trace3(Td2)*
      	(f_F*f_F*fCurrentState.Depsilon[i]+f_a*f_a*Trace3(TdD)*Td[i]
         +f_fd*f_a*f_F*(Td[i]
         +Tsd[i])*Norm3(fCurrentState.Depsilon));
  	} */
  }
  return error;
}

/*
	unknown stress rate in classical triaxial CID (D1 imposed, T3=0), or CIU(D imposed)
   This is specialized 2D routine == LinearStressPath() with 90 or 270 degrees
*/
int QWolffersdorffModel::TriaxStrainIncControlled(void){
	double L21,L22,N2;
	double trT,a,b,c,diskr,x1,x2,x;

	trT = Trace3(fCurrentState.sigma);
	L21 = f_a*f_a*fCurrentState.sigma[1]*fCurrentState.sigma[2]/trT/trT;
	L22 = (f_F*f_F+2.*f_a*f_a*fCurrentState.sigma[2]*fCurrentState.sigma[2]/trT/trT);
	N2 = f_F*f_fd*f_a/3.*(4.*fCurrentState.sigma[2]-fCurrentState.sigma[1])/trT;

	a = 2.*N2*N2-L22*L22;
	b = 4.*L21*N2*fCurrentState.Depsilon[1];
	c = (L22*L22+2.*L21*L21)*fCurrentState.Depsilon[1]*fCurrentState.Depsilon[1];

	diskr = b*b-4.*a*c;
	if ( diskr < 0. ) {
		return IDS_ER_NO_REAL_SOLUTION;
	} else {
		x1 = (-b+sqrt(diskr))/2./a;
		x2 = (-b-sqrt(diskr))/2./a;
	}
	if (x1 < 0. && x2 < 0.) {
		return IDS_ER_NEGATIVE_SOLUTION;
  } else if ( x1 > 0. && x2 > 0. ) {
		return IDS_ER_NOT_UNIQUE_SOLUTION;
	}	else {
			if ( x1 > 0. )
			x = x1;
			else
			x = x2;
	}

	fCurrentState.Depsilon[2] = -L21/L22*fCurrentState.Depsilon[1]-N2/L22*x;
	fCurrentState.Depsilon[3] = fCurrentState.Depsilon[2];
  return 0; // no error
}

/* Triaxial with stress increment controlled
	Resolution of dT = LD + N||D|| with D unknown
  W. Wu, A. Niemulis. "Beyond failure of granular materials",
  Int J for Numerical and Analytical Methods in Geomechanics
  1997, vol 21, n3, pp 153 - 174. Section 4.3
*/
int QWolffersdorffModel::HFindStrainIncrement(void){

  int i, n = 3;
  Matrix L(n,n);
  ColumnVector dT(n), T(n), N(n), A(n), B(n);
  Real a, b, c, x1, x2, x;
  Real L11, L12, L13, L21, L22, L23, L31, L32, L33, trT, trTtrT;
  Real N1, N2, N3;
  Real factor, tmp;
  Real diskr;

  	// components of T
  T << fCurrentState.sigma[1] << fCurrentState.sigma[2]
  	<< fCurrentState.sigma[3];

  factor = f_fb * f_fe * pow(T(1) + T(2) + T(3), 2.0) / (T(1)*T(1) + T(2)*T(2) + T(3)*T(3));

  	// components of dT
  dT << fCurrentState.Dsigma[1]/factor << fCurrentState.Dsigma[2]/factor
     << fCurrentState.Dsigma[3]/factor;

  	// components of N
  trT = Trace3(fCurrentState.sigma);
  trTtrT = trT * trT;
  tmp = f_F * f_fd * f_a /trT / 3.0;
  N1 = tmp * ( 5.*T(1) - T(2) - T(3) );
  N2 = tmp * ( 5.*T(2) - T(1) - T(3) );
  N3 = tmp * ( 5.*T(3) - T(2) - T(1) );
  N << N1 << N2 << N3;

  	// components of L
  tmp = f_a*f_a/trTtrT;
  L11 = f_F*f_F + tmp*T(1)*T(1);
  L12 = tmp*T(1)*T(2);
  L13 = tmp*T(1)*T(3);

  L21 = tmp*T(1)*T(2);
  L22 = f_F*f_F + tmp*T(2)*T(2);
  L23 = tmp*T(2)*T(3);

  L31 = tmp*T(1)*T(3);
  L32 = tmp*T(2)*T(3);
  L33 = f_F*f_F + tmp*T(3)*T(3);

  L << L11 << L12 << L13
  	 << L21 << L22 << L23
    << L31 << L32 << L33;

    // uncomment these lines to get only the linear part = hypoelastic envelope
  /*
  A = L.i() * dT;
  for ( i=1; i<=n; i++ ) {
  	fCurrentState.Depsilon[i] = A(i);
  }
  return 0; */

  	// resolution of dT = LD + N||D||, x = ||D||==> a x*x + bx + c = 0
  A = L.i() * dT;
  B = L.i() * N;
  a = (B.t() * B).AsScalar() - 1.0;
  b = (A.t() * B).AsScalar();
  c = (A.t() * A).AsScalar();

  diskr = pow(b/a,2.0) - c/a;
  if ( diskr < 0.0 ) {
		return IDS_ER_NO_REAL_SOLUTION;
  }
  x1 = b/a + sqrt( diskr );
  x2 = b/a - sqrt( diskr );

  if ( x1 < 0. && x2 < 0. ) {
		return IDS_ER_NEGATIVE_SOLUTION;
  } else if ( x1 > 0. && x2 > 0. ) {
		return IDS_ER_NOT_UNIQUE_SOLUTION;
  } else {
  	if ( x1 > 0. )
			x = x1;
    else
			x = x2;
	}
  	// finally, get the strain rate
  for ( i=1; i<=n; i++ ) {
  	fCurrentState.Depsilon[i] = A(i) - B(i) * x;
	}

  return 0; // no error
}

/*
	First attempt: toSigma3 must be > fromSigma3, toSigma1 must be > fromSigma1
  In triaxial plane (X,Y) == (Sigma3*sqrt(2.0),Sigma1)
*/
int QWolffersdorffModel::HLinearStressPath(double fromSigma3, double fromSigma1,
	double toSigma3, double toSigma1)
{
int stop;
HHypoResults currentResults;

  	// slope of the line fromTo
	if ( fromSigma3 - toSigma3 == 0.0 )
    defdIncAngle = 90.0;
  else
  	defdIncAngle = Rad2Deg( atan( (toSigma1 - fromSigma1)
   					/ (toSigma3 - fromSigma3) / sqrt(2.0) ));

  	// constant stress increment
  fCurrentState.Dsigma[1] = -.1*sin(Deg2Rad(defdIncAngle));
  fCurrentState.Dsigma[2] = fCurrentState.Dsigma[3] = -.1*cos(Deg2Rad(defdIncAngle))/sqrt(2.0);
  fCurrentState.sigma[1] = fromSigma1;
  fCurrentState.sigma[2] = fCurrentState.sigma[3] = fromSigma3;

	HIntermediateConstants();

		// loop until stop conditions
	do {
		// last state
		HSetCurrentState(currentResults);

		// stress rate
		error = CoreModel();

		// stop conditions
    stop = 0;
    if ( fCurrentState.sigma[1] + fCurrentState.Dsigma[1]*defdt <= toSigma1 ) {
    	fCurrentState.Dsigma[1] = (toSigma1 - fCurrentState.sigma[1])/defdt;
      stop = 1;
    }
    if ( fCurrentState.sigma[3] + fCurrentState.Dsigma[3]*defdt <= toSigma3 ) {
    	fCurrentState.Dsigma[3] = (toSigma3 - fCurrentState.sigma[3])/defdt;
      fCurrentState.Dsigma[2] = fCurrentState.Dsigma[3];
      stop = 1;
    }

		// integration
    for(int i=1; i<=3; i++) {
    	fCurrentState.sigma[i] += fCurrentState.Dsigma[i]*defdt;
      fCurrentState.epsilon[i] += fCurrentState.Depsilon[i]*defdt;
    }
		fCurrentState.e += (1.+fCurrentState.e)*Trace3(fCurrentState.Depsilon)*fTestParameter.dt;
	}
	while( !stop && !error );

	// last result
	HSetCurrentState(currentResults);

  return error;
}

/*
	First attempt: toEps3 must be > fromEps3, toEps1 must be > fromEps1
  In triaxial strain plane (X,Y) == (Eps3*sqrt(2.0),Eps1)
*/
int QWolffersdorffModel::HLinearStrainPath(void)
{
int stop;
HHypoResults currentResults;

  	// constant strain increment
  fCurrentState.Depsilon[1] = -.1*sin(Deg2Rad(defdIncAngle));
  fCurrentState.Depsilon[2] = fCurrentState.Depsilon[3] = -.1*cos(Deg2Rad(defdIncAngle))/sqrt(2.0);
  fCurrentState.sigma[1] = defsigma[1];
  fCurrentState.sigma[2] = fCurrentState.sigma[3] = defsigma[3];

	HIntermediateConstants();

		// loop until stop conditions
	do {
		// last state
		HSetCurrentState(currentResults);

		// stress rate
		error = CoreModel();

		// stop conditions
    stop = 0;
    for(int i=1; i<=3; i++) {
    	if ( fabs(fCurrentState.epsilon[i] + fCurrentState.Depsilon[i]*defdt) >= fabs(defepsilonmax) ) {
      	stop = 1;
    	}
    }

		// integration
    for(int i=1; i<=3; i++) {
    	fCurrentState.sigma[i] += fCurrentState.Dsigma[i]*defdt;
      fCurrentState.epsilon[i] += fCurrentState.Depsilon[i]*defdt;
    }
		fCurrentState.e += (1.+fCurrentState.e)*Trace3(fCurrentState.Depsilon)*fTestParameter.dt;
	}
	while( !stop && !error );

	// last result
	HSetCurrentState(currentResults);

  return error;
}

/*
	Triaxial CKD (anisotropic consolidation) + CIU
*/
int QWolffersdorffModel::SpecialTests(void)
{
	double X, Y;
  X = Y = -.02;  //  by dialog ?
  	// CKD: anisotropic consolidation
	error = HLinearStressPath(X, Y, defsigma[3], defsigma[1]);
  	// CIU: classical undrained test
  fTestParameter.test_type = TRIAXIAL_UNDRAINED;
  ClassicalTests();
  fTestParameter.test_type = LINEAR_STRESS_PATH;

  return error;
}

/*
  - Strain response envelope of point fromSigma3,fromSigma1
  	In triaxial plane (X,Y) == (Sigma3*sqrt(2.0),Sigma1)
  - defdIncAngle permit to choose a circle within the unit sphere, if
  	defdIncAngle = 45 ==> circle in triaxial stress plane
*/
int QWolffersdorffModel::HStrainResponseEnvelope(double fromSigma3, double fromSigma1)
{
double stepAngle;
int stop;
HHypoResults currentResults;

  stepAngle = 1.0;  			// by step of 1 degree
  double angle = -1.0;      // from this firts value, 0 degree
  double maxAngle = 360.0;  // to 360 degree

  	// current stress state
  fCurrentState.sigma[1] = fromSigma1;
  fCurrentState.sigma[2] = fCurrentState.sigma[3] = fromSigma3;
  fCurrentState.epsilon[1] = fCurrentState.epsilon[2] = fCurrentState.epsilon[3] = 0.0;
  fCurrentState.e = defe;

	HIntermediateConstants();

		// loop until stop conditions
	do {
		// last state
		HSetCurrentState(currentResults);

      angle += stepAngle;
      fCurrentState.Dsigma[1] = -.1 * sin(Deg2Rad(angle));
  		fCurrentState.Dsigma[2] = -.1 * cos(Deg2Rad(angle)) * cos(Deg2Rad(defdIncAngle));
      fCurrentState.Dsigma[3] = -.1 * cos(Deg2Rad(angle)) * sin(Deg2Rad(defdIncAngle));
      fCurrentState.sigma[1] = fromSigma1;
  		fCurrentState.sigma[2] = fCurrentState.sigma[3] = fromSigma3;
  		fCurrentState.epsilon[1] = fCurrentState.epsilon[2] = fCurrentState.epsilon[3] = 0.0;
  		fCurrentState.e = defe;   // void ratio do not change

		// stress rate
		error = CoreModel();

      	/* uncomment the code below, to check the response of dT = f(D), TriaxStrainIncControlled()-- CoreModel
         with the calculated strain increments of HFindStrainIncrement()-- InverseCoreModel, we
         must find the circular stress increment given by D = g(dT), g * f = I.
         */
      /*fCurrentState.Dsigma[1] = fCurrentState.Dsigma[2] = fCurrentState.Dsigma[3] = 0.0;
      fTestParameter.test_type = TRIAXIAL_UNDRAINED;
      error = CoreModel();
      fTestParameter.test_type = LINEAR_STRESS_PATH; */

		// stop conditions
    	stop = 0;
    	if ( angle + stepAngle > maxAngle ) {
      	stop = 1;
    	}

		// integration
		for(int i=1; i<=3; i++) {
			fCurrentState.sigma[i] += fCurrentState.Dsigma[i]*defdt;
			fCurrentState.epsilon[i] = fCurrentState.Depsilon[i]*defdt;
		}
	}
	while( !stop && !error );

	// last result
	HSetCurrentState(currentResults);

  return error;
}

/*
  - Stress response envelope of point fromSigma3,fromSigma1
  	In triaxial plane (X,Y) == (Eps3*sqrt(2.0),Eps1)
  - defdIncAngle permit to choose a circle within the unit sphere, if
  	defdIncAngle = 45 ==> circle in triaxial strain plane
*/
int QWolffersdorffModel::HStressResponseEnvelope(double fromSigma3, double fromSigma1)
{
double stepAngle;
int stop;
HHypoResults currentResults;

  stepAngle = 1.0;  // 1 degree
  double angle = -1.0;
  double maxAngle = 360.0;

  	// current stress state
  fCurrentState.sigma[1] = fromSigma1;
  fCurrentState.sigma[2] = fCurrentState.sigma[3] = fromSigma3;
  fCurrentState.epsilon[1] = fCurrentState.epsilon[2] = fCurrentState.epsilon[3] = 0.0;
  fCurrentState.e = defe;

	HIntermediateConstants();

		// loop until stop conditions
	do {
		// last state
		HSetCurrentState(currentResults);

      angle += stepAngle;
      fCurrentState.Depsilon[1] = -.1 * sin(Deg2Rad(angle));
  		fCurrentState.Depsilon[2] = -.1 * cos(Deg2Rad(angle)) * cos(Deg2Rad(defdIncAngle));
      fCurrentState.Depsilon[3] = -.1 * cos(Deg2Rad(angle)) * sin(Deg2Rad(defdIncAngle));
      fCurrentState.sigma[1] = fromSigma1;
  		fCurrentState.sigma[2] = fCurrentState.sigma[3] = fromSigma3;
  		fCurrentState.epsilon[1] = fCurrentState.epsilon[2] = fCurrentState.epsilon[3] = 0.0;
  		fCurrentState.e = defe;  // void ratio do not change

		// stress rate
		error = CoreModel();

		// stop conditions
    	stop = 0;
    	if ( angle + stepAngle > maxAngle ) {
      	stop = 1;
    	}

		// integration
		for(int i=1; i<=3; i++) {
			fCurrentState.sigma[i] += fCurrentState.Dsigma[i]*defdt;
			fCurrentState.epsilon[i] = fCurrentState.Depsilon[i]*defdt;
		}
	}
	while( !stop && !error );

	// last result
	HSetCurrentState(currentResults);

  return error;
}


/*
	Utility: Check the response of the model from a known strain state.
   Use the initial strain state (defepsilon) of the input dialog as the initial
   strain RATE state (Depsilon)
*/
int QWolffersdorffModel::HCheckFromStrainState(void)
{

  fCurrentState.Depsilon[1] = defepsilon[1];
  fCurrentState.Depsilon[3] = defepsilon[3];
  fCurrentState.Depsilon[2] = fCurrentState.Depsilon[3];

  fCurrentState.epsilon[1] = fCurrentState.epsilon[2] = fCurrentState.epsilon[3] = 0.0;
  defepsilon[1] = defepsilon[2] = defepsilon[3] = 0.0;

  HIntermediateConstants();
  	// stress rate
  error = CoreModel();
  	// integration
  for(int i=1; i<=3; i++) {
  		fCurrentState.sigma[i] += fCurrentState.Dsigma[i]*defdt;
      fCurrentState.epsilon[i] += fCurrentState.Depsilon[i]*defdt;
  }
  	// make a breakpoint here and examine the fCurrentState structure
  return error;
}

/* Test with strain increment controlled
	Direct resolution of dT = LD + N||D||, with dT unknown
  "A hypoplastic relation for granular materials with a predefined limit state surface"
   Mechanics of Cohesive-Frictional Materials, vol 1, 4, 251 - 271, 1996
*/
int QWolffersdorffModel::HSolve(void)
{
	int i, n = 3;
  Matrix L(n,n);
  ColumnVector dT(n), T(n), D(n), N(n);
  Real c;
  Real L11, L12, L13, L21, L22, L23, L31, L32, L33, trT, trTtrT;
  Real N1, N2, N3;
  Real factor, tmp;
  HHypoResults currentResults;

  	// components of T
  T << fCurrentState.sigma[1] << fCurrentState.sigma[2]
  	<< fCurrentState.sigma[3];

  	// components of D
  D << fCurrentState.Depsilon[1] << fCurrentState.Depsilon[2]
  	 << fCurrentState.Depsilon[3];

  factor = f_fb * f_fe * pow(T(1) + T(2) + T(3), 2.0) / (T(1)*T(1) + T(2)*T(2) + T(3)*T(3));

  	// components of N
  trT = Trace3(fCurrentState.sigma);
  trTtrT = trT * trT;
  tmp = f_F * f_fd * f_a / trT / 3.0;
  N1 = tmp * ( 5.*T(1) - T(2) - T(3) );
  N2 = tmp * ( 5.*T(2) - T(1) - T(3) );
  N3 = tmp * ( 5.*T(3) - T(2) - T(1) );
  N << N1 << N2 << N3;

  	// components of L
  tmp = f_a*f_a/trTtrT;
  L11 = f_F*f_F + tmp*T(1)*T(1);
  L12 = tmp*T(1)*T(2);
  L13 = tmp*T(1)*T(3);

  L21 = tmp*T(1)*T(2);
  L22 = f_F*f_F + tmp*T(2)*T(2);
  L23 = tmp*T(2)*T(3);

  L31 = tmp*T(1)*T(3);
  L32 = tmp*T(2)*T(3);
  L33 = f_F*f_F + tmp*T(3)*T(3);

  L << L11 << L12 << L13
  	 << L21 << L22 << L23
    << L31 << L32 << L33;

  c = sqrt(D(1)*D(1) + D(2)*D(2) + D(3)*D(3)); //c = D.Norm1();

  dT = factor * (L * D + c * N);
  for(i=1; i<=3; i++) {
  	fCurrentState.Dsigma[i] = dT(i);
  }

  return 0;  // no error
}

/*
  - Circular Stress Path of point fromSigma3,fromSigma1
  	In triaxial plane (X,Y) == (Sigma3*sqrt(2.0),Sigma1)
  - defdIncAngle permit to choose a circle within the unit sphere, if
  	defdIncAngle = 45 ==> circle in triaxial stress plane
*/
int QWolffersdorffModel::HCircularStressPath(double fromSigma3,
	double fromSigma1, double radius)
{
double stepAngle;
int stop;
HHypoResults currentResults;

  stepAngle = 1.0;  			// by step of 1 degree
  double angle = 0.0;      // from this firts value, 0 degree from Y axe
  double oldAngle = angle;
  double maxAngle = 360.0;  // to 360 degree
  double direction;
  if ( fTestParameter.direction == D_COMPRESSION )  // re-use it
  		direction = 1;   		// clockwise
	else
  		direction = -1;    	// anticlockwise

  	// current stress state
  fCurrentState.sigma[1] = fromSigma1 + radius*defdt;
  fCurrentState.sigma[2] = fCurrentState.sigma[3] = fromSigma3;
  //fCurrentState.epsilon[1] = fCurrentState.epsilon[2] = fCurrentState.epsilon[3] = 0.0;

	HIntermediateConstants();

		// loop until stop conditions
	do {
		// last state
		HSetCurrentState(currentResults);

      oldAngle = angle;

      angle += stepAngle * direction;
      fCurrentState.Dsigma[1] = radius * ( cos(Deg2Rad(angle)) - cos(Deg2Rad(oldAngle)) );
  		fCurrentState.Dsigma[2] = radius * ( sin(Deg2Rad(angle)) - sin(Deg2Rad(oldAngle)) )
      			* cos(Deg2Rad(defdIncAngle));
      fCurrentState.Dsigma[3] = radius * ( sin(Deg2Rad(angle)) - sin(Deg2Rad(oldAngle)) )
      			* sin(Deg2Rad(defdIncAngle));

		// stress rate
		error = CoreModel();

		// stop conditions
    	stop = 0;
    	if ( fabs(angle + stepAngle) > maxAngle ) {
      	stop = 1;
    	}

		// integration
		for(int i=1; i<=3; i++) {
			fCurrentState.sigma[i] += fCurrentState.Dsigma[i]*defdt;
			fCurrentState.epsilon[i] += fCurrentState.Depsilon[i]*defdt;
		}
      fCurrentState.e += (1.+fCurrentState.e)*Trace3(fCurrentState.Depsilon)*fTestParameter.dt;
	}
	while( !stop && !error );

	// last result
	HSetCurrentState(currentResults);

  return error;
}



/* extended model by Wei Wu

*/
// should be called CoreModel
int QWolffersdorffExtendedModel::HSolve(void)
{
	int i, n = 3;
  Matrix L(n,n);
  ColumnVector dT(n), T(n), D(n), N(n), A(n), B(n), L1N(n);
  Real a, b, c;
  Real L11, L12, L13, L21, L22, L23, L31, L32, L33, trT, trTtrT;
  Real N1, N2, N3;
  Real factor, tmp, costheta;

  	// components of T
  T << fCurrentState.sigma[1] << fCurrentState.sigma[2]
  	<< fCurrentState.sigma[3];

  	// components of D
  D << fCurrentState.Depsilon[1] << fCurrentState.Depsilon[2]
  	 << fCurrentState.Depsilon[3];

  factor = f_fb * f_fe * pow(T(1) + T(2) + T(3), 2.0) / (T(1)*T(1) + T(2)*T(2) + T(3)*T(3));

  	// components of N
  trT = Trace3(fCurrentState.sigma);
  trTtrT = trT * trT;
  tmp = f_F * f_fd * f_a / trT / 3.0;
  N1 = tmp * ( 5.*T(1) - T(2) - T(3) );
  N2 = tmp * ( 5.*T(2) - T(1) - T(3) );
  N3 = tmp * ( 5.*T(3) - T(2) - T(1) );
  N << N1 << N2 << N3;

  	// components of L
  tmp = f_a*f_a/trTtrT;
  L11 = f_F*f_F + tmp*T(1)*T(1);
  L12 = tmp*T(1)*T(2);
  L13 = tmp*T(1)*T(3);

  L21 = tmp*T(1)*T(2);
  L22 = f_F*f_F + tmp*T(2)*T(2);
  L23 = tmp*T(2)*T(3);

  L31 = tmp*T(1)*T(3);
  L32 = tmp*T(2)*T(3);
  L33 = f_F*f_F + tmp*T(3)*T(3);

  L << L11 << L12 << L13
  	 << L21 << L22 << L23
    << L31 << L32 << L33;

  	// resolution of costheta = |L-1:N:D| / ||L-1:N|| ||D||
  L1N = L.i() * N;
  a = (L1N.t() * D).AsScalar();
  b = sqrt(L1N(1)*L1N(1) + L1N(2)*L1N(2) + L1N(3)*L1N(3));
  c = sqrt(D(1)*D(1) + D(2)*D(2) + D(3)*D(3)); //c = D.Norm1();

  	// if b or c == 0 we will have an exception here
  costheta = fabs( a / (b*c) );

  dT = factor * (L * D + costheta * c * N);

  for(i=1; i<=3; i++) {
  	fCurrentState.Dsigma[i] = dT(i);
  }

  return 0;  // no error
}

// should be called InverseCoreModel
int QWolffersdorffExtendedModel::HFindStrainIncrement(void){

  int i, n = 3;
  Matrix L(n,n);
  ColumnVector dT(n), T(n), N(n), A(n), B(n), D(n), L1N(n);
  Real a, b, c, x1, x2, x;
  Real L11, L12, L13, L21, L22, L23, L31, L32, L33, trT, trTtrT;
  Real N1, N2, N3;
  Real factor, tmp;
  Real diskr;
  Real costheta;

  	// components of T
  T << fCurrentState.sigma[1] << fCurrentState.sigma[2]
  	<< fCurrentState.sigma[3];

  factor = f_fb * f_fe * pow(T(1) + T(2) + T(3), 2.0) / (T(1)*T(1) + T(2)*T(2) + T(3)*T(3));

  	// components of dT
  dT << fCurrentState.Dsigma[1]/factor << fCurrentState.Dsigma[2]/factor
  	<< fCurrentState.Dsigma[3]/factor;

  	// components of N
  trT = Trace3(fCurrentState.sigma);
  trTtrT = trT * trT;
  tmp = f_F * f_fd * f_a /trT / 3.0;
  N1 = tmp * ( 5.*T(1) - T(2) - T(3) );
  N2 = tmp * ( 5.*T(2) - T(1) - T(3) );
  N3 = tmp * ( 5.*T(3) - T(2) - T(1) );
  N << N1 << N2 << N3;

  	// components of L
  tmp = f_a*f_a/trTtrT;
  L11 = f_F*f_F + tmp*T(1)*T(1);
  L12 = tmp*T(1)*T(2);
  L13 = tmp*T(1)*T(3);

  L21 = tmp*T(1)*T(2);
  L22 = f_F*f_F + tmp*T(2)*T(2);
  L23 = tmp*T(2)*T(3);

  L31 = tmp*T(1)*T(3);
  L32 = tmp*T(2)*T(3);
  L33 = f_F*f_F + tmp*T(3)*T(3);

  L << L11 << L12 << L13
  	 << L21 << L22 << L23
    << L31 << L32 << L33;

    HMaterialState 		state1, state2;
    double conver;
    memset(&state1, 0, sizeof(state1));
    memset(&state2, 0, sizeof(state1));
    /* first, get a first estimation of D1 by the resolution of dT = LD + N||D||,
    then get the next estimation of D2 with the resolution of dT = LD + costheta N||D||,
    because we need the D1 to estimate the costheta. Next, we try to get the
    correct response of D when the error of D1-D2 < 1E-5. 1E-6 is too strict
    */
  	// resolution of dT = LD + N||D||, x = ||D||==> a1 x*x + bx + c = 0
  A = L.i() * dT;
  B = L.i() * N;  // B = L.i() * N * costheta;
  a = (B.t() * B).AsScalar() - 1.0;
  b = (A.t() * B).AsScalar();
  c = (A.t() * A).AsScalar();

  diskr = pow(b/a,2.0) - c/a;
  if ( diskr < 0.0 ) {
		return IDS_ER_NO_REAL_SOLUTION;
  }
  x1 = b/a + sqrt( diskr );
  x2 = b/a - sqrt( diskr );

  if ( x1 < 0. && x2 < 0. ) {
		return IDS_ER_NEGATIVE_SOLUTION;
  } else if ( x1 > 0. && x2 > 0. ) {
		return IDS_ER_NOT_UNIQUE_SOLUTION;
  } else {
  	if ( x1 > 0. )
			x = x1;
    else
			x = x2;
	}
  	// finally, get the strain rate
  for ( i=1; i<=n; i++ ) {
  	state1.Depsilon[i] = A(i) - B(i) * x;
	}

  do {
  	// components of D
  D << state1.Depsilon[1] << state1.Depsilon[2]
  	 << state1.Depsilon[3];
  	// resolution of costheta = |L-1:N:D| / ||L-1:N|| ||D||
  L1N = L.i() * N;
  a = (L1N.t() * D).AsScalar();
  b = sqrt(L1N(1)*L1N(1) + L1N(2)*L1N(2) + L1N(3)*L1N(3));
  c = sqrt(D(1)*D(1) + D(2)*D(2) + D(3)*D(3)); //c = D.Norm1();

  costheta = fabs( a / (b*c) );

  	// resolution of dT = LD + costheta N||D||, x = ||D||==> a1 x*x + bx + c = 0
  A = L.i() * dT;
  B = L.i() * N * costheta;
  a = (B.t() * B).AsScalar() - 1.0;
  b = (A.t() * B).AsScalar();
  c = (A.t() * A).AsScalar();

  diskr = pow(b/a,2.0) - c/a;
  if ( diskr < 0.0 ) {
		return IDS_ER_NO_REAL_SOLUTION;
  }
  x1 = b/a + sqrt( diskr );
  x2 = b/a - sqrt( diskr );

  if ( x1 < 0. && x2 < 0. ) {
		return IDS_ER_NEGATIVE_SOLUTION;
  } else if ( x1 > 0. && x2 > 0. ) {
		return IDS_ER_NOT_UNIQUE_SOLUTION;
  } else {
  	if ( x1 > 0. )
			x = x1;
    else
			x = x2;
	}
   for ( i=1; i<=n; i++ ) {
  		state2.Depsilon[i] = A(i) - B(i) * x;
	}
   conver = state2.Depsilon[1] - state1.Depsilon[1];
   for ( i=1; i<=n; i++ ) {
  		state1.Depsilon[i] = state2.Depsilon[i];
	}
   } while ( fabs(conver) >= 0.00001);

  	// finally, get the strain rate
  for ( i=1; i<=n; i++ ) {
  	fCurrentState.Depsilon[i] = A(i) - B(i) * x;
	}

  return 0; // no error
}




/* model of Wei Wu's thesis
   - Ci == -106.5, -801.5, -797.5, 1077.7 dense Karlsruhe medium sand taken in
   W. Wu, A. Niemulis. "Beyond failure of granular materials",
  Int J for Numerical and Analytical Methods in Geomechanics
  1997, vol 21, n3, pp 153 - 174. Section 2
*/
/*
	Quick and dirty class. It should derive from QHypoplasticModel and not
   from QWolffersdorffModel. We use 4 parameters of QWolffersdorffModel,
   see HInitSimulation(), because we do not have specific user interface
*/
void QWeiWuModel::HInitValues(){
  		// default values of Hostun RF sand
   QWolffersdorffModel::HInitValues();
	fC1 = -106.5;
	fC2 = -801.5;
	fC3 = -797.1;
	fC4 = 1077.7;
   	/* uncomment the next line to find material constants locally for
      	strain response envelope */
   //HFindCoefC();
}

void QWeiWuModel::HInitSimulation(){
   QWolffersdorffModel::HInitSimulation();
   	/* quick and dirty method to change the value of material constants
      	with a breakpoint
      */
   short ControlKeyDown = HiUint16(::GetKeyState(VK_SHIFT));
   if ( ControlKeyDown ) {
		fC1 = defphi;
		fC2 = defhs;
		fC3 = defn;
		fC4 = defalpha;
   }
}

int QWeiWuModel::HSolve(void)
{
	int i, n = 3;
  Matrix L(n,n);
  ColumnVector dT(n), T(n), D(n), N(n);
  Real c;
  Real L11, L12, L13, L21, L22, L23, L31, L32, L33, trT;
  Real N1, N2, N3;

  	// components of T
  T << fCurrentState.sigma[1] << fCurrentState.sigma[2]
  	<< fCurrentState.sigma[3];

  	// components of D
  D << fCurrentState.Depsilon[1] << fCurrentState.Depsilon[2]
  	 << fCurrentState.Depsilon[3];

  	// components of N
  trT = Trace3(fCurrentState.sigma);
  N1 = (fC3 * T(1) * T(1) + fC4 * pow((2.*T(1) - T(2) - T(3)),2.0)/9.) / trT;
  N2 = (fC3 * T(2) * T(2) + fC4 * pow((2.*T(2) - T(1) - T(3)),2.0)/9.) / trT;
  N3 = (fC3 * T(3) * T(3) + fC4 * pow((2.*T(3) - T(1) - T(2)),2.0)/9.) / trT;
  N << N1 << N2 << N3;
  //N << 0.0 << 0.0 << 0.0; //Linear part only

  	// components of L
  L11 = fC1*trT + fC2*T(1)*T(1)/trT;
  L12 = fC2*T(1)*T(2)/trT;
  L13 = fC2*T(1)*T(3)/trT;

  L21 = fC2*T(2)*T(1)/trT;
  L22 = fC1*trT + fC2*T(2)*T(2)/trT;;
  L23 = fC2*T(2)*T(3)/trT;

  L31 = fC2*T(3)*T(1)/trT;
  L32 = fC2*T(3)*T(2)/trT;
  L33 = fC1*trT + fC2*T(3)*T(3)/trT;

  L << L11 << L12 << L13
  	 << L21 << L22 << L23
    << L31 << L32 << L33;

  c = sqrt(D(1)*D(1) + D(2)*D(2) + D(3)*D(3)); //c = D.Norm1();

  dT = L * D + N * c;
  for(i=1; i<=3; i++) {
  	fCurrentState.Dsigma[i] = dT(i);
  }

  return 0;  // no error
}

int QWeiWuModel::HFindStrainIncrement(void){

  int i, n = 3;
  Matrix L(n,n);
  ColumnVector dT(n), T(n), N(n), A(n), B(n);
  Real a, b, c, x1, x2, x;
  Real L11, L12, L13, L21, L22, L23, L31, L32, L33, trT;
  Real N1, N2, N3;
  Real diskr;

  	// components of T
  T << fCurrentState.sigma[1] << fCurrentState.sigma[2]
  	 << fCurrentState.sigma[3];

  	// components of dT
  dT << fCurrentState.Dsigma[1] << fCurrentState.Dsigma[2]
     << fCurrentState.Dsigma[3];

  	// components of N
  trT = Trace3(fCurrentState.sigma);
  N1 = (fC3 * T(1) * T(1) + fC4 * pow((2.*T(1) - T(2) - T(3)),2.0)/9.) / trT;
  N2 = (fC3 * T(2) * T(2) + fC4 * pow((2.*T(2) - T(1) - T(3)),2.0)/9.) / trT;
  N3 = (fC3 * T(3) * T(3) + fC4 * pow((2.*T(3) - T(1) - T(2)),2.0)/9.) / trT;
  N << N1 << N2 << N3;
  //N << 0.0 << 0.0 << 0.0;  //Linear part only

  	// components of L
  L11 = fC1*trT + fC2*T(1)*T(1)/trT;
  L12 = fC2*T(1)*T(2)/trT;
  L13 = fC2*T(1)*T(3)/trT;

  L21 = fC2*T(2)*T(1)/trT;
  L22 = fC1*trT + fC2*T(2)*T(2)/trT;;
  L23 = fC2*T(2)*T(3)/trT;

  L31 = fC2*T(3)*T(1)/trT;
  L32 = fC2*T(3)*T(2)/trT;
  L33 = fC1*trT + fC2*T(3)*T(3)/trT;

  L << L11 << L12 << L13
  	 << L21 << L22 << L23
    << L31 << L32 << L33;

  	// resolution of dT = LD + N||D||, x = ||D||==> a1 x*x + bx + c = 0
  /* linear part only
  A = L.i() * dT;
  for ( i=1; i<=n; i++ ) {
  	fCurrentState.Depsilon[i] = A(i);
  }

  return 0; */
  A = L.i() * dT;
  B = L.i() * N;
  a = (B.t() * B).AsScalar() - 1.0;
  b = (A.t() * B).AsScalar();
  c = (A.t() * A).AsScalar();

  diskr = pow(b/a,2.0) - c/a;
  if ( diskr < 0.0 ) {
		return IDS_ER_NO_REAL_SOLUTION;
  }
  x1 = b/a + sqrt( diskr );
  x2 = b/a - sqrt( diskr );

  if ( x1 < 0. && x2 < 0. ) {
		return IDS_ER_NEGATIVE_SOLUTION;
  } else if ( x1 > 0. && x2 > 0. ) {
		return IDS_ER_NOT_UNIQUE_SOLUTION;
  } else {
  	if ( x1 > 0. )
			x = x1;
    else
			x = x2;
	}
  	// finally, get the strain rate
  for ( i=1; i<=n; i++ ) {
  	fCurrentState.Depsilon[i] = A(i) - B(i) * x;
  }

  return 0; // no error
}

/*  with experimental points of Hostun dense sand
*/
void QWeiWuModel::HFindCoefC(void){
	double T[4], dT1[4], D1[4], dT2[4], D2[4], C[5];
   int sigma1 = 200;
   int inverseProblem = 0;
   	// make a breakpoint and change these values (sigma1, inverseProblem)
   if ( sigma1 == 100 ) {
   	T[0] = 0.0; T[1] = -0.1001; T[2] = T[3] = -0.1;   		// MPa
   	dT1[0] = 0.0; dT1[1] = -0.01; dT1[2] = dT1[3] = 0.0; 	// probe of 10 kPa compression
   	dT2[0] = 0.0; dT2[1] = 0.01; dT2[2] = dT2[3] = 0.0; 	// 10 kPa extension
      if ( !inverseProblem ){
   		D1[0] = 0.0; D1[1] = -0.00014772; D1[2] = D1[3] = 0.00002604; // natural, not %
   		D2[0] = 0.0; D2[1] = 0.00029000; D2[2] = D2[3] = -0.00013524; // experimental
      } else {
   		D1[0] = 0.0; D1[1] = -0.000312453; D1[2] = D1[3] = 0.00005364; // inverse problem
   		D2[0] = 0.0; D2[1] = 0.000176391; D2[2] = D2[3] = -0.00136598; // theoretical result
      }
   } else if ( sigma1 == 200 ) {
   	T[0] = 0.0; T[1] = -0.200; T[2] = T[3] = -0.1;   // MPa
   	dT1[0] = 0.0; dT1[1] = -0.01; dT1[2] = dT1[3] = 0.0; // probe of 10 kPa compression
   	dT2[0] = 0.0; dT2[1] = 0.01; dT2[2] = dT2[3] = 0.0; // 10 kPa extension
      if ( !inverseProblem ){
   		D1[0] = 0.0; D1[1] = -0.0004636; D1[2] = D1[3] = 0.00018730;
   		D2[0] = 0.0; D2[1] = 0.000065683; D2[2] = D2[3] = -0.000017600;
      } else {
   		D1[0] = 0.0; D1[1] = -0.000343978; D1[2] = D1[3] = 0.0000914644;
   		D2[0] = 0.0; D2[1] = 0.000062122; D2[2] = D2[3] = -0.0000492224;
      }
      /*double angle = 125.0;
      dT1[0] = 0.0; dT1[1] = -0.01*sin(Deg2Rad(angle)); dT1[2] = dT1[3] = -0.01*cos(Deg2Rad(angle)); // probe of 10 kPa compression
      angle = 115.0;
      D1[0] = 0.0; D1[1] = -0.000618*sin(Deg2Rad(angle)); D1[2] = D1[3] = -0.000618*cos(Deg2Rad(angle));
      angle = 305.0;
   	dT2[0] = 0.0; dT2[1] = -0.01*sin(Deg2Rad(angle)); dT2[2] = dT2[3] = -0.01*cos(Deg2Rad(angle)); // 10 kPa extension
      angle = -32.0;
      D2[0] = 0.0; D2[1] = -0.000045*sin(Deg2Rad(angle)); D2[2] = D2[3] = -0.000045*cos(Deg2Rad(angle)); */
   } else if ( sigma1 == 400 ) {
   	T[0] = 0.0; T[1] = -0.400; T[2] = T[3] = -0.1;   // MPa
   	dT1[0] = 0.0; dT1[1] = -0.01; dT1[2] = dT1[3] = 0.0; // probe of 10 kPa compression
   	dT2[0] = 0.0; dT2[1] = 0.01; dT2[2] = dT2[3] = 0.0; // 10 kPa extension
      if ( !inverseProblem ){
   		D1[0] = 0.0; D1[1] = -0.00245578; D1[2] = D1[3] = 0.00198865;
   		D2[0] = 0.0; D2[1] = 0.00005995; D2[2] = D2[3] = -0.000019468;
      } else {
   		D1[0] = 0.0; D1[1] = -0.000687265; D1[2] = D1[3] = 0.000453784;
   		D2[0] = 0.0; D2[1] = 0.0000252023; D2[2] = D2[3] = -0.0000132405;
      }
   }

   	// make a breakpoint here and examine the coefficient C
   memset(&C, 0, sizeof(C));
   HFindCoefCi(T, dT1, D1, dT2, D2, C);
}

void QWeiWuModel::HFindCoefCi(double *T, double *dT1, double *D1, double *dT2, double *D2, double *C)
{
Real M11, M12, M13, M14, M21, M22, M23, M24;
Real M31, M32, M33, M34, M41, M42, M43, M44, trT, TD, c;

  trT = T[1] + T[2] + T[3];

  	// components of M
   // first probe test dT1, D1
  c = sqrt(D1[1]*D1[1] + D1[2]*D1[2] + D1[3]*D1[3]); //c = D.Norm1();
  TD = T[1]*D1[1] + T[2]*D1[2] + T[3]*D1[3];
  M11 = trT*D1[1];
  M12 = TD*T[1]/trT;
  M13 = c*T[1]*T[1]/trT;
  M14 = c*pow(2.*T[1]-T[2]-T[3], 2.0)/9.0/trT;

  M21 = trT*D1[2];
  M22 = TD*T[2]/trT;
  M23 = c*T[2]*T[2]/trT;
  M24 = c*pow(2.*T[2]-T[1]-T[3], 2.0)/9.0/trT;

   // second probe test dT2, D2
  c = sqrt(D2[1]*D2[1] + D2[2]*D2[2] + D2[3]*D2[3]); //c = D.Norm1();
  TD = T[1]*D2[1] + T[2]*D2[2] + T[3]*D2[3];
  M31 = trT*D2[1];
  M32 = TD*T[1]/trT;
  M33 = c*T[1]*T[1]/trT;
  M34 = c*pow(2.*T[1]-T[2]-T[3], 2.0)/9.0/trT;

  M41 = trT*D2[2];
  M42 = TD*T[2]/trT;
  M43 = c*T[2]*T[2]/trT;
  M44 = c*pow(2.*T[2]-T[1]-T[3], 2.0)/9.0/trT;

  if ( M44 == 0.0 ) {
  	int i, n = 3;
  	Matrix M(n,n);
  	ColumnVector dT(n), X(n);
  	dT << dT1[1] << dT1[2]
     << dT2[1];
  	M << M11 << M12 << M13
  	 	<< M21 << M22 << M23
    	<< M31 << M32 << M33 ;

  	X = M.i() * dT;

  	for ( i = 1; i <= 3; i++ ) {
  		C[i] = X(i);
  	}
  } else {
  int i, n = 4;
  Matrix M(n,n);
  ColumnVector dT(n), X(n);
  dT << dT1[1] << dT1[2]
     << dT2[1] << dT2[2];
  M << M11 << M12 << M13 << M14
  	 << M21 << M22 << M23 << M24
    << M31 << M32 << M33 << M34
    << M41 << M42 << M43 << M44;

  CroutMatrix IM = M;
  X = IM.i() * dT;
  /* attempt to solve an ill-conditioned matrix M
  DiagonalMatrix D;
  Matrix U, V;
  SVD(M, D, U, V);
  ColumnVector B = D.i()*U.t()*dT;
  CroutMatrix CV = V.t();
  X = CV.i()*B;  */

  for ( i = 1; i <= 4; i++ ) {
  	C[i] = X(i);
  }
  }
}


/* generalized hypoelastic model of Kolymbas
   - Ci == 66.265, 44.578, 149.676, -49.892 dense Karlsruhe medium sand taken in
   W. Wu, D. Kolymbas. "Numerical testing of the stability criterion for
   hypoplastic constitutives equations",
  Mechanics of Materials
  1990, vol 9, n?, pp 245 - 253.

*/
void QKolymbasModel::HInitValues(){
  	// default values of Hostun RF sand
   QWolffersdorffModel::HInitValues();
   fC1 = -66.265;fC2 = -133.080;fC3 = 201029;fC4 = -67033;//100 kPa envelope
   //fC1 = -167.574;fC2 = -167.478;fC3 = -629.374;fC4 = 127.735;//200 kPa envelope
   //fC1 = -115.675;fC2 = -107.386;fC3 = -246.728;fC4 = 68.132;//400 kPa envelope
	//fC1 = -66.265;fC2 = -44.578;fC3 = -149.676;fC4 = +49.892; // testing
   HFindCoefC();
}

int QKolymbasModel::HSolve(void)
{
	int i, n = 3;
  Matrix L(n,n);
  ColumnVector dT(n), T(n), D(n), N(n);
  Real c;
  Real L11, L12, L13, L21, L22, L23, L31, L32, L33, trT;
  Real N1, N2, N3;

  	// components of T
  T << fCurrentState.sigma[1] << fCurrentState.sigma[2]
  	<< fCurrentState.sigma[3];

  	// components of D
  D << fCurrentState.Depsilon[1] << fCurrentState.Depsilon[2]
  	 << fCurrentState.Depsilon[3];

  	// components of N
  trT = Trace3(fCurrentState.sigma);
  N1 = (fC3 * T(1) + fC4 * trT);
  N2 = (fC3 * T(2) + fC4 * trT);
  N3 = (fC3 * T(3) + fC4 * trT);
  N << N1 << N2 << N3;
  //N << 0.0 << 0.0 << 0.0; //Linear part only

  	// components of L
  L11 = fC1*T(1) + fC2*trT;
  L12 = fC2*T(2);
  L13 = fC2*T(3);

  L21 = fC1*T(1);
  L22 = fC1*T(2) + fC2*trT;
  L23 = fC1*T(3);

  L31 = fC1*T(1);
  L32 = fC1*T(2);
  L33 = fC1*T(3) + fC2*trT;

  L << L11 << L12 << L13
  	 << L21 << L22 << L23
    << L31 << L32 << L33;

  c = sqrt(D(1)*D(1) + D(2)*D(2) + D(3)*D(3)); //c = D.Norm1();

  dT = L * D + N * c;
  for(i=1; i<=3; i++) {
  	fCurrentState.Dsigma[i] = dT(i);
  }

  return 0;  // no error
}

int QKolymbasModel::HFindStrainIncrement(void){

  int i, n = 3;
  Matrix L(n,n);
  ColumnVector dT(n), T(n), N(n), A(n), B(n);
  Real a, b, c, x1, x2, x;
  Real L11, L12, L13, L21, L22, L23, L31, L32, L33, trT;
  Real N1, N2, N3;
  Real diskr;

  	// components of T
  T << fCurrentState.sigma[1] << fCurrentState.sigma[2]
  	 << fCurrentState.sigma[3];

  	// components of dT
  dT << fCurrentState.Dsigma[1] << fCurrentState.Dsigma[2]
     << fCurrentState.Dsigma[3];

  	// components of N
  trT = Trace3(fCurrentState.sigma);
  N1 = (fC3 * T(1) + fC4 * trT);
  N2 = (fC3 * T(2) + fC4 * trT);
  N3 = (fC3 * T(3) + fC4 * trT);
  N << N1 << N2 << N3;
  //N << 0.0 << 0.0 << 0.0;  //Linear part only

  	// components of L
  L11 = fC1*T(1) + fC2*trT;
  L12 = fC2*T(2);
  L13 = fC2*T(3);

  L21 = fC1*T(1);
  L22 = fC1*T(2) + fC2*trT;
  L23 = fC1*T(3);

  L31 = fC1*T(1);
  L32 = fC1*T(2);
  L33 = fC1*T(3) + fC2*trT;

  L << L11 << L12 << L13
  	 << L21 << L22 << L23
    << L31 << L32 << L33;

  	// resolution of dT = LD + N||D||, x = ||D||==> a1 x*x + bx + c = 0
   // uncomment these lines to get only the linear part
  /*A = L.i() * dT;
  for ( i=1; i<=n; i++ ) {
  	fCurrentState.Depsilon[i] = A(i);
  }
  return 0; */

  A = L.i() * dT;
  B = L.i() * N;
  a = (B.t() * B).AsScalar() - 1.0;
  b = (A.t() * B).AsScalar();
  c = (A.t() * A).AsScalar();

  diskr = pow(b/a,2.0) - c/a;
  if ( diskr < 0.0 ) {
		return IDS_ER_NO_REAL_SOLUTION;
  }
  x1 = b/a + sqrt( diskr );
  x2 = b/a - sqrt( diskr );

  if ( x1 < 0. && x2 < 0. ) {
		return IDS_ER_NEGATIVE_SOLUTION;
  } else if ( x1 > 0. && x2 > 0. ) {
		return IDS_ER_NOT_UNIQUE_SOLUTION;
  } else {
  	if ( x1 > 0. )
			x = x1;
    else
			x = x2;
	}
  	// finally, get the strain rate
  for ( i=1; i<=n; i++ ) {
  	fCurrentState.Depsilon[i] = A(i) - B(i) * x;
  }

  return 0; // no error
}

void QKolymbasModel::HFindCoefCi(double *T, double *dT1, double *D1, double *dT2, double *D2, double *C){
  //int i, n = 4;
  //Matrix M(n,n);
  //ColumnVector dT(n), X(n);
  Real M11, M12, M13, M14, M21, M22, M23, M24;
  Real M31, M32, M33, M34, M41, M42, M43, M44, trT, TD, c;

  	// components of dT
  //dT << dT1[1] << dT1[2]
  //   << dT2[1] << dT1[2];

  trT = T[1] + T[2] + T[3];

  	// components of M
   // first probe test dT1, D1
  c = sqrt(D1[1]*D1[1] + D1[2]*D1[2] + D1[3]*D1[3]); //c = D.Norm1();
  TD = T[1]*D1[1] + T[2]*D1[2] + T[3]*D1[3];
  M11 = TD;
  M12 = trT*D1[1];
  M13 = c*T[1];
  M14 = c*trT;

  M21 = TD;
  M22 = trT*D1[2];
  M23 = c*T[2];
  M24 = c*trT;

   // second probe test dT2, D2
  c = sqrt(D2[1]*D2[1] + D2[2]*D2[2] + D2[3]*D2[3]); //c = D.Norm1();
  TD = T[1]*D2[1] + T[2]*D2[2] + T[3]*D2[3];
  M31 = TD;
  M32 = trT*D2[1];
  M33 = c*T[1];
  M34 = c*trT;

  M41 = TD;
  M42 = trT*D2[2];
  M43 = c*T[2];
  M44 = c*trT;

  if ( M44 == 0.0 ) {
  	int i, n = 3;
  	Matrix M(n,n);
  	ColumnVector dT(n), X(n);
  	dT << dT1[1] << dT1[2]
     << dT2[1];
  	M << M11 << M12 << M13
  	 	<< M21 << M22 << M23
    	<< M31 << M32 << M33 ;

  	X = M.i() * dT;

  	for ( i = 1; i <= 3; i++ ) {
  		C[i] = X(i);
  	}
  } else {
  int i, n = 4;
  Matrix M(n,n);
  ColumnVector dT(n), X(n);
  dT << dT1[1] << dT1[2]
     << dT2[1] << dT2[2];
  M << M11 << M12 << M13 << M14
  	 << M21 << M22 << M23 << M24
    << M31 << M32 << M33 << M34
    << M41 << M42 << M43 << M44;

  CroutMatrix IM = M;
  X = IM.i() * dT;

  for ( i = 1; i <= 4; i++ ) {
  	C[i] = X(i);
  }
  }
}



/* locally determined material constants of QWolffersdorff


*/
void QWolffersdorff4ParamModel::HInitValues(){
  	// default values of Hostun RF sand
   QWolffersdorffModel::HInitValues();
   fC1 = 13.309;fC2 = 97.880;fC3 = 6.988;fC4 = -20105;//100 kPa envelope
   //fC1 = 20.096;fC2 = 80.431;fC3 = 17.137;fC4 = 73.574;//200 kPa envelope
   //fC1 = 15.814;fC2 = 102.208;fC3 = 18.887;fC4 = 42.754;//400 kPa envelope
	//fC1 = 53.9286;fC2 = 214.4828;fC3 = 45.6989;fC4 = 196.1977; // testing
   HFindCoefC();
}

int QWolffersdorff4ParamModel::HSolve(void)
{
	int i, n = 3;
  Matrix L(n,n);
  ColumnVector dT(n), T(n), D(n), N(n);
  Real c;
  Real L11, L12, L13, L21, L22, L23, L31, L32, L33, trT;
  Real N1, N2, N3;
  Real factor, tmp;
  HHypoResults currentResults;

  	// components of T
  T << fCurrentState.sigma[1] << fCurrentState.sigma[2]
  	<< fCurrentState.sigma[3];

  	// components of D
  D << fCurrentState.Depsilon[1] << fCurrentState.Depsilon[2]
  	 << fCurrentState.Depsilon[3];

    // 1 / tr(ThatThat)
  factor = pow(T(1) + T(2) + T(3), 2.0) / (T(1)*T(1) + T(2)*T(2) + T(3)*T(3));

  	// components of N
  trT = Trace3(fCurrentState.sigma);
  tmp = pow(T(1) + T(2) + T(3), 2.0);
  N1 = fC3*T(1)/trT + fC4*( T(1)/trT - 1./3. );
  N2 = fC3*T(2)/trT + fC4*( T(2)/trT - 1./3. );
  N3 = fC3*T(3)/trT + fC4*( T(3)/trT - 1./3. );
  N << N1 << N2 << N3;

  	// components of L
  L11 = fC1 + fC2*T(1)*T(1)/tmp;
  L12 = fC2*T(1)*T(2)/tmp;
  L13 = fC3*T(1)*T(3)/tmp;

  L21 = fC2*T(2)*T(1)/tmp;
  L22 = fC1 + fC2*T(2)*T(2)/tmp;
  L23 = fC2*T(2)*T(3)/tmp;

  L31 = fC2*T(3)*T(1)/tmp;
  L32 = fC2*T(3)*T(1)/tmp;
  L33 = fC1 + fC2*T(3)*T(3)/tmp;

  L << L11 << L12 << L13
  	 << L21 << L22 << L23
    << L31 << L32 << L33;

  c = sqrt(D(1)*D(1) + D(2)*D(2) + D(3)*D(3)); //c = D.Norm1();

  dT = factor * (L * D + c * N);
  for(i=1; i<=3; i++) {
  	fCurrentState.Dsigma[i] = dT(i);
  }

  return 0;  // no error
}

int QWolffersdorff4ParamModel::HFindStrainIncrement(void){

  int i, n = 3;
  Matrix L(n,n);
  ColumnVector dT(n), T(n), N(n), A(n), B(n);
  Real a, b, c, x1, x2, x;
  Real L11, L12, L13, L21, L22, L23, L31, L32, L33, trT, tmp;
  Real N1, N2, N3;
  Real diskr, factor;

  	// components of T
  T << fCurrentState.sigma[1] << fCurrentState.sigma[2]
  	 << fCurrentState.sigma[3];

      // 1 / tr(ThatThat)
  factor = pow(T(1) + T(2) + T(3), 2.0) / (T(1)*T(1) + T(2)*T(2) + T(3)*T(3));

  	// components of dT
  dT << fCurrentState.Dsigma[1]/factor << fCurrentState.Dsigma[2]/factor
     << fCurrentState.Dsigma[3]/factor;

  	// components of N
  trT = Trace3(fCurrentState.sigma);
  tmp = pow(T(1) + T(2) + T(3), 2.0);
  N1 = fC3*T(1)/trT + fC4*( T(1)/trT - 1./3. );
  N2 = fC3*T(2)/trT + fC4*( T(2)/trT - 1./3. );
  N3 = fC3*T(3)/trT + fC4*( T(3)/trT - 1./3. );
  N << N1 << N2 << N3;

  	// components of L
  L11 = fC1 + fC2*T(1)*T(1)/tmp;
  L12 = fC2*T(1)*T(2)/tmp;
  L13 = fC3*T(1)*T(3)/tmp;

  L21 = fC2*T(2)*T(1)/tmp;
  L22 = fC1 + fC2*T(2)*T(2)/tmp;
  L23 = fC2*T(2)*T(3)/tmp;

  L31 = fC2*T(3)*T(1)/tmp;
  L32 = fC2*T(3)*T(1)/tmp;
  L33 = fC1 + fC2*T(3)*T(3)/tmp;

  L << L11 << L12 << L13
  	 << L21 << L22 << L23
    << L31 << L32 << L33;

  	// resolution of dT = LD + N||D||, x = ||D||==> a1 x*x + bx + c = 0
   // uncomment these lines to get only the linear part
  /*A = L.i() * dT;
  for ( i=1; i<=n; i++ ) {
  	fCurrentState.Depsilon[i] = A(i);
  }

  return 0;  */

  A = L.i() * dT;
  B = L.i() * N;
  a = (B.t() * B).AsScalar() - 1.0;
  b = (A.t() * B).AsScalar();
  c = (A.t() * A).AsScalar();

  diskr = pow(b/a,2.0) - c/a;
  if ( diskr < 0.0 ) {
		return IDS_ER_NO_REAL_SOLUTION;
  }
  x1 = b/a + sqrt( diskr );
  x2 = b/a - sqrt( diskr );

  if ( x1 < 0. && x2 < 0. ) {
		return IDS_ER_NEGATIVE_SOLUTION;
  } else if ( x1 > 0. && x2 > 0. ) {
		return IDS_ER_NOT_UNIQUE_SOLUTION;
  } else {
  	if ( x1 > 0. )
			x = x1;
    else
			x = x2;
	}
  	// finally, get the strain rate
  for ( i=1; i<=n; i++ ) {
  	fCurrentState.Depsilon[i] = A(i) - B(i) * x;
  }

  return 0; // no error
}

void QWolffersdorff4ParamModel::HFindCoefCi(double *T, double *dT1, double *D1, double *dT2, double *D2, double *C){
  //int i, n = 4;
  //Matrix M(n,n);
  //ColumnVector dT(n), X(n);
  Real M11, M12, M13, M14, M21, M22, M23, M24;
  Real M31, M32, M33, M34, M41, M42, M43, M44, trT, TD, c, tmp;

  	// components of dT
  //dT << dT1[1] << dT1[2]
  //   << dT2[1] << dT1[2];

  trT = T[1] + T[2] + T[3];

  	// components of M
   // first probe test dT1, D1
  c = sqrt(D1[1]*D1[1] + D1[2]*D1[2] + D1[3]*D1[3]); //c = D.Norm1();
  TD = T[1]*D1[1] + T[2]*D1[2] + T[3]*D1[3];
  tmp = pow(T[1] + T[2] + T[3], 2.0);
  M11 = D1[1];
  M12 = T[1]*TD/tmp;
  M13 = c*T[1]/trT;
  M14 = c*(T[1]/trT -1./3.);

  M21 = D1[2];
  M22 = T[2]*TD/tmp;
  M23 = c*T[2]/trT;
  M24 = c*(T[2]/trT -1./3.);

   // second probe test dT2, D2
  c = sqrt(D2[1]*D2[1] + D2[2]*D2[2] + D2[3]*D2[3]); //c = D.Norm1();
  TD = T[1]*D2[1] + T[2]*D2[2] + T[3]*D2[3];
  M31 = D2[1];
  M32 = T[1]*TD/tmp;
  M33 = c*T[1]/trT;
  M34 = c*(T[1]/trT -1./3.);

  M41 = D2[2];
  M42 = T[2]*TD/tmp;
  M43 = c*T[2]/trT;
  M44 = c*(T[2]/trT -1./3.);

  if ( M44 == 0.0 ) {
  	int i, n = 3;
  	Matrix M(n,n);
  	ColumnVector dT(n), X(n);
  	dT << dT1[1] << dT1[2]
     << dT2[1];
  	M << M11 << M12 << M13
  	 	<< M21 << M22 << M23
    	<< M31 << M32 << M33 ;

  	X = M.i() * dT;

  	for ( i = 1; i <= 3; i++ ) {
  		C[i] = X(i);
  	}
  } else {
  int i, n = 4;
  Matrix M(n,n);
  ColumnVector dT(n), X(n);
      // 1 / tr(ThatThat)
  Real factor = pow(T[1] + T[2] + T[3], 2.0) / (T[1]*T[1] + T[2]*T[2] + T[3]*T[3]);
  dT << dT1[1]/factor << dT1[2]/factor
     << dT2[1]/factor << dT2[2]/factor;

  M << M11 << M12 << M13 << M14
  	 << M21 << M22 << M23 << M24
    << M31 << M32 << M33 << M34
    << M41 << M42 << M43 << M44;

  CroutMatrix IM = M;
  X = IM.i() * dT;

  for ( i = 1; i <= 4; i++ ) {
  	C[i] = X(i);
  }
  }
}


/********************* EOF ************************************************/
